//
//
// Author: yvind Natvik 1998.
//
//
//
//
// 

#include "stdafx.h"
#include <aygshell.h>  // New in PocketPC version
#include <commctrl.h>

//#define STRICT
#define FILTER_ORDER 4
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "resource.h"
#include "seislog.h"
#include "channel.h"
#include "library.h"
#include "msglist.h"
#include "ringbuffer.h"
#include "devices.h"
#include "process.h"
#include "eventlog.h"
#include "filter.h"
#include "monitor.h"

#define WINCE

//
// Theese are global variables used by functions in this file
//
extern HINSTANCE hInst;
extern HWND hWndFrame;
extern DWORD MainThreadID;
extern DWORD ProcKeeperThreadID;
extern DEVICE_INFO Devices[];
extern CHANNEL_INFO Channels[];
extern CHAR NumRegisteredDevices;
extern BYTE NumRegisteredChannels;
extern TCHAR RngBufDir[];
extern TCHAR ChannelDir[];
extern TCHAR NetworkName[];
extern BYTE DebugLevel;

//
// Definitions of structures used only by functions in this file..
//
struct CHANNEL_CNF_FILE    // Structure describing the contents of the 'channel.cnf' file.
{ 
   TCHAR StationCode[5+1];  // Station code.
   TCHAR Component[4+1];    // Component.
   BOOL ApplyFilter;       // TRUE if bandpass filtering is requested.
   FLOAT FilterLow;        // Bandpass-filter parameter (Low cutoff freq.).
   FLOAT FilterHigh;       // Bandpass-filter parameter (High cutoff freq.).
   BOOL UseTrigging;       // TRUE if channel is going to trig events.
   FLOAT ShortTermAverage; // Trigger parameter;
   FLOAT LongTermAverage;  // Trigger parameter;
   FLOAT TriggerRatio;     // Trigger parameter;
   FLOAT DetriggerRatio;   // Trigger parameter;
   BOOL LogToDisk;         // TRUE if channel should log data to disk (ringbuffer).
   BOOL LogFiltered;       // TRUE if filtered data is to be logged to disk (othervise unfiltered).
   TCHAR DeviceName[65];    // Name of the configured device the channel uses.
   BYTE DeviceChannel;     // The physical channel used on the device.
   BYTE BuffSizeInMinutes; // Size of memory-buffer measured in minutes.
};
 

struct STARTINFO_CHANNEL // Structure used to pass startup-info to a channel thread.
{
   BYTE LogicalChNr;                // The logical channel nr. for the thread.
   TCHAR ChannelDir[MAX_PATH+1];     // Home directory for this channel.
   TCHAR RingBufferDir[MAX_PATH+1];  // Ringbuffer directory for this channel.
   DWORD DevMngrThreadID;           // ThreadID of device-manager thread.
   HANDLE StartEvent;               // Handle to an event object. The channel manager
                                    // thread must signal this event when properly started.  
};


//
// Prototype definitions of functions defined and used only in this file..
//
BOOL CALLBACK AddChannelDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK ModifyChannelDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL RemoveChannelDirectories(PTCHAR);
BYTE FindFirstFreeChannel(VOID);
HANDLE StartChannelThread(STARTINFO_CHANNEL*, DWORD*);
VOID MoveChannelDown(BYTE);
VOID MoveChannelUp(BYTE);
BYTE GetChannelIndexByNumber(BYTE);
BYTE GetChnNumberByChnName(PTCHAR);
DOUBLE CalculateBlockAboluteAverage(DOUBLE*, WORD);
BOOL IsValidCompCode(PTCHAR);


//
// Prototype definition for thread routines started from this file.
//
DWORD WINAPI RingBufferThrd(PVOID);
DWORD WINAPI MonitorThrd(PVOID);


HWND hAddChannelDlg;  // [PocketPC] Handle to the property page
HWND hModifyChannelDlg;  // [PocketPC] Handle to the property page



/**************************************************************************/
/* Channel thread. Thread handles one channel.                            */
/**************************************************************************/
DWORD WINAPI ChannelThrd(PVOID StartupInfo) 
{
   MSG msg;
   DWORD CurrentThreadID, WaitStatus, BlockNr, Index, LastBlockUpdated;
   DWORD RngBuffThreadID, BuffSizeInBytes, NumBlocksInBuffer, StartIndex;
   DWORD NumBlocksBeforeTrigstart, NumBlocksDCCalculation, FirstBlock, LastBlock;
   WORD Sample;
   HANDLE hEvent, hThread;
   MSG_PARAM RP, IP, MsgParams;
   STARTINFO_CHANNEL SI;
   STARTINFO_RINGBUFFER SIRB;
   DEVMNGR_INTERF_STRUCT DS;
   CHANNEL_INFO_EX *pChInfo;
   CHANNEL_STATUS *pChStat;
   BLOCK_HEADER *pBlockHeader, *pDMBlockHeader;
   CHANNEL_CNF_FILE ChannelConf;
   SYSTEMTIME SearchTime, TimeTagPrevBlock;
   SUBSCRIBE SubscrData;
   BLOCK_RANGE *pBlockRange;
   BUFF_PROP BufferProperties;
   BYTE *pBlock, *pDMBlock, *pBlockSamples;
   VOID *pMemBuffer, *pDBBuffer;
   TCHAR str6[7], str10[11], UniqueProcName[65];
   TCHAR ConfigFile[MAX_PATH+1], Message[200];
   char CmpResult;
   DOUBLE *DblPtr, LTA, STA, NSTA, NLTA;
   DOUBLE Average, Average2, SamplesPrSec, dblValue, ChannelOffset;
   LONG *LngPtr, PrevBlock;
   MsgList NotifyList;
   BOOL MemAllocOK, ChannelRunning, ConfigRead, Trigger;
   BOOL OldTrigger, Finished, BufferFull, Found;

   // Variables used by the filter routines.
   INT nsects;
   DOUBLE sn[FILTER_ORDER*3+1], sd[FILTER_ORDER*3+1], ts, x1[5], x2[5], y1[5], y2[5];



   //
   // Initialize some variables.
   //
   CurrentThreadID=GetCurrentThreadId();  // Get and save the Thread-ID for this thread.
   RngBuffThreadID=0;
   ChannelRunning=FALSE;                  // The channel operation has not been started yet. 
	
	//
	// Theese are initialized just to get rid of a compiler warning.
	// (Variable 'xxx' may be used without having been initialized)
	//
	ChannelOffset=0.0;
	nsects=PrevBlock=0;
	NumBlocksInBuffer=0;
	Average=Average2=0.0;
	NSTA=NLTA=LTA=STA=0.0;
	NumBlocksDCCalculation=0;
	NumBlocksBeforeTrigstart=0;
	BlockNr=LastBlockUpdated=0;
	pMemBuffer=pDBBuffer=(PVOID)NULL;
	MemAllocOK=Trigger=BufferFull=FALSE;
	ZeroMemory((PVOID)&IP, sizeof(MSG_PARAM));
	ZeroMemory((PVOID)&TimeTagPrevBlock, sizeof(SYSTEMTIME));
	
      
   
   //
   // Copy thread's startup information to a local buffer (SI).
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(STARTINFO_CHANNEL));


   //
   // Create a unique name for this instance of the channel process. 
   //
   wcscpy(UniqueProcName, TEXT("Channel"));
   swprintf(str6, TEXT("%02u"), SI.LogicalChNr);
   wcscat(UniqueProcName, str6);


   //
   // Register this process in the process database.
   // Also register a dependency with the device manager process.
   //
   RegisterProcess(UniqueProcName, 5, CurrentThreadID);
   RegisterDependency(CurrentThreadID, SI.DevMngrThreadID);


   //
   // Copy device-manager's interface structure to local buffer (DS).
   // 
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);                        // Create an event object.
   PostThreadMessage(SI.DevMngrThreadID, TM_GET_INTERFACE, (WPARAM)hEvent, (LPARAM)&MsgParams);
   WaitStatus=WaitForSingleObject(hEvent, INFINITE);                    // Wait for the DM thread to process the messsage.
   CopyMemory((PVOID)&DS, MsgParams.vPointer, sizeof(DEVMNGR_INTERF_STRUCT)); // Copy interface structure to local buffer.
   CloseHandle(hEvent);                                                 // Delete the event object.
   

   //
   // Read channel's configuration from the 'Channel.cnf'
   // file located in this channel's home directory.
   //
   AddPathAndFilename(ConfigFile, SI.ChannelDir, TEXT("Channel.cnf"));
   if (ReadFileToMemory(ConfigFile, (PVOID)&ChannelConf, (DWORD)sizeof(CHANNEL_CNF_FILE))) {
      // Config file was succesfully read.
      ConfigRead=TRUE;
   } else {
      // Config file is missing or corrupt.
      // The channel can not operate without the information in the configfile.
      ConfigRead=FALSE;

      // Write message in the program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Configuration file is missing or corrupt. Channel operation can not start."));
      WriteSystemLog(Message, 3);
   }


   if (ConfigRead) {
      //
      // Calculate the size of the memory-ringbuffer.
      //
      NumBlocksInBuffer=(DWORD)(ChannelConf.BuffSizeInMinutes*(60000/DS.BlockSizeInMS));
      BuffSizeInBytes=NumBlocksInBuffer*DS.BlockSize;
   

      //
      // Allocate memory.
      //
      // Allocate the memory-ringbuffer.
      MemAllocOK=TRUE;
      pMemBuffer=(PVOID)LocalAlloc(LPTR, (UINT)BuffSizeInBytes);
      if (pMemBuffer==NULL) MemAllocOK=FALSE;

      // Allocate a block of doubles big enough to hold one block of samples.
      pDBBuffer=(PVOID)LocalAlloc(LPTR, (UINT)(sizeof(double)*DS.SamplesPrBlock));
      if (pDBBuffer==NULL) MemAllocOK=FALSE;

      // Check if memory allocation succseeded.
      if (!MemAllocOK) {
         // Write error message in the program log.
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Memory allocation failed. Channel operation can not start."));
         WriteSystemLog(Message, 3);
      }

      //
      // If data is going to be logged to disk, start the ringbuffer thread.
      //
      if (MemAllocOK && ChannelConf.LogToDisk) {
         SIRB.hWndFrame=hWndFrame;
         SIRB.hInst=hInst;
         SIRB.ProcKeeprThreadID=ProcKeeperThreadID;
         SIRB.ChnMngrThreadID=CurrentThreadID;
         wcscpy(SIRB.RingBufferDir, SI.RingBufferDir);
         wcscpy(SIRB.ConfigFileDir, SI.ChannelDir);
         SIRB.ChannelNr=ChannelConf.DeviceChannel;
         SIRB.LogicalChNr=SI.LogicalChNr;
         wcscpy(SIRB.NetworkName, NetworkName);
         wcscpy(SIRB.StationCode, ChannelConf.StationCode);
         wcscpy(SIRB.Component, ChannelConf.Component);
         SIRB.BlockSizeInMS=DS.BlockSizeInMS;
         SIRB.SamplesPrBlock=DS.SamplesPrBlock;
         SIRB.BlockSize=DS.BlockSize;
         SIRB.SampleRate=DS.SampleRate;
         SIRB.pMemBuffer=pMemBuffer;
         SIRB.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);   // Create an event object.
         hThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)RingBufferThrd, (PVOID)&SIRB, 0, &RngBuffThreadID);
         WaitForSingleObject(SIRB.StartEvent, INFINITE);          // Wait for the thread to start.
         CloseHandle(SIRB.StartEvent);                            // Delete the event object.
			CloseHandle(hThread);
      }

      //
      // If filtering is enabled, calculate filter coefficients.
      //
      if (MemAllocOK && ChannelConf.ApplyFilter) {
         // Calculate time sample interval in seconds.
         ts=(double)((DS.BlockSizeInMS/1000.0)/DS.SamplesPrBlock);

         // Calculate the filter coefficients.
         nsects=0;
         ZeroMemory((PVOID)sn, (DWORD)sizeof(sn));
         ZeroMemory((PVOID)sd, (DWORD)sizeof(sn));
         design(FILTER_ORDER, (double)ChannelConf.FilterLow, 
               (double)ChannelConf.FilterHigh, ts, sn, sd, &nsects);

         // Clear array's used by the apply function.
         ZeroMemory((PVOID)x1, (DWORD)sizeof(x1));
         ZeroMemory((PVOID)x2, (DWORD)sizeof(x2));
         ZeroMemory((PVOID)y1, (DWORD)sizeof(y1));
         ZeroMemory((PVOID)y2, (DWORD)sizeof(y2));
      }

      //
      // If trigging is enabled, calculate some parameters.
      //
      if (MemAllocOK && ChannelConf.UseTrigging) {
         // Calculate how many blocks we have to wait
         // before starting trig detection  (5 seconds).
         NumBlocksBeforeTrigstart=(WORD)(5000.0/DS.BlockSizeInMS);
         if (!NumBlocksBeforeTrigstart) NumBlocksBeforeTrigstart=1;

         // Calculate how many blocks we need to process for 
         // calculating a decent DC (offset) of the device.
         // Approximatly 500 samples will be processed.
         NumBlocksDCCalculation=(WORD)(1000/DS.SamplesPrBlock)+1;
         NumBlocksDCCalculation=min(NumBlocksDCCalculation, NumBlocksInBuffer);

         // Calculate number of samples in short term average.
         SamplesPrSec=(DOUBLE)(DS.SamplesPrBlock/(DS.BlockSizeInMS/1000.0));
         NSTA=(DOUBLE)(ChannelConf.ShortTermAverage*SamplesPrSec);
         NLTA=(DOUBLE)(ChannelConf.LongTermAverage*SamplesPrSec);

         // Set status of trigger flags.
         Trigger=FALSE;
         OldTrigger=FALSE;
      }
   }
   

   //
   // If running in debug mode, write a startup message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }


   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   // 
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {
      //
      // If debuglevel is 2 then log all messages recieved.
      //
      if (DebugLevel>1) {
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }
      

      //
      // If message is accompanied by a MSG_PARAM structure, then 
      // copy this structure to a local buffer (IP). Message is 
      // assumed to be accompanied by a structure if lParam is 
      // not 0. The 'TM_BUFFERUPDATED' message is an exception.
      //
      if (msg.message!=TM_BUFFERUPDATED) {
         if (msg.lParam) IP=*((MSG_PARAM*)msg.lParam);
      }
      

      //
      // Initialize return parameters (RP) to default values.
      //
      ZeroMemory((PVOID)&RP, (DWORD)sizeof(MSG_PARAM));
      RP.idThread=CurrentThreadID;
      RP.bStatus=TRUE; 

      
      //
      // Process the incoming message.
      //
      switch (msg.message) {
         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         // Add window/thread to notify list.
         if (!NotifyList.Subscribe((SUBSCRIBE*)IP.vPointer)) {
            RP.bStatus=FALSE;
            RP.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         // Remove notification subscribtion from list.
         NotifyList.UnSubscribe((SUBSCRIBE*)IP.vPointer);
         break;   

         

         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start the channel operation.
         //
         if (!ChannelRunning && MemAllocOK && ConfigRead) {

            // Update internal status flag.
            ChannelRunning=TRUE;

            // Initialize som variables.
            BlockNr=0;              // Start at the beginning of the buffer
            LastBlockUpdated=0;     // No blocks have been updated.
            BufferFull=FALSE;       // The buffer is empty.
            Average=Average2=0.0;
            LTA=STA=ChannelOffset=0.0;

            // Start the ringbuffer operation.
            if (ChannelConf.LogToDisk) {
               hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
               PostThreadMessage(RngBuffThreadID, TM_START, (WPARAM)hEvent, (LPARAM)&MsgParams); 
               WaitForSingleObject(hEvent, INFINITE);
               CloseHandle(hEvent);
            }

            // Subscribe to the TM_BUFFERUPDATED message. This will
            // cause the device manager to send this message every
            // time a new block of data has placed into the DM buffer.
            SubscrData.uiMessage=TM_BUFFERUPDATED;
            SubscrData.idThread=CurrentThreadID;
            SubscrData.hWnd=(HWND)NULL;
            MsgParams.vPointer=(PVOID)&SubscrData;
            hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
            PostThreadMessage(SI.DevMngrThreadID, TM_SUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
            WaitForSingleObject(hEvent, INFINITE);
            CloseHandle(hEvent);
         }
         break;

         

         case TM_STOP:
         //
         // This message is sendt from a higher level
         // thread to stop the operation of this channel.
         //
         if (ChannelRunning) {
            // Update internal status flag.
            ChannelRunning=FALSE;

            // Unsubscribe to the TM_BUFFERUPDATED message.
            SubscrData.uiMessage=TM_BUFFERUPDATED;
            SubscrData.idThread=CurrentThreadID;
            SubscrData.hWnd=(HWND)NULL;
            MsgParams.vPointer=(PVOID)&SubscrData;
            hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
            PostThreadMessage(SI.DevMngrThreadID, TM_UNSUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
            WaitForSingleObject(hEvent, INFINITE);
            CloseHandle(hEvent);

            // Stop the ringbuffer operation.
            if (ChannelConf.LogToDisk) {
               hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
               PostThreadMessage(RngBuffThreadID, TM_STOP, (WPARAM)hEvent, (LPARAM)&MsgParams); 
               WaitForSingleObject(hEvent, INFINITE);
               CloseHandle(hEvent);
            }
         }
         break;
         
         
         case TM_BUFFERUPDATED:
         //
         // The devicemanager indicates that a new block of data has arrived.
         //
         if (ChannelRunning) {
            // Create pointers to the DM block's header.
            pDMBlock=(PBYTE)GetPointerToBlock(&DS, ChannelConf.DeviceChannel, (DWORD)msg.lParam);
            pDMBlockHeader=(BLOCK_HEADER*)pDMBlock;

            // Create pointers to the local block's header and samples.
            pBlock=(PBYTE)pMemBuffer+(BlockNr*DS.BlockSize);
            pBlockHeader=(BLOCK_HEADER*)pBlock;
            pBlockSamples=pBlock+sizeof(BLOCK_HEADER);

            // Copy DM block to local memory-ringbuffer.
            CopyMemory((PVOID)pBlock, (PVOID)pDMBlock, DS.BlockSize);

            // Handle DC calculation and nulldata block.
            if (!pBlockHeader->NullData) {
               // Do offset (DC) calculation.
               if (BlockNr<NumBlocksDCCalculation && !BufferFull) {
                  // This is an initial DC calculation.
                  Average2=Average2+CalculateBlockAverage((long*)pBlockSamples, DS.SamplesPrBlock);
                  ChannelOffset=Average2/(BlockNr+1);
               } else {
                  // The channel DC might change over time so we must
                  // therefore continue to slowly adjust the offset
                  // after the initial DC calculation are finished.
                  Average2=CalculateBlockAverage((long*)pBlockSamples, DS.SamplesPrBlock);
                  ChannelOffset=ChannelOffset+(Average2-ChannelOffset)/50.0;
               }
            } else {
               // This block contains only zeros. Replace them 
               // with the currently calculated offset value.
               LngPtr=(LONG*)pBlockSamples;
               for (Index=1;Index<=DS.SamplesPrBlock;Index++) {
                  *LngPtr=(LONG)ChannelOffset;
                  LngPtr++;
               }
            }

            // Handle filtering and trigging as requested.
            if (ChannelConf.ApplyFilter || ChannelConf.UseTrigging) {
               // Filtering or trigging is enabled. Copy incoming
               // samples into the allocated array of doubles.
               DblPtr=(DOUBLE*)pDBBuffer;
               LngPtr=(LONG*)pBlockSamples;
               for (Index=1;Index<=DS.SamplesPrBlock;Index++) {
                  *DblPtr=(DOUBLE)*LngPtr;
                  DblPtr++;
                  LngPtr++;
               }

               // Filter incoming block if requested and replace the unfiltered
               // samples with the filtered samples in the memory ring-buffer.
               if (ChannelConf.ApplyFilter) {
                  // Filter samples.
                  DblPtr=((DOUBLE*)pDBBuffer);
                  apply(DblPtr, (INT)DS.SamplesPrBlock, nsects, sn, sd, x1, x2, y1, y2);

                  // Then replace the unfiltered samples with filtered samples.
                  if (ChannelConf.LogFiltered) {
                     DblPtr=(DOUBLE*)pDBBuffer;
                     LngPtr=(LONG*)pBlockSamples;
                     for (Index=1;Index<=DS.SamplesPrBlock;Index++) {
                        *LngPtr=(LONG)*DblPtr;
                        // Round values up or down to nearest whole number.
                        if (*DblPtr>0.0) if (((*DblPtr)-(DOUBLE)(*LngPtr)) > 0.5) (*LngPtr)++;
                        if (*DblPtr<0.0) if (((DOUBLE)(*LngPtr)-(*DblPtr)) > 0.5) (*LngPtr)--;
                        DblPtr++;
                        LngPtr++;
                     }
                  }
               } else {
                  // Filtering has not been applied.
                  // Remove DC offset from the DOUBLE buffer.
                  DblPtr=(DOUBLE*)pDBBuffer;
                  for (Index=1;Index<=DS.SamplesPrBlock;Index++) {
                     *DblPtr=*DblPtr-ChannelOffset;
                     DblPtr++;
                  }
               }

               // Do trigging if enabled.
               if (ChannelConf.UseTrigging) {
                  if (BlockNr<NumBlocksBeforeTrigstart && !BufferFull) {
                     // Calculate Initial STA and LTA.
                     Average=Average+CalculateBlockAboluteAverage((DOUBLE*)pDBBuffer, DS.SamplesPrBlock);
                     STA=LTA=Average/(BlockNr+1);
                  } else {
                     // Calculate new trigger status.
                     Sample=0;
                     Finished=FALSE;
                     OldTrigger=Trigger;
                     DblPtr=(DOUBLE*)pDBBuffer;
                     while (Sample<DS.SamplesPrBlock && !Finished) {
                        dblValue=fabs(*DblPtr);
                        STA=STA+((dblValue-STA)/NSTA);
                        if (!Trigger) {
                           LTA=LTA+((STA-LTA)/NLTA);
                           if (STA/LTA > ChannelConf.TriggerRatio) Trigger=Finished=TRUE;
                        } else {
                           LTA=LTA+((STA-LTA)/NLTA*10.0);
                        }
                        if (Trigger && !Finished && STA/LTA<ChannelConf.DetriggerRatio) {
                           Trigger=FALSE;
                           Finished=TRUE;
                        }
                        Sample++;
                        DblPtr++;
                     }
                     // Handle the status of the trigger flag.
                     if (Trigger || OldTrigger) pBlockHeader->EventDetected=TRUE;
                     if (Trigger && !OldTrigger) {
                        // Handle the EventTime value if needed.
                        pBlockHeader->EventTime=(WORD)(Sample*(FLOAT)(DS.BlockSizeInMS/DS.SamplesPrBlock));
                        // Inform other processes that might have subscribed to the TM_TRIG_START message.
                        if (Trigger && !OldTrigger) NotifyList.Notify((UINT)TM_TRIG_START, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);  
                     }
                     // Inform other processes that might have subscribed to the TM_TRIG_STOP message.
                     if (!Trigger && OldTrigger) NotifyList.Notify((UINT)TM_TRIG_STOP, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
                  }
               }

            }

            // Inform ringbuffer thread that channel-buffer is updated.
            if (ChannelConf.LogToDisk) PostThreadMessage(RngBuffThreadID, TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr); 

            // Inform other processes that might have subscribed to the TM_BUFFERUPDATED message.
            NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

            // Update 'last block updated' number.
            LastBlockUpdated=BlockNr;

            // Increase block nr for channel-buffer.
            BlockNr++;
            if (BlockNr==NumBlocksInBuffer) {
               BlockNr=0;
               BufferFull=TRUE;
            }
         }

         // Avoid returning status values.
         msg.lParam=msg.wParam=NULL;
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the device-manager interface structure.
         //
         RP.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_CHANNEL_INFO:
         //
         // This message is used to get information about this channel thread.
         // The information is returned in a CHANNEL_INFO_EX structure pointed
         // to by the incoming 'IP.vPointer' pointer.
         //
         pChInfo=(CHANNEL_INFO_EX*)IP.vPointer;
         pChInfo->DevMngrThreadID=SI.DevMngrThreadID;
         wcscpy(pChInfo->ChannelName, UniqueProcName);
         pChInfo->LogicalChNr=SI.LogicalChNr;
         wcscpy(pChInfo->StationCode, ChannelConf.StationCode);
         wcscpy(pChInfo->Component, ChannelConf.Component);
         pChInfo->UseTrigging=ChannelConf.UseTrigging;
         pChInfo->ShortTermAverage=ChannelConf.ShortTermAverage;
         pChInfo->LongTermAverage=ChannelConf.LongTermAverage;
         pChInfo->ApplyFilter=ChannelConf.ApplyFilter;
         pChInfo->FilterLow=ChannelConf.FilterLow;
         pChInfo->FilterHigh=ChannelConf.FilterHigh;
         pChInfo->LogToDisk=ChannelConf.LogToDisk;
		 wcscpy(pChInfo->DeviceName, ChannelConf.DeviceName);
		 pChInfo->DeviceChannel=ChannelConf.DeviceChannel;
         pChInfo->DS=(PVOID)&DS;
         break;   


         case TM_GET_CHANNEL_STATUS:
         //
         // This message is used to get information about this channel thread.
         // The information is returned in a CHANNEL_STATUS structure pointed
         // to by the incoming 'IP.vPointer' pointer.
         //
         pChStat=(CHANNEL_STATUS*)IP.vPointer;
         pChStat->Running=ChannelRunning;
         pChStat->LastBlockUpdated=LastBlockUpdated;
         pChStat->LTA=LTA;
         pChStat->STA=STA;
         pChStat->ChannelOffset=(LONG)DblRound(ChannelOffset);
         break;   



         case TM_GET_BUFFER:
         //
         // This message is used to get a pointer
         // the channel's buffer.
         //
         RP.vPointer=(PVOID)pMemBuffer;
         break;   


         case TM_GET_BUFFER_PROPERTIES:
         //
         // This message is used to get a set
         // of properties of the channelbuffer.
         //
         BufferProperties.pBuffer=pMemBuffer;
         BufferProperties.NumBlocksInBuffer=NumBlocksInBuffer;
         BufferProperties.BlocksizeInBytes=DS.BlockSize;
         BufferProperties.SamplesPrBlock=DS.SamplesPrBlock;
         BufferProperties.BlockSizeInMS=DS.BlockSizeInMS;
         BufferProperties.SampleRate=DS.SampleRate;
         CopyMemory(IP.vPointer, (PVOID)&BufferProperties, (DWORD)sizeof(BUFF_PROP));
         break;   



         case TM_GET_NUMOFBLOCKS:
         //
         // This message is used to get the number
         // of blocks in the memory buffer.
         //
         RP.dwStatus=NumBlocksInBuffer;
         break;



         case TM_GET_CURRENT_BLOCK:
         //
         // This message is used to get the        
         // the number of the last updated block.
         //

         // If the channel is not running, or buffer is empty then return FALSE.               
         if (!ChannelRunning || !NumBlocksInBuffer) { RP.bStatus=FALSE; break; }

         // Return the block number.
         RP.dwStatus=LastBlockUpdated;
         break;   



         case TM_GET_BLOCKNR:
         //
         // This message is used by another thread to get
         // the block-number of a block in the ringbuffer         
         // wich covers a certain time. The time is found
         // in a SYSTEMTIME structure pointed to by the
         // incoming vPointer member.
         //

         // If the channel is not running, or buffer is empty then return FALSE.               
         if (!ChannelRunning || !NumBlocksInBuffer) { RP.bStatus=FALSE; break; }
            
         // Copy the time to search for into a local buffer.
         CopyMemory((PVOID)&SearchTime, IP.vPointer, sizeof(SYSTEMTIME));
         SearchTime.wMilliseconds=456;

         // Find the oldest and the newest block in the buffer.
         if (BufferFull) {
            FirstBlock=LastBlockUpdated+1;
            if (FirstBlock==NumBlocksInBuffer) FirstBlock=0;
            LastBlock=LastBlockUpdated;
         } else {
            FirstBlock=0;
            LastBlock=LastBlockUpdated;
         }

         // Scan trough the blocks to find the block we are searching for.
         // We will search until we find the first block with a newer start
         // time than the search time. The block before this block will 
         // normally be the one we are searching for.
         Index=FirstBlock;
         Found=Finished=FALSE;
         while (!Found && !Finished) {
            pDMBlock=(PBYTE)pMemBuffer+(Index*DS.BlockSize);
            pDMBlockHeader=(BLOCK_HEADER*)pDMBlock;
            CmpResult=TimeCmp(&pDMBlockHeader->TimeTag, &SearchTime);
            if (CmpResult==0) {
               // We have found the block we are searching for.
               Found=Finished=TRUE;
               RP.dwStatus=Index;
            }
            if (CmpResult==1) {
               // We have found a block with a newer time than 
               // the search time. Hopefully the block right 
               // before this one, is the one we are searching for.
               Finished=TRUE;
               if (Index!=FirstBlock && !pDMBlockHeader->DataLost) {
                  Found=TRUE;
                  RP.dwStatus=PrevBlock; 
               }
            }
            if (Index==LastBlock) {
               // If we have searched trough the whole buffer then stop,
               Finished=TRUE;
            } else {
               // else save this block number and jump to the next block.
               PrevBlock=Index; Index++; 
               if (Index==NumBlocksInBuffer) Index=0;
            }
         }
         // Return FALSE if block was not found
         if (!Found) RP.bStatus=FALSE;
         break;   



         case TM_GET_BLOCK_RANGE:
         //
         // This message is used by another thread to get
         // a block range covering as close as possible the
         // the time window given in the input structure.
         // The block-range returned is guarantied to be
         // continues (no timeholes). The input and output
         // parameters are located in the incoming structure   
         // of type 'BLOCK_RANGE'. The times given in the input
         // structure will be overwritten with the actual times
         // corresponding to the returned block numbers. The
         // StartTime returned will be equal to the timetag
         // of the first block. The end time returned will be
         // equal to the startime of the block that follows
         // the last block. 
         //

         // If the channel is not running, or buffer is empty then return FALSE.               
         if (!ChannelRunning || !NumBlocksInBuffer) { RP.bStatus=FALSE; break; }
            
         // Create a pointer to the message parameter structure.
         pBlockRange=(BLOCK_RANGE*)IP.vPointer;

         // Find the oldest and the newest block in the buffer.
         if (BufferFull) {
            FirstBlock=LastBlockUpdated+1;
            if (FirstBlock==NumBlocksInBuffer) FirstBlock=0;
            LastBlock=LastBlockUpdated;
         } else {
            FirstBlock=0;
            LastBlock=LastBlockUpdated;
         }
        
         // Scan trough the blocks to find the block corresponding to the
         // start time. We will search until we find the first block with 
         // a newer or equal timetag compared to the start time.
         Index=FirstBlock;
         Found=Finished=FALSE;
         SearchTime=pBlockRange->StartTime;
         while (!Found && !Finished) {
            pDMBlock=(PBYTE)pMemBuffer+(Index*DS.BlockSize);
            pDMBlockHeader=(BLOCK_HEADER*)pDMBlock;
            CmpResult=TimeCmp(&pDMBlockHeader->TimeTag, &SearchTime);
            if (CmpResult==0) {
               // We have found the block we are searching for.
               Found=Finished=TRUE;
               pBlockRange->FirstBlock=Index;
               pBlockRange->StartTime=pDMBlockHeader->TimeTag;
            }
            if (CmpResult==1) {
               // We have found a block with a newer time than 
               // the search time. The block we are searching for
               // is this one or the one before this one.
               Found=Finished=TRUE;
               if (Index!=FirstBlock && !pDMBlockHeader->DataLost) {
                  pBlockRange->FirstBlock=PrevBlock;
                  pBlockRange->StartTime=TimeTagPrevBlock;
               } else {
                  pBlockRange->FirstBlock=Index;
                  pBlockRange->StartTime=pDMBlockHeader->TimeTag;
               }
            }
            if (Index==LastBlock) {
               // If we have searched trough the whole buffer then stop,
               Finished=TRUE;
            } else {
               // else save this block number and jump to the next block.
               PrevBlock=Index; Index++; 
               TimeTagPrevBlock=pDMBlockHeader->TimeTag;
               if (Index==NumBlocksInBuffer) Index=0;
            }
         }
         // If nothing was found then all the blocks
         // in the buffer are older then the StartTime.
         // The block range we are searching for does
         // not exist in the buffer.
         if (!Found) { RP.bStatus=FALSE; break; }

         // Check if the timestamp of the block found
         // is newer then the EndTime. If so, then all
         // blocks in the buffer is newer then the end
         // time (and the start time).
         if (TimeCmp(&pBlockRange->StartTime, &pBlockRange->EndTime)==1) {
            RP.bStatus=FALSE;
            break; 
         }

         // 
         // Now we have found the first block. The timestamp
         // of this block is either eqal to, or newer, then the
         // time we searched for. The two variables'StartTime' 
         // and 'FirstBlock' has been updated in the return structure.
         // The next step is to find the last block in the block range.
         // To find it we continue searching through the buffer until
         // we find a block with a time newer then the EndTime, or until
         // we find a timejump in the buffer, or until we reach the end
         // of the buffer. 
         Finished=FALSE;
         pBlockRange->NumBlocksInRange=1;
         SearchTime=pBlockRange->EndTime;
         PrevBlock=pBlockRange->FirstBlock;
         TimeTagPrevBlock=pBlockRange->StartTime;
         StartIndex=pBlockRange->FirstBlock+1; 
         if (StartIndex==NumBlocksInBuffer) StartIndex=0;
         Index=StartIndex;
         while (!Finished) {
            pDMBlock=(PBYTE)pMemBuffer+(Index*DS.BlockSize);
            pDMBlockHeader=(BLOCK_HEADER*)pDMBlock;
            pBlockRange->NumBlocksInRange++;
            if (pDMBlockHeader->DataLost) {
               // There is a time jump between the last block
               // and this block. The block before this one is
               // the block we are searching for.
               Finished=TRUE;
               pBlockRange->LastBlock=PrevBlock;
               pBlockRange->EndTime=TimeTagPrevBlock;
               AddToTime(&pBlockRange->EndTime, (DOUBLE)(DS.BlockSizeInMS/1000.0));
               pBlockRange->NumBlocksInRange--;
            } else {
               // Compare time of current block to EndTime.
               CmpResult=TimeCmp(&pDMBlockHeader->TimeTag, &SearchTime);
               if (CmpResult>=0) {
                  // We have found a block that matches the EndTime
                  // exactly, or a block with a newer tile then EndTime.
                  // The block we are searching for is the one before this one.
                  Finished=TRUE;
                  pBlockRange->LastBlock=PrevBlock;
                  pBlockRange->EndTime=pDMBlockHeader->TimeTag;
                  pBlockRange->NumBlocksInRange--;
               }
            }
            if (!Finished) {
               if (Index==LastBlock) {
                  // If we have searched trough the whole buffer then stop.
                  // The last block in the buffer will be the end block.
                  Finished=TRUE;
                  pBlockRange->LastBlock=Index;
                  pBlockRange->EndTime=pDMBlockHeader->TimeTag;
                  AddToTime(&pBlockRange->EndTime, (DOUBLE)(DS.BlockSizeInMS/1000.0));
               } else {
                  // Save this block number and jump to the next block.
                  PrevBlock=Index; Index++; 
                  TimeTagPrevBlock=pDMBlockHeader->TimeTag;
                  if (Index==NumBlocksInBuffer) Index=0;
               }
            }
         }
         break;   






         case TM_GET_PROC_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique channel instance.
         //
         wcscpy(RP.cStatus, UniqueProcName);
         break;   


         case TM_GET_CHNUMBER:
         //
         // This message is used to get the physical
         // device channel nr. used by this channel.
         //
         RP.wStatus=(WORD)ChannelConf.DeviceChannel;
         break;


         case TM_GET_LOG_CHNUMBER:
         //
         // This message is used to get the logical
         // channel nr. used by this channel.
         //
         RP.wStatus=(WORD)SI.LogicalChNr;
         break;



         case TM_GET_STATION:
         //
         // This message is used to get a string, 
         // identifying this channel's stationcode.
         //
         wcscpy(RP.cStatus, ChannelConf.StationCode);
         break;   


         case TM_GET_COMPONENT:
         //
         // This message is used to get a string, 
         // identifying this channel's component code.
         //
         wcscpy(RP.cStatus, ChannelConf.Component);
         break;
         

         case TM_GET_BLOCKSIZE:
         //
         // This message is used to get the size (in bytes), 
         // of each block (header included) in the memory ringbuffer.
         //
         RP.wStatus=DS.BlockSize;
         break;


         case TM_GET_DEVMANAGER:
         //
         // This message is used to get the thread-id, 
         // for this channel's device manager.
         //
         RP.dwStatus=SI.DevMngrThreadID;
         break;
      

         case TM_GET_RINGBUFFER:
         //
         // This message is used to get the thread-id, 
         // for this channel's ringbuffer.
         //
         RP.dwStatus=RngBuffThreadID;
         break;


         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         if (ChannelRunning) {
            // Post TM_STOP message to stop the channel before exiting.
            PostThreadMessage(CurrentThreadID, TM_STOP, 0, 0); 

            // Repost the TM_EXIT message so that this 
            // thread will receive a new TM_EXIT message 
            // after having processed TM_STOP message.
            PostThreadMessage(CurrentThreadID, TM_EXIT, 0, 0); 
         } else {
            // Stop all threads dependant on this thread.
            StopDependantThreads();
         }
         break;

         
         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //
         // Unregister this process from the process database.
         UnregisterProcess(CurrentThreadID);

         // Release all allocated memory.
         if (ConfigRead && MemAllocOK) {
            LocalFree((HLOCAL)pMemBuffer);
            LocalFree((HLOCAL)pDBBuffer);
         }

         // End the message loop.
         ClearMessageQueue();  // Delete all messages in message queue.
         PostQuitMessage(0);   // Post WM_QUIT message to end the message loop.
         break;
      }
      
      // Return status values, and release calling process.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=RP;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Thread dies here.
   ExitThread(0);
   return 0;
   
}

/*******************************************************************/
/*   Dialog box procedure for the 'Administer channels' dialog.    */
/*******************************************************************/
BOOL CALLBACK AdministerChannelsDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
   SHINITDLGINFO shidi; // New in PocketPC version
#endif
   HWND hWndDlgItem;
   BYTE Index, ChannelNr, DevIndex;
   TCHAR str2[3], string85[86], MessageText[150], LastChannel[10]; 
   TCHAR TmpChDir[MAX_PATH+1], ChDir[MAX_PATH+1];
   TCHAR ChannelName[10], ChConfFile[MAX_PATH+1];
   TCHAR TmpRingbufDir[MAX_PATH+1], RingbufDir[MAX_PATH+1];
   INT Status;
   HANDLE hChannelThread, hEvent;   
	UNREFERENCED_PARAMETER(lParam);
   DWORD dwIndex, DeviceThreadID, ChannelThreadID, RngBufThreadID;
   CHANNEL_CNF_FILE ChannelConf;
   STARTINFO_CHANNEL SICM;
   MSG_PARAM MsgParam;
   BOOL DeviceStarted, bStatus;


   switch(message)  {
      case WM_INITDIALOG:
#ifdef WINCE
	  // Create a Done button and size it. New in PocketPC version.
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
#endif
      // Initialize the 'Channels' list box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
      for (Index=0;Index<NumRegisteredChannels;Index++) {
         wcscpy(string85, Channels[Index].ChannelName);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index);
         if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

      // Disable buttons as needed.
      if (IsSystemRunning()) {
         // If system is running, then user can not add or remove channels.
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON1), FALSE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), FALSE); 
      }
      if (!NumRegisteredChannels) {
         // No channels are defined. Disable all buttons accept the 'Add' button. 
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), FALSE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), FALSE); 
      }


      // Bring window to front.
      BringWindowToTop(hWndDlg);
      return TRUE; 

      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_BUTTON1:
         //
         // This is the 'Add' button. The user wants to add a new channel.
         //

         //
         // Check that there are room for more channels.
         //
         if (NumRegisteredChannels==MAX_NUM_OF_CHANNELS) {
            wcscpy(MessageText, TEXT("Cant add any more channels.\nThe program limit has been reached."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONHAND);
            return FALSE;
         }

         //
         // Check that there are a least one device configured.
         //
         if (NumRegisteredDevices==0) {
            wcscpy(MessageText, TEXT("There are no configured devices.\nYou must configure at least one device before adding channels."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONHAND);
            return FALSE;
         }

         //
         // Transfer control to the 'Add channel' dialog box.
         // Here the user will enter the parameters for the channel. 
         // The information is returned in the 'ChannelConf' structure.
         // The logical channel nr. for the new channel is returned
         // as the status value from the call to 'DialogBoxParam'.
         //
         // Initialize all parameters to default values.
         ZeroMemory((PVOID)&ChannelConf, (DWORD)sizeof(CHANNEL_CNF_FILE));  // Clear the return structure.
         ChannelConf.ApplyFilter=FALSE;
		 wcscpy(ChannelConf.Component, TEXT("A  E"));
         ChannelConf.FilterLow=3.0;
         ChannelConf.FilterHigh=8.0;
         ChannelConf.UseTrigging=FALSE;
         ChannelConf.ShortTermAverage=1.0;
         ChannelConf.LongTermAverage=200.0;
         ChannelConf.TriggerRatio=3.0;
         ChannelConf.DetriggerRatio=1.5;
         ChannelConf.LogToDisk=TRUE;
         ChannelConf.LogFiltered=FALSE;
         ChannelConf.BuffSizeInMinutes=5;
         //
         // Bring up the dialog box.
#ifdef WINCE         
		 ChannelNr = (BYTE) AddChannelDialogBox(hInst, hWndDlg, (LPARAM)(CHANNEL_CNF_FILE*)&ChannelConf);
#else
		 ChannelNr=(BYTE)DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_ADD_CHANNEL), hWndDlg, (DLGPROC)AddChannelDlgProc, (LPARAM)(CHANNEL_CNF_FILE*)&ChannelConf);
#endif

         
		 if (ChannelNr==0) return FALSE;  // The user has cancelled the operation.

         //
         // Create name of new channel.
         //
         wcscpy(ChannelName, TEXT("Channel")); 
         swprintf(str2, TEXT("%02u"), ChannelNr);
         wcscat(ChannelName, str2);

         //
         // Create a temporary directory for the new channel. 
         // For now the directory will be named 'NewChannel'.
         //
         wcscpy(TmpChDir, ChannelDir);
         StripBackslashFromPath(TmpChDir);
         wcscat(TmpChDir, TEXT("\\NewChannel"));
         CreateDirectory(TmpChDir, NULL);

         //
         // Create the config file (channel.cnf) for the new channel.
         // The file will be placed in the temporary directory.
         //
         wcscpy(ChConfFile, TmpChDir);
         wcscat(ChConfFile, TEXT("\\channel.cnf"));
         bStatus=WriteFileFromMemory(ChConfFile, (PVOID)&ChannelConf , (DWORD) sizeof(CHANNEL_CNF_FILE));
         if (!bStatus) {
            // For some reason we could not create the 
            // channel config file. Clean up and exit.
            ClearDirectory(TmpChDir);
            RemoveDirectory(TmpChDir);

            // Inform user.
            wcscpy(MessageText, TEXT("Channel configuration failed!\n"));
            wcscpy(MessageText, TEXT("Unable to create the channel config file."));
            MessageBox(NULL, MessageText, TEXT("Seislog message"),  MB_OK|MB_ICONERROR);
            return FALSE;
         }



         //       
         // Create the ring-buffer directory if requested.
         // If so, also create the ringbuffer.
         //
         DeviceStarted=FALSE;
         if (ChannelConf.LogToDisk) {
            // Make sure that device-manager is running.
            DeviceThreadID=GetProcessThreadIDByName(ChannelConf.DeviceName);
            if (DeviceThreadID==0) {
               // Device is not running, start it.
               DeviceStarted=TRUE;                                // Device had to be started.           
               ExistDevice(ChannelConf.DeviceName, &DevIndex);    // Get the index to the device.
               DeviceThreadID=StartDeviceManager(TRUE, DevIndex); // Start device manager in configure modus.
            }

            // Create a temporary ringbuffer directory.
            // For now the directory will be named 'NewChannel'.
            wcscpy(TmpRingbufDir, RngBufDir);
            StripBackslashFromPath(TmpRingbufDir);
            wcscat(TmpRingbufDir, TEXT("\\NewChannel"));
            CreateDirectory(TmpRingbufDir, NULL);

            // Create the ringbuffer by starting a
            // copy of the channel thread. The channel
            // thread will in turn start the ringbuffer
            // thread so that the ringbuffer thread can
            // create the ringbuffer.

            // Start channel thread.
            SICM.LogicalChNr=ChannelNr;
            wcscpy(SICM.ChannelDir, TmpChDir);
            wcscpy(SICM.RingBufferDir, TmpRingbufDir);
            SICM.DevMngrThreadID=DeviceThreadID;
            SICM.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
            hChannelThread=StartChannelThread(&SICM, &ChannelThreadID);
            CloseHandle(SICM.StartEvent);
				CloseHandle(hChannelThread);

            // Get threadID of the ringbuffer thread.
            hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
            PostThreadMessage(ChannelThreadID, TM_GET_RINGBUFFER, (WPARAM)hEvent, (LPARAM)&MsgParam); 
            WaitForSingleObject(hEvent, INFINITE);
            RngBufThreadID=MsgParam.dwStatus;
            CloseHandle(hEvent);

            // Send 'TM_CONFIGURE' message to the ring-buffer thread.
            // This will make the ring-buffer open it's configure 
            // dialogbox, thereby allowing the user to configure 
            // and create a new ringbuffer.
            hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
            PostThreadMessage(RngBufThreadID, TM_CONFIGURE, (WPARAM)hEvent, (LPARAM)&MsgParam); 
            WaitForSingleObject(hEvent, INFINITE); // Wait for the user to finish before continuing.
            CloseHandle(hEvent);

            // Stop the channel thread. This will also stop the ringbuffer thread.
            PostThreadMessage(ChannelThreadID, TM_EXIT, 0, 0); 
            while (PostThreadMessage(ChannelThreadID, TM_DUMMY, 0, 0)) Sleep(100);

            // If started above, then stop the device manager thread.
            if (DeviceStarted) {
               PostThreadMessage(DeviceThreadID, TM_EXIT, 0, 0); 
               while (PostThreadMessage(DeviceThreadID, TM_DUMMY, 0, 0)) Sleep(100);
            }

            // If user canceled the operation, or somthing
            // went wrong, then clean up and exit.
            if (!MsgParam.bStatus) {
               // Clean up and remove the channel directory.
               ClearDirectory(TmpChDir);
               RemoveDirectory(TmpChDir);

               // Clean up and remove the ring-buffer directory.
               if (ChannelConf.LogToDisk) {
                  ClearDirectory(TmpRingbufDir);
                  RemoveDirectory(TmpRingbufDir);
               }

               if (MsgParam.wStatus==150) {
                  // Creation of ringbuffer failed. Inform user.
                  wcscpy(MessageText, TEXT("Unable to create ringbuffer.\n"));
                  wcscat(MessageText, TEXT("Check your amount of free disk space."));
                  MessageBox(NULL, MessageText, TEXT("Seislog message"),  MB_OK|MB_ICONERROR);
               }
               return FALSE;  
            }
         }

         
         // The channel and ringbuffer directories
         // will now be renamed to their proper names.
         // If a channel by this name already exist, then
         // it will be moved down to the next channel nr. 
         //
         // Create full path to the channel directory
         wcscpy(ChDir, ChannelDir);
         StripBackslashFromPath(ChDir);
         wcscat(ChDir, TEXT("\\"));
         wcscat(ChDir, ChannelName);
         //
         // Check if this directory already exist.
         // If this directory already exist then move the 
         // existing, and all following directories one 
         // channel number down, therby making room for 
         // the new directory. 
         if (ExistDirectory(ChDir)) MoveChannelDown(ChannelNr);
         //
         // Rename the channel directory.
         MoveFile(TmpChDir, ChDir);
         //
         // Rename the ringbuffer directory.
         if (ChannelConf.LogToDisk) {
            wcscpy(RingbufDir, RngBufDir);
            StripBackslashFromPath(RingbufDir);
            wcscat(RingbufDir, TEXT("\\"));
            wcscat(RingbufDir, ChannelName);
            MoveFile(TmpRingbufDir, RingbufDir);
         }
         
         
         //
         // Recreate the 'Channels'table.
         //
         RegisterChannels();

         //
         // Update the configured channels list box.
         //
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         SendMessage(hWndDlgItem, LB_RESETCONTENT , 0, 0);
         for (Index=0;Index<NumRegisteredChannels;Index++) {
            wcscpy(string85, Channels[Index].ChannelName);
            dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
            SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index);
            if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
         }

         // Inform user of the name of the new channel.
         wcscpy(MessageText, TEXT("Channel was sucessfully configured!\n"));
         wcscat(MessageText, TEXT("This channel will be referred to by the system as '"));
         wcscat(MessageText, ChannelName); wcscat(MessageText, TEXT("'.\n"));
         wcscat(MessageText, TEXT("Please update your triggersets now if necessary."));
         MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);

         // Enable buttons.
         if (NumRegisteredChannels) {
            EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), TRUE); 
            EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), TRUE); 
         }
         break;
      



         case IDC_BUTTON2:
         //
         // This is the 'Remove' button. The user wants to remove a channel.
         //

         // Check that there are any channels to remove.
         if (!NumRegisteredChannels) {
            MessageBeep(MB_ICONHAND);
            return FALSE;
         }

         // Get selection from list box. (Get the channel name).
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem, LB_GETCURSEL, 0, 0);
         SendMessage(hWndDlgItem, LB_GETTEXT, (WPARAM)dwIndex, (LPARAM)(LPCTSTR)string85);

         // Force user to confirm removal of channel.
         wcscpy(MessageText, TEXT("Are you sure you want to remove "));
         wcscat(MessageText, string85);
         wcscat(MessageText, TEXT(" ?"));
         Status=MessageBox(hWndDlg, MessageText, TEXT("Seislog"), MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
         if (Status==IDYES) { 
            // Remove the channel directories.
            SetCursor(LoadCursor(NULL, IDC_WAIT));
            RemoveChannelDirectories(string85);
            SetCursor(LoadCursor(NULL, IDC_ARROW));

            // If there exist channels with higher 
            // channelnumbers then the one that was
            // just deleted, then theese must be moved
            // up to fill the void in the channel list.
            //
            // Get the name of the last channel in the channel list.
            dwIndex=(DWORD)NumRegisteredChannels-1;
            SendMessage(hWndDlgItem, LB_GETTEXT, (WPARAM)dwIndex, (LPARAM)(LPCTSTR)LastChannel);
            //
            // Check if this channel has a greater channel 
            // number then the one that was deleted.
            if (_wcsicmp(LastChannel, string85)) {
               MoveChannelUp(GetChnNumberByChnName(LastChannel));
            }

            // Recreate the 'Channels'table.
            RegisterChannels();

            // Update the 'Registered Channels' list box.
            SendMessage(hWndDlgItem, LB_RESETCONTENT, 0, 0);
            for (Index=0;Index<NumRegisteredChannels;Index++) {
               wcscpy(string85, Channels[Index].ChannelName);
               dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)string85);
               SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index);
               if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
            }

            // Inform user that the channel has been removed.
            wcscpy(MessageText, TEXT("The channel has been removed.\n"));
            wcscat(MessageText, TEXT("Update your triggersets if necessary."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);
         }

         // Disable buttons if needed.
         if (!NumRegisteredChannels) {
            EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON2), FALSE); 
            EnableWindow(GetDlgItem(hWndDlg, IDC_BUTTON3), FALSE); 
         }
         break;


         
         
         case IDC_BUTTON3:
         //
         // This is the 'Modify' button.
         // Let user change the parameters for the selected channel.
         //

         // Get selection from list box. (Get the channel name).
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem, LB_GETCURSEL, 0, 0);
         SendMessage(hWndDlgItem, LB_GETTEXT, (WPARAM)dwIndex, (LPARAM)(LPCTSTR)ChannelName);

         //
         // Read the contents of the config file into the 'ChannelConf' variable.
         //
         wcscpy(ChConfFile, ChannelDir);
         StripBackslashFromPath(ChConfFile);
         wcscat(ChConfFile, TEXT("\\"));
         wcscat(ChConfFile, ChannelName);
         wcscat(ChConfFile, TEXT("\\channel.cnf"));
         bStatus=ReadFileToMemory(ChConfFile, (PVOID)&ChannelConf, (DWORD)sizeof(CHANNEL_CNF_FILE));
         if (!bStatus) {
            // Unable to read the channel config file.
            wcscpy(MessageText, TEXT("An error occured while reading the channel config file.\n"));
            wcscat(MessageText, TEXT("The file may be missing or corrupt. Operation is aborted."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
            return FALSE;
         }

         //
         // Bring up the 'Modify channel parameters' dialog-box
         // so that user can modify the current channel parameters.
         //
#ifdef WINCE
		 Status=ModifyChannelDialogBox(hInst, hWndDlg, (LPARAM)(CHANNEL_CNF_FILE*)&ChannelConf);
		 if (Status==0) return FALSE;
#else
         Status=DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MODIFY_CHANNEL), hWndDlg, (DLGPROC)ModifyChannelDlgProc, (LPARAM)(CHANNEL_CNF_FILE*)&ChannelConf);
         if (Status==0) return FALSE;  // The user has cancelled the operation.
#endif



         //
         // Recreate the channel.cnf file.
         //
         bStatus=WriteFileFromMemory(ChConfFile, (PVOID)&ChannelConf, (DWORD)sizeof(CHANNEL_CNF_FILE));
         if (!bStatus) {
            // Unable to write the new channel config file.
            wcscpy(MessageText, TEXT("An error occurrred while writing to the channel config file.\n"));
            wcscat(MessageText, TEXT("You are strongly adviced to delete the channel and recreate it."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
            return FALSE;
         }

         //
         // Recreate the 'Channels'table.
         //
         RegisterChannels();


         //
         // If the systen is running, inform user that it is neccesarry
         // to restart the systen for changes to take effect.
         //
         if (IsSystemRunning()) {
            wcscpy(MessageText, TEXT("You must restart the system for changes to take effect."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);
         }
         break;

         
         case IDOK: //[PocketPC] This ID is changed. ID is IDC_BUTTON3 in Win9X version.
         // This is the 'Close' button.
         // Close the dialog box
         EndDialog(hWndDlg , 0);
         break;

      }
      return FALSE;
   }
   return FALSE;
}











/************************************************************************************************/
/* Dialog box procedure for the 'Add channel' dialog.                                           */
/************************************************************************************************/
BOOL CALLBACK AddChannelDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR DeviceName[65], ChannelName[65];
   TCHAR MessageText[200], str3[4], str4[5], str5[6];
   TCHAR str7[8];
   INT result;
   HWND hWndDlgItem;
   BYTE ChannelNr, Index, DevIndex;
   DWORD dwIndex;
   CHANNEL_CNF_FILE *ChParams;

   switch(message)  {
      case WM_INITDIALOG:

      // Save pointer to return structure.
      SetWindowLong(hWndDlg, GWL_USERDATA, (LONG)lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*)lParam;

      //
      // Initialize 'Channel number for new channel' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT1, _itow((int)FindFirstFreeChannel(), str3,10));

      //
      // Initialize 'station code' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->StationCode);

      //
      // Initialize 'component' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT3, ChParams->Component);


      //
      // Initialize the 'Enable filtering' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->ApplyFilter) {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE);
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
      }
      
      //
      // Initialize values for the two filter parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->FilterLow);
      SetDlgItemText(hWndDlg, IDC_EDIT4, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->FilterHigh);
      SetDlgItemText(hWndDlg, IDC_EDIT5, str7);

      //
      // Initialzie the 'Create ringbuffer' check box.
      //
      if (ChParams->LogToDisk) CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED); 

      //
      // Initialzie the 'Record data filtered' check box.
      //
      if (ChParams->LogFiltered) CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);

      //
      // Initialize the 'Size in minutes' combo box.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("1"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (ChParams->BuffSizeInMinutes==1) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("2"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (ChParams->BuffSizeInMinutes==2) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (ChParams->BuffSizeInMinutes==3) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==4) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("5"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==5) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("7"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 7);
      if (ChParams->BuffSizeInMinutes==7) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("10"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 10);
      if (ChParams->BuffSizeInMinutes==10) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("13"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 13);
      if (ChParams->BuffSizeInMinutes==13) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("16"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 16);
      if (ChParams->BuffSizeInMinutes==16) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("20"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 20);
      if (ChParams->BuffSizeInMinutes==20) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);

      //
      // Initialize the 'Enable trigging' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->UseTrigging) {
         CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
      }
      
      //
      // Initialize default values for the trigger parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->ShortTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT6, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->LongTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT7, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->TriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT8, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->DetriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT9, str7);

      //
      // Initialize the devices list box. All configured devices must be inserted.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
      for (DevIndex=0;DevIndex<NumRegisteredDevices;DevIndex++) {
         wcscpy(DeviceName, Devices[DevIndex].UniqueDeviceName);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)DeviceName);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)DevIndex);
         if (DevIndex==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

      //
      // Initialize the physical channels list box.
      // The list box will list all channels for the first device in device listbox.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
      for (Index=0;Index<Devices[0].NumChannels;Index++) {
         wcscpy(ChannelName, TEXT("Physical channel "));
         swprintf(str3, TEXT("%u"), Index+1);
         wcscat(ChannelName, str3);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)ChannelName);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index+1);
         if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

      // Set the input focus to the 'Station code' edit box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_EDIT2);
      SetFocus(hWndDlgItem);
      return FALSE; 


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_LIST1:
         switch (HIWORD(wParam)) { 
            case LBN_SELCHANGE:
            //
            // The user has made a selection in the devices list box.
            // The physical channel list box must be updated to reflect
            // the number of channels of the new device.
            //

            // Get index into 'Devices' of currently selected device.
            hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
            dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
            DevIndex=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);
            
            // Update the physical channel list box.
            hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
            SendMessage(hWndDlgItem, LB_RESETCONTENT , 0, 0);
            for (Index=0;Index<Devices[DevIndex].NumChannels;Index++) {
               wcscpy(ChannelName, TEXT("Physical channel "));
               swprintf(str3, TEXT("%u"), Index+1);
               wcscat(ChannelName, str3);
               dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)ChannelName);
               SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index+1);
               if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
            }
            break;
         }
         break;

         case IDC_CHECK1:
         // This is the 'Enable filtering' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               // Enable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), FALSE); 
               // Disable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE); 
            }
         }
         break;

         case IDC_CHECK3:
         // This is the 'Enable trigging' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), FALSE); 
            }
         }
         break;


         case IDOK:
         // 
         // This is the 'OK' button.
         //

         //
         // Get saved pointer to returndata structure.
         //
         ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, GWL_USERDATA);

         //
         // Read and check chosen channel number.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT1, str3, 3);
         ChannelNr=(BYTE)_wtoi(str3);
         if (wcslen(str3)==0 || ChannelNr==0 || ChannelNr>FindFirstFreeChannel()) { 
            // The user may only select an existing channel
            // or the next free channel in the list.
            wcscpy(MessageText, TEXT("The channel number is invalid.\n"));
            wcscat(MessageText, TEXT("Enter the the next free channel-number,\n"));
            wcscat(MessageText, TEXT("or the number of a channel that already exist."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
            SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1); 
            break;
         }
         

         //
         // Read and check the 'Station code' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT2, str5, 6);
         StrRmvSpaces(str5);
         if (!wcslen(str5)) {
            // The field is empty.
            wcscpy(MessageText, TEXT("The field is empty.\n"));
            wcscat(MessageText, TEXT("Please enter a value."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT2));
            SendDlgItemMessage(hWndDlg, IDC_EDIT2, EM_SETSEL, 0, -1); 
            break;
         }
         wcscpy(ChParams->StationCode, str5);

         //
         // Read and check the 'Component' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT3, str4, 5);
         if (!IsValidCompCode(str4)) {
            // Component code given is not valid.
            wcscpy(MessageText, TEXT("The component code entered is invalid, please correct!  "));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT3));
            SetDlgItemText(hWndDlg, IDC_EDIT3, str4);
            SendDlgItemMessage(hWndDlg, IDC_EDIT3, EM_SETSEL, 0, -1); 
            break;
         }
         wcscpy(ChParams->Component, str4);

         //
         // Read the 'Enable filtering' check box.
         // If enabled, then read and check the filter paramerters.
         //
         ChParams->ApplyFilter=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 
         if (ChParams->ApplyFilter) {
            // User has enabled filtering. Get and check filter parameters.
            GetDlgItemText(hWndDlg, IDC_EDIT4, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->FilterLow);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT5, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->FilterHigh);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
               SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
               break;
            }

            // Check values.
            if (ChParams->FilterLow >= ChParams->FilterHigh) {
               // The value for FilterHigh must be > FilterLow.
               wcscpy(MessageText, TEXT("The 'FilterHigh' value must be\n"));
               wcscat(MessageText, TEXT("greater than the 'FilterLow' value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to this field.
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               break;
            }
            if (ChParams->FilterLow<0.0 || ChParams->FilterHigh<0.0) {
               // The value for the filter-parameters must be GE 0.0.
               wcscpy(MessageText, TEXT("The filterparameters can not be negative."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to the offending field.
               if (ChParams->FilterLow<0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               } else {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
               }
               break;
            }
         }

         //
         // Read the 'Create ring-buffer' check box.
         //
         ChParams->LogToDisk=(IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_CHECKED); 

         //
         // Read the 'Record data filtered' check box.
         //
         ChParams->LogFiltered=(IsDlgButtonChecked(hWndDlg, IDC_CHECK4)==BST_CHECKED); 

         //
         // Read the 'Size in minutes' combo box.
         //
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hWndDlgItem, CB_GETCURSEL, 0, 0);
         ChParams->BuffSizeInMinutes=(BYTE)SendMessage(hWndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         //
         // Read the 'Enable trigging' check box.
         // If enabled, then read and check the trigger paramerters.
         //
         ChParams->UseTrigging=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED); 
         if (ChParams->UseTrigging) {
            // User has enabled trigging. Get and check parameters.
            GetDlgItemText(hWndDlg, IDC_EDIT6, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->ShortTermAverage);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
               SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT7, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->LongTermAverage);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
               SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT8, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->TriggerRatio);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
               SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT9, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->DetriggerRatio);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
               SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
               break;
            }

            // Check values.
            if (ChParams->ShortTermAverage <= 0.0 ||
                ChParams->LongTermAverage <= 0.0 ||
                ChParams->TriggerRatio <= 0.0 ||
                ChParams->DetriggerRatio <= 0.0 ) {

               // One or several values are less then or equal to zero.
               wcscpy(MessageText, TEXT("Value must be greater than zero, please correct."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to the offending field.
               if (ChParams->ShortTermAverage <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
               }
               if (ChParams->LongTermAverage <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
               }
               if (ChParams->TriggerRatio <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
               }
               if (ChParams->DetriggerRatio <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
               }
               break;
            }
         }

         // Get the selected device.
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
         dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
         DevIndex=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);
         wcscpy(ChParams->DeviceName, Devices[DevIndex].UniqueDeviceName);

         // Get selected physical channel.
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
         dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
         ChParams->DeviceChannel=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);

         // Close the dialog box ,return the channel nr, and exit.
         EndDialog(hWndDlg , (int)ChannelNr);
         break;
         
         case IDCANCEL:
         // This is the 'Cancel' button.
         // User wants to cancel the operation. Close the dialog box and exit.
         EndDialog(hWndDlg , 0);
         break;
      }
      return FALSE;
   }
   return FALSE;
}







/************************************************************************************************/
/* Dialog box procedure for the 'Modify channel parameters' dialog.                                            */
/************************************************************************************************/
BOOL CALLBACK ModifyChannelDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200], str4[5], str5[6];
   TCHAR str7[8];
   INT result;
   HWND hWndDlgItem;
   DWORD dwIndex;
   CHANNEL_CNF_FILE *ChParams;

   switch(message)  {
      case WM_INITDIALOG:

      // Save pointer to return structure.
      SetWindowLong(hWndDlg, GWL_USERDATA, (LONG)lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*)lParam;

      //
      // Initialize 'station code' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->StationCode);

      //
      // Initialize 'component' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT3, ChParams->Component);

      //
      // Initialize the 'Enable filtering' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->ApplyFilter) {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
      }
      
      //
      // Initialize values for the two filter parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->FilterLow);
      SetDlgItemText(hWndDlg, IDC_EDIT4, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->FilterHigh);
      SetDlgItemText(hWndDlg, IDC_EDIT5, str7);

      //
      // Initialzie the 'Record data filtered' check box.
      //
      if (ChParams->LogFiltered) CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);

      //
      // Initialize the 'Size in minutes' combo box.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("1"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (ChParams->BuffSizeInMinutes==1) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("2"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (ChParams->BuffSizeInMinutes==2) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (ChParams->BuffSizeInMinutes==3) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==4) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("5"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==5) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("7"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 7);
      if (ChParams->BuffSizeInMinutes==7) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("10"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 10);
      if (ChParams->BuffSizeInMinutes==10) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("13"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 13);
      if (ChParams->BuffSizeInMinutes==13) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("16"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 16);
      if (ChParams->BuffSizeInMinutes==16) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("20"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 20);
      if (ChParams->BuffSizeInMinutes==20) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);

      //
      // Initialize the 'Enable trigging' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->UseTrigging) {
         CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
      }
      
      //
      // Initialize default values for the trigger parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->ShortTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT6, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->LongTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT7, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->TriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT8, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->DetriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT9, str7);

      // Set the input focus to the 'Station code' edit box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_EDIT2);
      SetFocus(hWndDlgItem);
      return FALSE; 


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_CHECK1:
         // This is the 'Enable filtering' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE);
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               // Enable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE);
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), FALSE);
               // Disable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE);
            }
         }
         break;

         case IDC_CHECK3:
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), FALSE); 
            }
         }
         break;


         case IDOK:
         // 
         // This is the 'OK' button.
         //

         //
         // Get saved pointer to returndata structure.
         //
         ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, GWL_USERDATA);

         //
         // Read and check the 'Station code' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT2, str5, 6);
         if (wcslen(str5)==0) {
            // The field is empty.
            wcscpy(MessageText, TEXT("The field is empty.\n"));
            wcscat(MessageText, TEXT("Please enter a value."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT2));
            SendDlgItemMessage(hWndDlg, IDC_EDIT2, EM_SETSEL, 0, -1); 
            break;
         }
         wcscpy(ChParams->StationCode, str5);

         //
         // Read and check the 'Component' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT3, str4, 5);
         if (!IsValidCompCode(str4)) {
            // Component code given is not valid.
            wcscpy(MessageText, TEXT("The component code entered is invalid, please correct!  "));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT3));
            SetDlgItemText(hWndDlg, IDC_EDIT3, str4);
            SendDlgItemMessage(hWndDlg, IDC_EDIT3, EM_SETSEL, 0, -1); 
            break;
         }
         wcscpy(ChParams->Component, str4);

         //
         // Read the 'Enable filtering' check box.
         // If enabled, then read and check the filter paramerters.
         //
         ChParams->ApplyFilter=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 
         if (ChParams->ApplyFilter) {
            // User has enabled filtering. Get and check filter parameters.
            GetDlgItemText(hWndDlg, IDC_EDIT4, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->FilterLow);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT5, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->FilterHigh);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
               SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
               break;
            }

            // Check values.
            if (ChParams->FilterLow >= ChParams->FilterHigh) {
               // The value for FilterHigh must be > FilterLow.
               wcscpy(MessageText, TEXT("The 'FilterHigh' value must be\n"));
               wcscat(MessageText, TEXT("greater than the 'FilterLow' value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to this field.
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               break;
            }
            if (ChParams->FilterLow<0.0 || ChParams->FilterHigh<0.0) {
               // The value for the filter-parameters must be GE 0.0.
               wcscpy(MessageText, TEXT("The filterparameters can not be negative."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to the offending field.
               if (ChParams->FilterLow<0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
               } else {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
               }
               break;
            }
         }

         //
         // Read the 'Record data filtered' check box.
         //
         ChParams->LogFiltered=(IsDlgButtonChecked(hWndDlg, IDC_CHECK4)==BST_CHECKED); 

         //
         // Read the 'Size in minutes' combo box.
         //
         hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hWndDlgItem, CB_GETCURSEL, 0, 0);
         ChParams->BuffSizeInMinutes=(BYTE)SendMessage(hWndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         //
         // Read the 'Enable trigging' check box.
         // If enabled, then read and check the trigger paramerters.
         //
         ChParams->UseTrigging=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED); 
         if (ChParams->UseTrigging) {
            // User has enabled trigging. Get and check parameters.
            GetDlgItemText(hWndDlg, IDC_EDIT6, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->ShortTermAverage);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
               SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT7, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->LongTermAverage);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
               SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT8, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->TriggerRatio);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
               SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
               break;
            }
            GetDlgItemText(hWndDlg, IDC_EDIT9, str7, 8);
            result=swscanf(str7, TEXT("%f"), &ChParams->DetriggerRatio);
            if (result==0 || wcslen(str7)==0) {
               wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
               wcscat(MessageText, TEXT("Please enter a valid value."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
               SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
               break;
            }

            // Check values.
            if (ChParams->ShortTermAverage <= 0.0 ||
                ChParams->LongTermAverage <= 0.0 ||
                ChParams->TriggerRatio <= 0.0 ||
                ChParams->DetriggerRatio <= 0.0 ) {

               // One or several values are less then or equal to zero.
               wcscpy(MessageText, TEXT("Value must be greater than zero, please correct."));
               MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

               // Move the input focus to the offending field.
               if (ChParams->ShortTermAverage <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
               }
               if (ChParams->LongTermAverage <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
               }
               if (ChParams->TriggerRatio <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
               }
               if (ChParams->DetriggerRatio <= 0.0) {
                  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
                  SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
               }
               break;
            }
         }

         // Close the dialog box and exit.
         EndDialog(hWndDlg , 1);
         break;
         
         case IDCANCEL:
         // This is the 'Cancel' button.
         // User wants to cancel the operation. Close the dialog box and exit.
         EndDialog(hWndDlg , 0);
         break;
      }
      return FALSE;
   }
   return FALSE;
}




BOOL CALLBACK ModifyChannelDlgProc1(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200], str4[5], str5[6];
   HWND hWndDlgItem;
   CHANNEL_CNF_FILE *ChParams;
   BYTE Index;
   DWORD dwIndex;
   TCHAR CompCodes[47][5]={
	   TEXT("S  Z"),TEXT("S  N"),TEXT("S  E"),TEXT("L  Z"),TEXT("L  N"),
	   TEXT("L  E"),TEXT("AL Z"),TEXT("AL N"),TEXT("AL E"),TEXT("AH Z"),
	   TEXT("AH N"),TEXT("AH E"),TEXT("A  Z"),TEXT("A  N"),TEXT("A  E"),
	   TEXT("BV Z"),TEXT("BV N"),TEXT("BV E"),TEXT("BH Z"),TEXT("BH N"),
	   TEXT("BH E"),TEXT("B  Z"),TEXT("B  N"),TEXT("B  E"),TEXT("SV Z"),
	   TEXT("SV N"),TEXT("SV E"),TEXT("SH Z"),TEXT("SH N"),TEXT("SH E"),
	   TEXT("LV Z"),TEXT("LV N"),TEXT("LV E"),TEXT("S  R"),TEXT("S  T"),
       TEXT("L  R"),TEXT("L  T"),TEXT("AL R"),TEXT("AL T"),TEXT("AH R"),
	   TEXT("AH T"),TEXT("A  R"),TEXT("A  T"),TEXT("BV R"),TEXT("BV T"),
	   TEXT("SV R"),TEXT("SV T")
   };


   switch(message)  {
      case WM_INITDIALOG:

      // Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;

      //
      // Initialize 'station code' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->StationCode);

      //
      // Initialize 'component' edit field.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (Index=0;Index<47;Index++) {
         dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)CompCodes[Index]);
         if (!wcscmp(ChParams->Component, CompCodes[Index])) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Set the input focus to the 'Station code' edit box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_EDIT2);
      SetFocus(hWndDlgItem);

      return TRUE;

	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{

         //
         // Get saved pointer to returndata structure.
         //
         ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

         //
         // Read and check the 'Station code' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT2, str5, 6);
         if (wcslen(str5)==0) {
            // The field is empty.
            wcscpy(MessageText, TEXT("The field is empty.\n"));
            wcscat(MessageText, TEXT("Please enter a value."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT2));
            SendDlgItemMessage(hWndDlg, IDC_EDIT2, EM_SETSEL, 0, -1); 

			// [PocketPC] Change to this Page and Prevent closing 

			PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 0);
			SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
			return TRUE;

         }
         wcscpy(ChParams->StationCode, str5);

/*
         //
         // Read and check the 'Component' name.
         //
         GetDlgItemText(hWndDlg, IDC_EDIT3, str4, 5);
         if (!IsValidCompCode(str4)) {
            // Component code given is not valid.
            wcscpy(MessageText, TEXT("The component code entered is invalid, please correct!  "));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT3));
            SetDlgItemText(hWndDlg, IDC_EDIT3, str4);
            SendDlgItemMessage(hWndDlg, IDC_EDIT3, EM_SETSEL, 0, -1); 

			// [PocketPC] Change to this Page and Prevent closing 

			PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 0);
			SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
			return TRUE;

         }
         wcscpy(ChParams->Component, str4);
*/

         //
         // Read and check the 'Component' name.
         //
         GetDlgItemText(hWndDlg, IDC_COMBO1, str4, 5);
         wcscpy(ChParams->Component, str4);


		 // [PocketPC] OK to close the dialog box
		 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
		 return TRUE;

			 
		}
		return FALSE;
	}
	return FALSE;
}

BOOL CALLBACK ModifyChannelDlgProc2(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

   TCHAR MessageText[200];
   TCHAR str7[8];
   INT result;
   HWND hWndDlgItem;
   DWORD dwIndex;
   CHANNEL_CNF_FILE *ChParams;


	switch(message)  {

	case WM_INITDIALOG:

		// Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;


      //
      // Initialize the 'Enable filtering' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->ApplyFilter) {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
      }
      
      //
      // Initialize values for the two filter parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->FilterLow);
      SetDlgItemText(hWndDlg, IDC_EDIT4, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->FilterHigh);
      SetDlgItemText(hWndDlg, IDC_EDIT5, str7);

      //
      // Initialzie the 'Record data filtered' check box.
      //
      if (ChParams->LogFiltered) CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);

      //
      // Initialize the 'Size in minutes' combo box.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("1"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (ChParams->BuffSizeInMinutes==1) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("2"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (ChParams->BuffSizeInMinutes==2) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (ChParams->BuffSizeInMinutes==3) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==4) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("5"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==5) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("7"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 7);
      if (ChParams->BuffSizeInMinutes==7) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("10"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 10);
      if (ChParams->BuffSizeInMinutes==10) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("13"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 13);
      if (ChParams->BuffSizeInMinutes==13) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("16"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 16);
      if (ChParams->BuffSizeInMinutes==16) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("20"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 20);
      if (ChParams->BuffSizeInMinutes==20) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
	  return TRUE;
     
      case WM_COMMAND:
		  switch(LOWORD(wParam)) {
			 case IDC_CHECK1:
			 // This is the 'Enable filtering' checkbox.
			 switch (HIWORD(wParam)) { 
				case BN_CLICKED:
				if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
				   // Enable the paramater fields.
				   EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE);
				   EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
				   // Enable the 'Record filtered data' checkbox.
				   EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE);
				} else {
				   // Disable the paramater fields.
				   EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), FALSE); 
				   EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), FALSE);
				   // Disable the 'Record filtered data' checkbox.
				   EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE);
				}
			 }
		  }
		break;

	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{
		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

			 //
			 // Read the 'Enable filtering' check box.
			 // If enabled, then read and check the filter paramerters.
			 //
			 ChParams->ApplyFilter=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 
			 if (ChParams->ApplyFilter) {
				// User has enabled filtering. Get and check filter parameters.
				GetDlgItemText(hWndDlg, IDC_EDIT4, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->FilterLow);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
					// Change to this page and prevent closing
					PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 1);
					SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
					return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT5, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->FilterHigh);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 

				   // Change to this page and prevent closing
					PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 1);
					SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
					return TRUE;

				}

				// Check values.
				if (ChParams->FilterLow >= ChParams->FilterHigh) {
				   // The value for FilterHigh must be > FilterLow.
				   wcscpy(MessageText, TEXT("The 'FilterHigh' value must be\n"));
				   wcscat(MessageText, TEXT("greater than the 'FilterLow' value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to this field.
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 

				   // Change to this page and prevent closing
					PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 1);
					SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
					return TRUE;

				}
				if (ChParams->FilterLow<0.0 || ChParams->FilterHigh<0.0) {
				   // The value for the filter-parameters must be GE 0.0.
				   wcscpy(MessageText, TEXT("The filterparameters can not be negative."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to the offending field.
				   if (ChParams->FilterLow<0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
				   } else {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
				   }
   					// Change to this page and prevent closing
					PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 1);
					SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
					return TRUE;

				}
			 }

			 //
			 // Read the 'Record data filtered' check box.
			 //
			 ChParams->LogFiltered=(IsDlgButtonChecked(hWndDlg, IDC_CHECK4)==BST_CHECKED); 

			 //
			 // Read the 'Size in minutes' combo box.
			 //
			 hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
			 dwIndex=SendMessage(hWndDlgItem, CB_GETCURSEL, 0, 0);
			 ChParams->BuffSizeInMinutes=(BYTE)SendMessage(hWndDlgItem, CB_GETITEMDATA, dwIndex, 0);

   
			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;
			 
		}
		return FALSE;
	}
	return FALSE;

}


BOOL CALLBACK ModifyChannelDlgProc3(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200];
   TCHAR str7[8];
   INT result;
   CHANNEL_CNF_FILE *ChParams;


	switch(message)  {

	case WM_INITDIALOG:

		// Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;

      //
      // Initialize the 'Enable trigging' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->UseTrigging) {
         CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
      }
      
      //
      // Initialize default values for the trigger parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->ShortTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT6, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->LongTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT7, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->TriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT8, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->DetriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT9, str7);

	  return TRUE;

      case WM_COMMAND:

		 switch(LOWORD(wParam)) {
     
             case IDC_CHECK3:
         // This is the 'Enable trigging' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), FALSE); 
            }
         }
	    }
	 break;	  

	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{
		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

  			 //
			 // Read the 'Enable trigging' check box.
			 // If enabled, then read and check the trigger paramerters.
			 //
			 ChParams->UseTrigging=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED); 
			 if (ChParams->UseTrigging) {
				// User has enabled trigging. Get and check parameters.
				GetDlgItemText(hWndDlg, IDC_EDIT6, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->ShortTermAverage);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
				   
				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;
				}
				GetDlgItemText(hWndDlg, IDC_EDIT7, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->LongTermAverage);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT8, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->TriggerRatio);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT9, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->DetriggerRatio);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}

				// Check values.
				if (ChParams->ShortTermAverage <= 0.0 ||
					ChParams->LongTermAverage <= 0.0 ||
					ChParams->TriggerRatio <= 0.0 ||
					ChParams->DetriggerRatio <= 0.0 ) {

				   // One or several values are less then or equal to zero.
				   wcscpy(MessageText, TEXT("Value must be greater than zero, please correct."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to the offending field.
				   if (ChParams->ShortTermAverage <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->LongTermAverage <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->TriggerRatio <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->DetriggerRatio <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
				   }

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hModifyChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
			 }
			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

			 
		}
		return FALSE;
	}
	return FALSE;
}




int ModifyChannelDialogBox(HINSTANCE hInst, HWND hWnd, LPARAM lParam)
{
	PROPSHEETHEADER psh;    // The entire Property Sheet
	PROPSHEETPAGE   psp[4]; // The property pages
	INT i;

	// Init page structures with generic information.
	memset (&psp, 0, sizeof (psp));      // Zero out all unused values.

	for (i = 0; i < 3; i++) {
		psp[i].dwSize = sizeof (PROPSHEETPAGE);
		psp[i].dwFlags = PSP_DEFAULT | PSP_PREMATURE;
		psp[i].hInstance = hInst;         // Instance handle where the
	                                      // dialog templates are located
		psp[i].lParam  = lParam; 
	}
	// Now do the page specific stuff.
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_MODIFY_CHANNEL_1); // Name of dialog resource for page 1
	psp[0].pfnDlgProc = ModifyChannelDlgProc1;    // Pointer to dialog proc for page 1

	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_MODIFY_CHANNEL_2); // Name of dialog resource for page 2
	psp[1].pfnDlgProc = ModifyChannelDlgProc2;    // Pointer to dialog proc for page 2

	psp[2].pszTemplate = MAKEINTRESOURCE(IDD_MODIFY_CHANNEL_3); // Name of dialog resource for page 2
	psp[2].pfnDlgProc = ModifyChannelDlgProc3;    // Pointer to dialog proc for page 2

	// Init property sheet header structure.
	psh.dwSize = sizeof (PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USECALLBACK;    // We are using templates not handles.
	psh.hwndParent = hWnd;              // Handle of the owner window
	psh.hInstance = hInst;              // Instance handle of the application
	psh.pszCaption = TEXT ("Modify channel");
	psh.nPages = 3;                    // Number of pages
	psh.nStartPage = 0;                  // Index of page to be shown first
	psh.ppsp = psp;                     // Pointer to page structures
	psh.pfnCallback = ModifyChannelPropSheetProc;   

	// Create property sheet. This returns when the user dismisses the sheet
	// by tapping OK or the Close button.

	int res = PropertySheet (&psh);

	return res;
}


BOOL CALLBACK ModifyChannelPropSheetProc(HWND hwndDlg, UINT uMsg, LPARAM lParam )
{
	
	SHINITDLGINFO shidi;

	switch (uMsg)
	{
		
	case PSCB_INITIALIZED: // Note: This may be the wrong message to do this!
		 
		// Initialize handle to the Property Sheet
		hModifyChannelDlg = hwndDlg;
	
		// Create a Done button and size it.  
		shidi.dwMask = SHIDIM_FLAGS;
		shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hwndDlg;
		SHInitDialog(&shidi);
		
		return TRUE;
	}
	return TRUE;
}











/************************************************************************************************/
/* Dialog box procedure for the 'Add channel - Modify channel parameters' dialog.               */
/************************************************************************************************/
BOOL CALLBACK ModifyChParamsDlgProc(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[100];
   CHANNEL_CNF_FILE *ChParams;


   switch(message)  {
      case WM_INITDIALOG:
      //
      // Initialize the dialog box.
      //
      
      // Save pointer to return structure.
      SetWindowLong(hWndDlg, GWL_USERDATA, (LONG)lParam);

      // Create pointer to the return parameters.
      ChParams=(CHANNEL_CNF_FILE*) lParam;

      // Initialize the edit boxes.
      SetDlgItemText(hWndDlg, IDC_EDIT1, ChParams->StationCode);
      SetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->Component);

      // Set focus to the first edit box.
      SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1)); 
      return FALSE; 


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDOK:
         // 
         // This is the 'OK' button. 
         //

         // Return the values from the edit boxes.
         ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, GWL_USERDATA);
         GetDlgItemText(hWndDlg, IDC_EDIT1, ChParams->StationCode, 6);
         GetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->Component, 5);

         // Check that parameters is valid.
         if (wcslen(ChParams->StationCode)==0 || wcslen(ChParams->Component)==0) {
            wcscpy(MessageText, TEXT("Please make sure all fields are filled in."));
            MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONINFORMATION);
            return FALSE;
         }

         // Close the dialog box and exit.
         EndDialog(hWndDlg , 1);
         break;
         
         case IDCANCEL:
         // This is the 'Cancel' button.
         // User wants to cancel the operation. Close the dialog box and exit.
         EndDialog(hWndDlg , 0);
         break;
      }
      return FALSE;
   }
   return FALSE;
}













/*******************************************************************************/
/* Reads and load information from disk about all configured channels.         */
/* Save number of channels found in the global variable NumRegisteredChannels. */
/*******************************************************************************/
VOID RegisterChannels(VOID)
{
   // Browse through all the subdirectories in the 'channels' directory.
   // Register all directories that has a 'channel.cnf' file as
   // a channel, with name equal to the directory name. Directories
   // which do not have a 'channel.cnf' are considered to be invalid.

   TCHAR FileSpec[MAX_PATH+1], FileSpec2[MAX_PATH+1], Message[100];
   BYTE Index, Index2;
   HANDLE SearchHndl;
	CHANNEL_INFO ChInfo;
   CHANNEL_CNF_FILE ChannelInfo;
   WIN32_FIND_DATA FileData;

   //
   // Clear/initialize the 'Channels' array.
   //
   for (Index=0;Index<MAX_NUM_OF_CHANNELS;Index++) {
      wcscpy(Channels[Index].ChannelName, TEXT(""));
      wcscpy(Channels[Index].DeviceName, TEXT(""));
      Channels[Index].DeviceChannel=0;
   }
   NumRegisteredChannels=0;


   //
   // Register channels.
   //
   //
   // Build file-search-specification string.
   wcscpy(FileSpec, ChannelDir);
   StripBackslashFromPath(FileSpec);

   // [PocketPC] The search string has been modified because
   //   og a unknown bug on Cassiopeia (but not in emulator.
   //   The original string was 
   //   wcscat(FileSpec, TEXT("\\Channel??.*"));
   wcscat(FileSpec, TEXT("\\Channel??"));

   // Find the first directory.
   SearchHndl=FindFirstFile(FileSpec, &FileData);
   if (SearchHndl==INVALID_HANDLE_VALUE) return;
   if ((FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
      // We have found a directory.
      // Create a string containing the full path to this directory.

	  AddPathAndFilename(FileSpec, ChannelDir, FileData.cFileName);
   
      // Check if the direcory contains a 'channel.cnf' file.
      if (ExistFile(FileSpec, TEXT("channel.cnf"))) {
         // Register this channel.
         NumRegisteredChannels++;

         // Read channel information from config file.
         wcscpy(FileSpec2, FileSpec);
         wcscat(FileSpec2, TEXT("\\channel.cnf"));
         ReadFileToMemory(FileSpec2, &ChannelInfo, (DWORD)sizeof(CHANNEL_CNF_FILE));

         // Register channel name.
         wcscpy(Channels[NumRegisteredChannels-1].ChannelName, FileData.cFileName);

         // Register name of the device that the channel depends on.
         wcscpy(Channels[NumRegisteredChannels-1].DeviceName, ChannelInfo.DeviceName);

         // Get the physical device channel.
         Channels[NumRegisteredChannels-1].DeviceChannel=ChannelInfo.DeviceChannel;

         // Register trigger information.
         Channels[NumRegisteredChannels-1].UseTrigging=ChannelInfo.UseTrigging;

      } else {
         // Update program log.
         wcscpy(Message, TEXT("Seislog: No 'channel.cnf' file found in '"));
         wcscat(Message, FileSpec);
         wcscat(Message, TEXT("'. Channel will be ignored."));
         WriteSystemLog(Message, 2);
      }
   }

   // Find and process the rest of the directories.
   while (FindNextFile(SearchHndl, &FileData)) {
      if ((FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
         // We have found a directory.
         // Create a string containing the full path to this directory.
         AddPathAndFilename(FileSpec, ChannelDir, FileData.cFileName);
      
         // Check if the direcory contains a 'channel.cnf' file.
         if (ExistFile(FileSpec, TEXT("channel.cnf"))) {
            if (NumRegisteredChannels<MAX_NUM_OF_CHANNELS) {
               // Register this channel.
               NumRegisteredChannels++;

               // Read channel information from config file.
               wcscpy(FileSpec2, FileSpec);
               wcscat(FileSpec2, TEXT("\\channel.cnf"));
               ReadFileToMemory(FileSpec2, &ChannelInfo, (DWORD)sizeof(CHANNEL_CNF_FILE));

               // Register channel name.
               wcscpy(Channels[NumRegisteredChannels-1].ChannelName, FileData.cFileName);

               // Register name of the device that the channel depends on.
               wcscpy(Channels[NumRegisteredChannels-1].DeviceName, ChannelInfo.DeviceName);

               // Get the physical device channel.
               Channels[NumRegisteredChannels-1].DeviceChannel=ChannelInfo.DeviceChannel;

               // Register trigger information.
               Channels[NumRegisteredChannels-1].UseTrigging=ChannelInfo.UseTrigging;
            }
         } else {
            // This channel directory does not contain a 'channel.cnf' file.
            // Write a warning message in the program log.
            wcscpy(Message, TEXT("Seislog: No 'channel.cnf' file found in '"));
            wcscat(Message, FileSpec);
            wcscat(Message, TEXT("'. Channel will be ignored."));
            WriteSystemLog(Message, 2);
         }
      }
   }

   // Close the handle.
   FindClose(SearchHndl);


   //
   // Sort the 'Channels' array by channel name.
   //
   if (NumRegisteredChannels>=2) {
	   for (Index=0;Index<=NumRegisteredChannels-2;Index++) {
			for (Index2=(BYTE)(Index+1);Index2<=NumRegisteredChannels-1;Index2++) {
				if (_wcsicmp(Channels[Index].ChannelName, Channels[Index2].ChannelName)>0) {
					ChInfo=Channels[Index];
					Channels[Index]=Channels[Index2];
					Channels[Index2]=ChInfo;
				}
			}
	   }
   }


   return;  

}


/*******************************************************************************/
/* Start all channles registered in the Channels table.                        */
/* The channel threads are not sendt the TM_START message.                     */
/*******************************************************************************/
VOID LoadChannels(VOID)
{
   TCHAR DeviceName[65], RingBufferDir[MAX_PATH+1];
   TCHAR ChDir[MAX_PATH+1], ConfFileName[MAX_PATH+1];
   TCHAR ChannelName[10], Message[200];
   BYTE Index, DevIndex;
   DWORD DeviceThreadID, ChannelThreadID;
   STARTINFO_CHANNEL SICH;
   BOOL DeviceFound;
#ifndef WINCE
   STARTINFO_MONITOR SIMON;
   DWORD MonThreadID;
   HANDLE hThread;
   BOOL bStatus;
   MON_WND_CNF MonConfig;
#endif
   CHANNEL_CNF_FILE ChannelConfig;


   //
   // Go trough the Channels' table.
   //
   for (Index=0;Index<NumRegisteredChannels;Index++) {
      //
      // Start channel thread for this channel.
      //
      // Get the name of the device manager for this channel.
      wcscpy(DeviceName, Channels[Index].DeviceName);

      // Check if device exists.
      DeviceFound=ExistDevice(DeviceName, &DevIndex);

      // Get the thread id for the device manager.
      DeviceThreadID=GetProcessThreadIDByName(DeviceName);

      // Get the channel name.
      wcscpy(ChannelName, Channels[Index].ChannelName);

      // If device exist and is running, start channel thread.
      if (DeviceFound && DeviceThreadID) {

         // Create path to ringbuffer directory.
         wcscpy(RingBufferDir, RngBufDir);
         StripBackslashFromPath(RingBufferDir);
         wcscat(RingBufferDir, TEXT("\\"));
         wcscat(RingBufferDir, ChannelName);

         // Create path to channel directory.
         wcscpy(ChDir, ChannelDir);
         StripBackslashFromPath(ChDir);
         wcscat(ChDir, TEXT("\\"));
         wcscat(ChDir, ChannelName);

         // Read the channel configuration file
         wcscpy(ConfFileName, ChDir);
         wcscat(ConfFileName, TEXT("\\channel.cnf"));
         ReadFileToMemory(ConfFileName, (LPVOID)&ChannelConfig, (DWORD)sizeof(CHANNEL_CNF_FILE));

         // Create startup information for channel thread.
         SICH.LogicalChNr=GetChnNumberByChnName(ChannelName);
         wcscpy(SICH.ChannelDir, ChDir);
         wcscpy(SICH.RingBufferDir, RingBufferDir);
         SICH.DevMngrThreadID=DeviceThreadID;
         SICH.StartEvent=CreateEvent(NULL, FALSE, FALSE, NULL);

         // Start channel thread.
         StartChannelThread(&SICH, &ChannelThreadID);

         // Delete the event object.
         CloseHandle(SICH.StartEvent);

#ifndef WINCE

         // Also open the monitor window if requested.
		 // This is never done on PocketPC
         wcscpy(ConfFileName, ChDir);
         wcscat(ConfFileName, TEXT("\\Monitor.wnd"));
         bStatus=ReadFileToMemory(ConfFileName, (LPVOID)&MonConfig, (DWORD)sizeof(MON_WND_CNF));
         if (bStatus && MonConfig.AutoOpen) {
            SIMON.LogicalChannelNr=SICH.LogicalChNr;
            SIMON.ChThreadID=ChannelThreadID;
            hThread=CreateThread(NULL, 0, MonitorThrd, (PVOID)&SIMON, 0, &MonThreadID);
            while (!PostThreadMessage(MonThreadID, TM_DUMMY, 0, 0)) Sleep(50);
				CloseHandle(hThread);
         }
#endif

      } else {
         // Device-manager not running or device do not exist.
         // Write message in the event log.
         if (!DeviceFound) {
            wcscpy(Message, TEXT("Seislog: Unable to load "));
            wcscat(Message, ChannelName);
            wcscat(Message, TEXT(". Channel is configured to use a none-existent device ("));
            wcscat(Message, DeviceName); wcscat(Message, TEXT(")."));
         } else {
            wcscpy(Message, TEXT("Seislog: Unable to start "));
            wcscat(Message, ChannelName);
            wcscat(Message, TEXT(". Device driver does not seem to be running."));
         }
         WriteSystemLog(Message, 3);
      }
   }

}



/*******************************************************************/
/* Start a channel thread.                                         */
/*******************************************************************/
HANDLE StartChannelThread(STARTINFO_CHANNEL *SICH, DWORD *ThreadID)
{
   HANDLE hThread;

   // Create the thread.
   hThread=CreateThread(NULL, 0, ChannelThrd, (PVOID)SICH, CREATE_SUSPENDED, ThreadID);

   // Check if 'CreateThread' succeeded.
   if (hThread==NULL) return (HANDLE)NULL;

   // Set the thread's priority.
   SetThreadPriority(hThread, THREAD_PRIORITY_NORMAL);

   // Resume thread. The thread will now start running.
   ResumeThread(hThread);

   // Do not continue before the thread is properly started.
   WaitForSingleObject(SICH->StartEvent, INFINITE);

   // Return handle to the new thread. 
   return hThread;
}




/*********************************************************************************/
/* Start operation of all running channels by sending them the TM_START message. */
/*********************************************************************************/
VOID StartChannels(VOID)
{
   BYTE NumChannels, Index;
   DWORD Channels[MAX_NUM_OF_CHANNELS];

   // Get a list of all channels running.
   NumChannels=GetRunningChannels(Channels);

   // Send TM_START message to all channel threads.
   for (Index=0;Index<NumChannels;Index++) PostThreadMessage(Channels[Index], TM_START, 0, 0); 

}





/*************************************************************************/
/* Removes all files and directories accosiated with a channel.          */      
/*************************************************************************/
BOOL RemoveChannelDirectories(PTCHAR ChannelName)
{
   BOOL Status;
   TCHAR Directory[MAX_PATH+1];
   
   // Remove the channel directory.
   wcscpy(Directory, ChannelDir);
   StripBackslashFromPath(Directory);
   wcscat(Directory, TEXT("\\"));
   wcscat(Directory, ChannelName);
   Status=TRUE;
   if (!ClearDirectory(Directory)) Status=FALSE;
   if (!RemoveDirectory(Directory)) Status=FALSE;

   // Remove the ringbuffer directory.
   wcscpy(Directory, RngBufDir);
   StripBackslashFromPath(Directory);
   wcscat(Directory, TEXT("\\"));
   wcscat(Directory, ChannelName);
   Status=TRUE;
   if (!ClearDirectory(Directory)) Status=FALSE;
   if (!RemoveDirectory(Directory)) Status=FALSE;

   return Status;
}



/*************************************************************************/
/* Returns the number for the first unused channel, ex 3 for Channel03.  */      
/*************************************************************************/
BYTE FindFirstFreeChannel(VOID)
{
   BYTE Index;

   for (Index=1;Index<=MAX_NUM_OF_CHANNELS;Index++) {
      // Check if channelname indicated by Index is free.
      if (GetChannelIndexByNumber(Index)==255) return Index;
   }

   // All channels are in use.
   return 255;
}



/*************************************************************************/
/* Returns an index into the 'Channels' table for channel found with a   */
/* name corresponding to the 'ChannelNr' identifier.                     */      
/*************************************************************************/
BYTE GetChannelIndexByNumber(BYTE ChannelNr)
{
   BYTE Index;
   TCHAR SearchCh[10], str2[3];

   // Create a compare string.
   wcscpy(SearchCh, TEXT("Channel"));
   swprintf(str2, TEXT("%02u"), ChannelNr);
   wcscat(SearchCh, str2);

   // Search trough the 'Channels' table.
   for (Index=0;Index<NumRegisteredChannels;Index++) {
      if (!_wcsicmp(SearchCh, Channels[Index].ChannelName)) return Index;     
   }

   // Channel not found.
   return 255;
}


/*************************************************************************/
/* Moves a channel directory down in the channel list by renaming it to  */
/* the next number.  Channel02 -> Channel03. If Channel03 exist it will  */
/* first be renamed to Channel04 and so on.                              */   
/*************************************************************************/
VOID MoveChannelDown(BYTE OldChNumber)
{
   BYTE NewChNumber;
   TCHAR NewChDir[MAX_PATH+1], ExistingChDir[MAX_PATH+1], str2[3]; 
   TCHAR NewRBDir[MAX_PATH+1], ExistingRBDir[MAX_PATH+1];

   // Calculate channel number for the new channelname.
   NewChNumber=(BYTE)(OldChNumber+1);

   // Create the full path to the existing channel directory.     
   wcscpy(ExistingChDir, ChannelDir);
   StripBackslashFromPath(ExistingChDir);
   wcscat(ExistingChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), OldChNumber);
   wcscat(ExistingChDir, str2);

   // Create the full path to the new channel directory.    
   wcscpy(NewChDir, ChannelDir);
   StripBackslashFromPath(NewChDir);
   wcscat(NewChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), NewChNumber);
   wcscat(NewChDir, str2);

   // Create the full path to the existing ringbuffer directory.     
   wcscpy(ExistingRBDir, RngBufDir);
   StripBackslashFromPath(ExistingRBDir);
   wcscat(ExistingRBDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), OldChNumber);
   wcscat(ExistingRBDir, str2);

   // Create the full path to the new ringbuffer directory.    
   wcscpy(NewRBDir, RngBufDir);
   StripBackslashFromPath(NewRBDir);
   wcscat(NewRBDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), NewChNumber);
   wcscat(NewRBDir, str2);


   // Check if there already exist a directory
   // with the same name as the new directory.
   // If so, recursively rename it to the next number.
   if (ExistDirectory(NewChDir)) MoveChannelDown(NewChNumber);

   // Rename the directories and return status.
   MoveFile(ExistingChDir, NewChDir); 
   if (ExistDirectory(ExistingRBDir)) MoveFile(ExistingRBDir, NewRBDir); 
   
}




/*************************************************************************/
/* Moves a channel directory up in the channel list by renaming it to    */
/* the previous number. Channel03 -> Channel02. If Channel02 exist it    */
/* will first be renamed to Channel01 and so on.                         */   
/*************************************************************************/
VOID MoveChannelUp(BYTE OldChNumber)
{
   BYTE NewChNumber;
   TCHAR NewChDir[MAX_PATH+1], ExistingChDir[MAX_PATH+1], str2[3]; 
   TCHAR NewRBDir[MAX_PATH+1], ExistingRBDir[MAX_PATH+1];

   // Calculate channel number for the new channelname.
   NewChNumber=(BYTE)(OldChNumber-1);

   // Create the full path to the existing directory.    
   wcscpy(ExistingChDir, ChannelDir);
   StripBackslashFromPath(ExistingChDir);
   wcscat(ExistingChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), OldChNumber);
   wcscat(ExistingChDir, str2);

   // Create the full path to the new directory.      
   wcscpy(NewChDir, ChannelDir);
   StripBackslashFromPath(NewChDir);
   wcscat(NewChDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), NewChNumber);
   wcscat(NewChDir, str2);

   // Create the full path to the existing ringbuffer directory.     
   wcscpy(ExistingRBDir, RngBufDir);
   StripBackslashFromPath(ExistingRBDir);
   wcscat(ExistingRBDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), OldChNumber);
   wcscat(ExistingRBDir, str2);

   // Create the full path to the new ringbuffer directory.    
   wcscpy(NewRBDir, RngBufDir);
   StripBackslashFromPath(NewRBDir);
   wcscat(NewRBDir, TEXT("\\Channel"));
   swprintf(str2, TEXT("%02u"), NewChNumber);
   wcscat(NewRBDir, str2);

   // Check if there already exist a directory
   // with the same name as the new directory.
   // If so, recursively rename it to the previous number.
   if (ExistDirectory(NewChDir)) MoveChannelUp(NewChNumber);

   // Rename the directories and return status.
   MoveFile(ExistingChDir, NewChDir); 
   if (ExistDirectory(ExistingRBDir)) MoveFile(ExistingRBDir, NewRBDir); 
   
}




/************************************************************/
/* Returns the channel number based on the channel name.    */
/************************************************************/
BYTE GetChnNumberByChnName(PTCHAR ChannelName)
{
   return (BYTE) _wtoi(ChannelName+7);
}



/******************************************************************************/
/* Returns the average of the samples in a block, absolute values are used    */
/******************************************************************************/
DOUBLE CalculateBlockAboluteAverage(DOUBLE *pBlockSamples, WORD SamplesPrBlock)
{
   WORD Index;
   DOUBLE Sum, Value; 

   Sum=0.0;   
   for (Index=1;Index<=SamplesPrBlock; Index++) {
      Value=*pBlockSamples;
      if (Value<0.0) Value=-Value;
      Sum=Sum+Value;
      pBlockSamples++;
   }

   return Sum/(DOUBLE)SamplesPrBlock;
   
}



/*************************************************************/
/* Checks if a given component code is valid.                */
/* NOTE:Changes made to this function, should also be        */
/*      made to the 'ConvertComponentCode' function.         */
/*************************************************************/
BOOL IsValidCompCode(PTCHAR CompCode)
{
   // Return if component code is
   // shorter then 4 characters.
   if (wcslen(CompCode)<4) return FALSE;

   if (!wcscmp(CompCode, TEXT("S  Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("S  N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("S  E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("L  Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("L  N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("L  E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AL Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AL N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AL E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AH Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AH N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AH E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("A  Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("A  N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("A  E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BV Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BV N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BV E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BH Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BH N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BH E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("B  Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("B  N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("B  E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SV Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SV N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SV E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SH Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SH N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SH E"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("LV Z"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("LV N"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("LV E"))) return TRUE;

   if (!wcscmp(CompCode, TEXT("S  R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("S  T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("L  R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("L  T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AL R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AL T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AH R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("AH T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("A  R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("A  T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BV R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("BV T"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SV R"))) return TRUE;
   if (!wcscmp(CompCode, TEXT("SV T"))) return TRUE;

   return FALSE;
}



#ifdef WINCE


/*******************************************************************/
/*  Callback for the 'Add Channel' Property Sheet (Wizard)         */
/*  This procedure is called when the property sheet               */
/*  is being created and initialized.                              */
/*******************************************************************/

BOOL CALLBACK AddChannelPropSheetProc(HWND hwndDlg, UINT uMsg, LPARAM lParam )
{
	
	SHINITDLGINFO shidi;

	switch (uMsg)
	{
		
	case PSCB_INITIALIZED: // Note: This may be the wrong message to do this!
		 
		// Initialize handle to the Property Sheet
		hAddChannelDlg = hwndDlg;
	
		// Create a Done button and size it.  
		shidi.dwMask = SHIDIM_FLAGS;
		shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hwndDlg;
		SHInitDialog(&shidi);
		
		return TRUE;
	}
	return TRUE;
}



/*******************************************************************/
/* Dialog box procedure for the first property page                */
/* of the 'Add channel' property page                              */
/*******************************************************************/
BOOL CALLBACK AddChannelDlgProc1(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200], str3[4], str4[5], str5[6];
   BYTE ChannelNr, Index;
   CHANNEL_CNF_FILE *ChParams;
   HWND hWndDlgItem;
   DWORD dwIndex;
   TCHAR CompCodes[47][5]={
	   TEXT("S  Z"),TEXT("S  N"),TEXT("S  E"),TEXT("L  Z"),TEXT("L  N"),
	   TEXT("L  E"),TEXT("AL Z"),TEXT("AL N"),TEXT("AL E"),TEXT("AH Z"),
	   TEXT("AH N"),TEXT("AH E"),TEXT("A  Z"),TEXT("A  N"),TEXT("A  E"),
	   TEXT("BV Z"),TEXT("BV N"),TEXT("BV E"),TEXT("BH Z"),TEXT("BH N"),
	   TEXT("BH E"),TEXT("B  Z"),TEXT("B  N"),TEXT("B  E"),TEXT("SV Z"),
	   TEXT("SV N"),TEXT("SV E"),TEXT("SH Z"),TEXT("SH N"),TEXT("SH E"),
	   TEXT("LV Z"),TEXT("LV N"),TEXT("LV E"),TEXT("S  R"),TEXT("S  T"),
       TEXT("L  R"),TEXT("L  T"),TEXT("AL R"),TEXT("AL T"),TEXT("AH R"),
	   TEXT("AH T"),TEXT("A  R"),TEXT("A  T"),TEXT("BV R"),TEXT("BV T"),
	   TEXT("SV R"),TEXT("SV T")
   };


	switch(message)  {

	case WM_INITDIALOG:

     // Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;

      //
      // Initialize 'Channel number for new channel' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT1, _itow((int)FindFirstFreeChannel(), str3,10));

      //
      // Initialize 'station code' edit field.
      //
      SetDlgItemText(hWndDlg, IDC_EDIT2, ChParams->StationCode);

      //
      // Initialize 'component' combo box.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      for (Index=0;Index<47;Index++) {
         dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)CompCodes[Index]);
         if (!wcscmp(ChParams->Component, CompCodes[Index])) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

     // Set the input focus to the 'Station code' edit box.
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_EDIT2);
      SetFocus(hWndDlgItem);
  	  
	  return TRUE; 

      case WM_COMMAND:
 		 switch(wParam) {
		 case IDCANCEL:
			 PropSheet_PressButton(hAddChannelDlg, PSBTN_CANCEL);
			 return TRUE;
		 }
	 return FALSE;

	
	case WM_NOTIFY:

    switch (((NMHDR *)lParam)->code)
	{
	    case PSN_QUERYCANCEL:
	        return TRUE;


		case PSN_APPLY:
		
		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

			 //
			 // Read and check chosen channel number.
			 //
			 GetDlgItemText(hWndDlg, IDC_EDIT1, str3, 3);
			 ChannelNr=(BYTE)_wtoi(str3);
			 if (wcslen(str3)==0 || ChannelNr==0 || ChannelNr>FindFirstFreeChannel()) { 
				// The user may only select an existing channel
				// or the next free channel in the list.
				wcscpy(MessageText, TEXT("The channel number is invalid.\n"));
				wcscat(MessageText, TEXT("Enter the the next free channel-number,\n"));
				wcscat(MessageText, TEXT("or the number of a channel that already exist."));
				MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				// Move the input focus to this field.
				SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
				SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1); 

				// [PocketPC] Change to this Page and Prevent closing 

				PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 0);
				SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				return TRUE;

			 }
         

			 //
			 // Read and check the 'Station code' name.
			 //
			 GetDlgItemText(hWndDlg, IDC_EDIT2, str5, 6);
			 StrRmvSpaces(str5);
			 if (!wcslen(str5)) {
				// The field is empty.
				wcscpy(MessageText, TEXT("The field is empty.\n"));
				wcscat(MessageText, TEXT("Please enter a value."));
				MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				// Move the input focus to this field.
				SetFocus(GetDlgItem(hWndDlg, IDC_EDIT2));
				SendDlgItemMessage(hWndDlg, IDC_EDIT2, EM_SETSEL, 0, -1); 

				// [PocketPC] Change to this Page and Prevent closing 

				PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 0);
				SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				return TRUE;

			 }
			 wcscpy(ChParams->StationCode, str5);

			 //
			 // Read the 'Component' name.
			 //
			 GetDlgItemText(hWndDlg, IDC_COMBO1, str4, 5);
			 wcscpy(ChParams->Component, str4);

			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

			 
		}
		return FALSE;
	}
	return FALSE; 
} 

/*******************************************************************/
/* Dialog box procedure for the second property page               */
/* of the 'Add channel' property page                              */
/*******************************************************************/
BOOL CALLBACK AddChannelDlgProc2(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200];
   TCHAR str7[8];
   INT result;
   HWND hWndDlgItem;
   DWORD dwIndex;
   CHANNEL_CNF_FILE *ChParams;


	switch(message)  {

	case WM_INITDIALOG:

		// Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;


      //
      // Initialize the 'Enable filtering' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->ApplyFilter) {
         CheckDlgButton(hWndDlg, IDC_CHECK1, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE);
         EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
      }
      
      //
      // Initialize values for the two filter parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->FilterLow);
      SetDlgItemText(hWndDlg, IDC_EDIT4, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->FilterHigh);
      SetDlgItemText(hWndDlg, IDC_EDIT5, str7);

      //
      // Initialzie the 'Create ringbuffer' check box.
      //
      if (ChParams->LogToDisk) CheckDlgButton(hWndDlg, IDC_CHECK2, BST_CHECKED); 

      //
      // Initialzie the 'Record data filtered' check box.
      //
      if (ChParams->LogFiltered) CheckDlgButton(hWndDlg, IDC_CHECK4, BST_CHECKED);

      //
      // Initialize the 'Size in minutes' combo box.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("1"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (ChParams->BuffSizeInMinutes==1) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("2"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (ChParams->BuffSizeInMinutes==2) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (ChParams->BuffSizeInMinutes==3) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==4) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("5"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 5);
      if (ChParams->BuffSizeInMinutes==5) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("7"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 7);
      if (ChParams->BuffSizeInMinutes==7) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("10"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 10);
      if (ChParams->BuffSizeInMinutes==10) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("13"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 13);
      if (ChParams->BuffSizeInMinutes==13) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("16"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 16);
      if (ChParams->BuffSizeInMinutes==16) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hWndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("20"));
      SendMessage(hWndDlgItem, CB_SETITEMDATA, dwIndex, 20);
      if (ChParams->BuffSizeInMinutes==20) SendMessage(hWndDlgItem, CB_SETCURSEL, dwIndex, 0);


	  return TRUE;


      case WM_COMMAND:

		 switch(LOWORD(wParam)) {
     
         case IDC_CHECK1:
         // This is the 'Enable filtering' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
               // Enable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), TRUE); 
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT4), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT5), FALSE); 
               // Disable the 'Record filtered data' checkbox.
               EnableWindow(GetDlgItem(hWndDlg, IDC_CHECK4), FALSE); 
            }
         }

	    }
	 break;

	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{
		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

			 //
			 // Read the 'Enable filtering' check box.
			 // If enabled, then read and check the filter paramerters.
			 //
			 ChParams->ApplyFilter=(IsDlgButtonChecked(hWndDlg, IDC_CHECK1)==BST_CHECKED); 
			 if (ChParams->ApplyFilter) {
				// User has enabled filtering. Get and check filter parameters.
				GetDlgItemText(hWndDlg, IDC_EDIT4, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->FilterLow);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
				
				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 1);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT5, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->FilterHigh);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
    			   
				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 1);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}

				// Check values.
				if (ChParams->FilterLow >= ChParams->FilterHigh) {
				   // The value for FilterHigh must be > FilterLow.
				   wcscpy(MessageText, TEXT("The 'FilterHigh' value must be\n"));
				   wcscat(MessageText, TEXT("greater than the 'FilterLow' value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to this field.
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
				   
				   // [PocketPC] Change to this Page and Prevent closing 
			       PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 1);
			       SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
			       return TRUE;

				}
				if (ChParams->FilterLow<0.0 || ChParams->FilterHigh<0.0) {
				   // The value for the filter-parameters must be GE 0.0.
				   wcscpy(MessageText, TEXT("The filterparameters can not be negative."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to the offending field.
				   if (ChParams->FilterLow<0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT4));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT4, EM_SETSEL, 0, -1); 
				   } else {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT5));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT5, EM_SETSEL, 0, -1); 
				   }
				   
				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 1);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
			 }

			 //
			 // Read the 'Create ring-buffer' check box.
			 //
			 ChParams->LogToDisk=(IsDlgButtonChecked(hWndDlg, IDC_CHECK2)==BST_CHECKED); 

			 //
			 // Read the 'Record data filtered' check box.
			 //
			 ChParams->LogFiltered=(IsDlgButtonChecked(hWndDlg, IDC_CHECK4)==BST_CHECKED); 

			 //
			 // Read the 'Size in minutes' combo box.
			 //
			 hWndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
			 dwIndex=SendMessage(hWndDlgItem, CB_GETCURSEL, 0, 0);
			 ChParams->BuffSizeInMinutes=(BYTE)SendMessage(hWndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

		}
		return FALSE;
	}
	return FALSE;
}

/*******************************************************************/
/* Dialog box procedure for the third property page                */
/* of the 'Add channel' property page                              */
/*******************************************************************/
BOOL CALLBACK AddChannelDlgProc3(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   TCHAR MessageText[200];
   TCHAR str7[8];
   INT result;
   CHANNEL_CNF_FILE *ChParams;


	switch(message)  {

	case WM_INITDIALOG:

		// Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;

      //
      // Initialize the 'Enable trigging' check box
      // and activate parameter edit boxes if needed.
      //
      if (ChParams->UseTrigging) {
         CheckDlgButton(hWndDlg, IDC_CHECK3, BST_CHECKED); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
         EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
      }
      
      //
      // Initialize default values for the trigger parameters.
      //
      swprintf(str7, TEXT("%.2f"), ChParams->ShortTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT6, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->LongTermAverage);
      SetDlgItemText(hWndDlg, IDC_EDIT7, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->TriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT8, str7);
      swprintf(str7, TEXT("%.2f"), ChParams->DetriggerRatio);
      SetDlgItemText(hWndDlg, IDC_EDIT9, str7);

	  return TRUE;

      case WM_COMMAND:

		 switch(LOWORD(wParam)) {
     
             case IDC_CHECK3:
         // This is the 'Enable trigging' checkbox.
         switch (HIWORD(wParam)) { 
            case BN_CLICKED:
            if (IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED) {
               // Enable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), TRUE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), TRUE); 
               SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
            } else {
               // Disable the paramater fields.
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT6), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT7), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT8), FALSE); 
               EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT9), FALSE); 
            }
         }
	    }
	 break;	  

	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{
		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

  			 //
			 // Read the 'Enable trigging' check box.
			 // If enabled, then read and check the trigger paramerters.
			 //
			 ChParams->UseTrigging=(IsDlgButtonChecked(hWndDlg, IDC_CHECK3)==BST_CHECKED); 
			 if (ChParams->UseTrigging) {
				// User has enabled trigging. Get and check parameters.
				GetDlgItemText(hWndDlg, IDC_EDIT6, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->ShortTermAverage);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
				   
				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;
				}
				GetDlgItemText(hWndDlg, IDC_EDIT7, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->LongTermAverage);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT8, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->TriggerRatio);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
				GetDlgItemText(hWndDlg, IDC_EDIT9, str7, 8);
				result=swscanf(str7, TEXT("%f"), &ChParams->DetriggerRatio);
				if (result==0 || wcslen(str7)==0) {
				   wcscpy(MessageText, TEXT("No or invalid value entered.\n"));
				   wcscat(MessageText, TEXT("Please enter a valid value."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);
				   SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
				   SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}

				// Check values.
				if (ChParams->ShortTermAverage <= 0.0 ||
					ChParams->LongTermAverage <= 0.0 ||
					ChParams->TriggerRatio <= 0.0 ||
					ChParams->DetriggerRatio <= 0.0 ) {

				   // One or several values are less then or equal to zero.
				   wcscpy(MessageText, TEXT("Value must be greater than zero, please correct."));
				   MessageBox(hWndDlg, MessageText, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

				   // Move the input focus to the offending field.
				   if (ChParams->ShortTermAverage <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT6));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT6, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->LongTermAverage <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT7));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT7, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->TriggerRatio <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT8));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT8, EM_SETSEL, 0, -1); 
				   }
				   if (ChParams->DetriggerRatio <= 0.0) {
					  SetFocus(GetDlgItem(hWndDlg, IDC_EDIT9));
					  SendDlgItemMessage(hWndDlg, IDC_EDIT9, EM_SETSEL, 0, -1); 
				   }

				   // [PocketPC] Change to this Page and Prevent closing 
				   PropSheet_SetCurSel(hAddChannelDlg, hWndDlg, 2);
				   SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE);
				   return TRUE;

				}
			 }
			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

			 
		}
		return FALSE;
	}
	return FALSE;
}

/*******************************************************************/
/* Dialog box procedure for the fourth property page               */
/* of the 'Add channel' property page                              */
/*******************************************************************/
BOOL CALLBACK AddChannelDlgProc4(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

   TCHAR DeviceName[65], ChannelName[65];
   TCHAR str3[4];
   HWND hWndDlgItem;
   BYTE Index, DevIndex;
   DWORD dwIndex;

   CHANNEL_CNF_FILE *ChParams;


	switch(message)  {

	case WM_INITDIALOG:

		// Save pointer to return structure.
      SetWindowLong(hWndDlg, DWL_USER, (LONG)((LPPROPSHEETPAGE)lParam)->lParam);

      // Create a pointer to the configuration structure.
      ChParams=(CHANNEL_CNF_FILE*) ((LPPROPSHEETPAGE)lParam)->lParam;

      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
      for (DevIndex=0;DevIndex<NumRegisteredDevices;DevIndex++) {
         wcscpy(DeviceName, Devices[DevIndex].UniqueDeviceName);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)DeviceName);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)DevIndex);
         if (DevIndex==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

      //
      // Initialize the physical channels list box.
      // The list box will list all channels for the first device in device listbox.
      //
      hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
      for (Index=0;Index<Devices[0].NumChannels;Index++) {
         wcscpy(ChannelName, TEXT("Physical channel "));
         swprintf(str3, TEXT("%u"), Index+1);
         wcscat(ChannelName, str3);
         dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)ChannelName);
         SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index+1);
         if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
      }

		return TRUE; 

	  case WM_COMMAND:

      switch(LOWORD(wParam)) {
         case IDC_LIST1:
         switch (HIWORD(wParam)) { 
            case LBN_SELCHANGE:
            //
            // The user has made a selection in the devices list box.
            // The physical channel list box must be updated to reflect
            // the number of channels of the new device.
            //

            // Get index into 'Devices' of currently selected device.
            hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
            dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
            DevIndex=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);
            
            // Update the physical channel list box.
            hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
            SendMessage(hWndDlgItem, LB_RESETCONTENT , 0, 0);
            for (Index=0;Index<Devices[DevIndex].NumChannels;Index++) {
               wcscpy(ChannelName, TEXT("Physical channel "));
               swprintf(str3, TEXT("%u"), Index+1);
               wcscat(ChannelName, str3);
               dwIndex=SendMessage(hWndDlgItem, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)ChannelName);
               SendMessage(hWndDlgItem, LB_SETITEMDATA, (WPARAM)dwIndex, (LPARAM)Index+1);
               if (Index==0) SendMessage(hWndDlgItem, LB_SETCURSEL, dwIndex, 0);
            }
         }
     }

     break;
	
	case WM_NOTIFY:

		if (((NMHDR *)lParam)->code == PSN_APPLY)
		{

		     //
			 // Get saved pointer to returndata structure.
			 //
			 ChParams=(CHANNEL_CNF_FILE*)GetWindowLong(hWndDlg, DWL_USER);

  			 // Get the selected device.
			 hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST1);
			 dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
			 DevIndex=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);
			 wcscpy(ChParams->DeviceName, Devices[DevIndex].UniqueDeviceName);

			 // Get selected physical channel.
			 hWndDlgItem=GetDlgItem(hWndDlg, IDC_LIST2);
			 dwIndex=SendMessage(hWndDlgItem,LB_GETCURSEL,0,0);
			 ChParams->DeviceChannel=(BYTE)SendMessage(hWndDlgItem, LB_GETITEMDATA, (WPARAM)dwIndex, 0);

             // Close the dialog box ,return the channel nr, and exit.
             //EndDialog(hWndDlg , (int)ChannelNr);


			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

			 
		}
		return FALSE;
	}
	return FALSE;
}


/*******************************************************************/
/* This function opens the 'Add Channel' property sheet             */
/*******************************************************************/

BYTE AddChannelDialogBox(HINSTANCE hInst, HWND hWnd, LPARAM lParam)
{
	PROPSHEETHEADER psh;    // The entire Property Sheet
	PROPSHEETPAGE   psp[4]; // The property pages
	INT i;

	// Init page structures with generic information.
	memset (&psp, 0, sizeof (psp));      // Zero out all unused values.

	for (i = 0; i < 4; i++) {
		psp[i].dwSize = sizeof (PROPSHEETPAGE);
		psp[i].dwFlags = PSP_DEFAULT | PSP_PREMATURE;
		psp[i].hInstance = hInst;         // Instance handle where the
	                                      // dialog templates are located
		psp[i].lParam  = lParam; 
	}
	// Now do the page specific stuff.
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_ADD_CHANNEL_1); // Name of dialog resource for page 1
	psp[0].pfnDlgProc = AddChannelDlgProc1;    // Pointer to dialog proc for page 1

	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_ADD_CHANNEL_2); // Name of dialog resource for page 2
	psp[1].pfnDlgProc = AddChannelDlgProc2;    // Pointer to dialog proc for page 2

	psp[2].pszTemplate = MAKEINTRESOURCE(IDD_ADD_CHANNEL_3); // Name of dialog resource for page 2
	psp[2].pfnDlgProc = AddChannelDlgProc3;    // Pointer to dialog proc for page 2

	psp[3].pszTemplate = MAKEINTRESOURCE(IDD_ADD_CHANNEL_4); // Name of dialog resource for page 2
	psp[3].pfnDlgProc = AddChannelDlgProc4;    // Pointer to dialog proc for page 2


	// Init property sheet header structure.
	psh.dwSize = sizeof (PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USECALLBACK;    // We are using templates not handles.
	psh.hwndParent = hWnd;              // Handle of the owner window
	psh.hInstance = hInst;              // Instance handle of the application
	psh.pszCaption = TEXT ("Add channel");
	psh.nPages = 4;                    // Number of pages
	psh.nStartPage = 0;                  // Index of page to be shown first
	psh.ppsp = psp;                     // Pointer to page structures
	psh.pfnCallback = AddChannelPropSheetProc;   

	// Create property sheet. This returns when the user dismisses the sheet
	// by tapping OK or the Close button.

	int res = PropertySheet (&psh);

	if (res > 0)
	{
		 return (BYTE) ((CHANNEL_CNF_FILE*) lParam)->DeviceChannel;
	}	
	else 
	{
		return 0;
	}
}

#endif
