/*
	#define UNICODE
	#define _UNICODE
	#define STRICT

*/

#include "stdafx.h"


#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"

#define WINCE

#ifdef WINCE
    // The type pointer to SYSTEMTIME is not defined 
    // in Windows CE so we define it here.
	typedef SYSTEMTIME* PSYSTEMTIME;
#endif


#define FSOS 1 // FIND START OF SENTENCE
#define FEOS 2 // FIND END OF SENTENCE
#define TIMECORR 0.4




//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_GPSREADER { 
   HANDLE hComm;              // Handle of communications device.
   DWORD ThreadID;            // Thread-ID for parent thread.
   TCHAR UniqueProcName[65];   // Thread instance name.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
};


extern TCHAR SerialPortGPS[];
extern DOUBLE MeanTimeBeforeGPRMC;
extern BYTE DebugLevel;
extern BYTE GPSDebug;
extern HWND hWndFrame;


DWORD WINAPI GPSReaderThread(PVOID StartupStruct);
BOOL GetSentenceField(BYTE, PTCHAR, PTCHAR);
BOOL GetTimeFromGPRMCSentence(PTCHAR, PSYSTEMTIME);
VOID GetFixQualityFromGPGGASentence(PTCHAR, PBYTE);


/* *********************************************************************** */
/*  System clock synchronization thread.                                    */
/* *********************************************************************** */
DWORD WINAPI SysClkSyncThrd(PVOID StartupInfo)
{
   MSG msg;
   TCHAR UniqueProcName[65], Message[100], str10[11];
   BYTE FixStatus, SyncStatus;
   BOOL SerialRead;
   HANDLE hComm, hReaderThread;
   MSG_PARAM RP, IP;
   STARTINFO_GPSREADER StartInfo;
   DWORD CurrentThreadID, status;
   DWORD ReaderThreadID;
   MsgList NotifyList;
   BOOL DataLost;

   
   //
   // Perform thread initialization.
   //
   FixStatus=0;
   SyncStatus=2;
   DataLost=TRUE;
   hReaderThread=NULL;                    // The serial-reader thread is not running.
   CurrentThreadID=GetCurrentThreadId();  // Get ThreadID for this thread.
   

   // 
   // Create a thread name.
   //
   wcscpy(UniqueProcName, TEXT("GPS on "));
   wcscat(UniqueProcName, SerialPortGPS);


   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }


   //
   // Open and initialize the serial port.
   //

#ifdef WINCE
   // Add ':' to the port number (required in Windows CE)
   TCHAR CESerialPortGPS[7];
   wcscpy(CESerialPortGPS, SerialPortGPS);
   wcscat(CESerialPortGPS, TEXT(":"));
   hComm=CreateFile(CESerialPortGPS, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
#else
   hComm=CreateFile(SerialPortGPS, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
#endif

   if (hComm==INVALID_HANDLE_VALUE) {
      // Update program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Unable to open communication port "));
      wcscat(Message, SerialPortGPS);
      wcscat(Message, TEXT("."));
      WriteSystemLog(Message, 3);
   } else {
      // Set up startup information for the serial reader thread routine.
      SerialRead=TRUE;
      StartInfo.hComm=hComm;
      StartInfo.ThreadID=CurrentThreadID;
      wcscpy(StartInfo.UniqueProcName, UniqueProcName);
      StartInfo.SerialRead=&SerialRead;

      // Start the thread.
      hReaderThread=CreateThread(NULL, 0, GPSReaderThread, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
      SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);            
      ResumeThread(hReaderThread); 
   }

   // Write text on status bar decscibing the initial sync status.
   DrawStatusBarText(SyncStatus, UniqueProcName);


   // 
   // Enter the thread message loop. 
   // Thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {
      //
      // If debuglevel is 2 then log all messages recieved.
      //
      if (DebugLevel>1) {
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }
      
      //
      // If message is accompanied by a MSG_PARAM structure, then 
      // copy this structure to a local buffer (IP). Message is 
      // assumed to be accompanied by a structure if lParam is 
      // not 0.
      //
      if (msg.lParam) IP=*((MSG_PARAM*)msg.lParam);

      //
      // Initialize return parameters (RP) to default values.
      //
      ZeroMemory((PVOID)&RP, (DWORD)sizeof(MSG_PARAM));
      RP.idThread=CurrentThreadID;
      RP.bStatus=TRUE; 

      //
      // Process message.
      //
      switch (msg.message) {
         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (IP.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)IP.vPointer)) {
            RP.bStatus=FALSE;
            RP.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (IP.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)IP.vPointer);
         break;   


         case TM_GET_TIMESYNC_STATUS:
         //
         // This message is sendt from a window or thread who
         // wants to know the status of the GPS time synchronisation.
         // This message must be accompanied by a MSG_PARAM structure.
         //
         //
         // Returns the operational status of the thread.
         // Values are:
         // 0 : System clock is beeing synchronized, satelite fix OK.
         // 1 : System clock is beeing synchronized, but satelite fix is lost.
         // 2 : System clock is NOT beeing synchronized, communication error.
         //
         RP.wStatus=(WORD)SyncStatus;
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The GPS reader-thread has lost data input.
         //
         DataLost=TRUE;
         SyncStatus=2;

         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);

         // Update status bar text.
         DrawStatusBarText(SyncStatus, UniqueProcName);

         // Write a message to the log.
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Not receiving data from "));
         wcscat(Message, SerialPortGPS);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
         break;


         
         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The GPS reader-thread has regained data input.
         //
         DataLost=FALSE;
         SyncStatus=0;
         if (!FixStatus) SyncStatus=1;

         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);

         // Update status bar text.
         DrawStatusBarText(SyncStatus, UniqueProcName);

         // Write a message to the log.
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Receiving data from "));
         wcscat(Message, SerialPortGPS);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 1);
         break;



         case TM_DEVICEREADER_NOFIX:   
         //
         // The GPS reader-thread indicates that fix has been lost.
         //

         // Change the fix status.
         FixStatus=0;

         // Change the sync status.
         if (!SyncStatus) SyncStatus=1;
            
         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);
         
         // Update status bar text.
         DrawStatusBarText(SyncStatus, UniqueProcName);

         // Write a message to the log.
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Satellite fix lost."));
         WriteSystemLog(Message, 2);
         break;



         case TM_DEVICEREADER_FIXOK:   
         //
         // The GPS reader-thread indicates that fix has been regained.
         //
         FixStatus=1;
         if (SyncStatus==1) SyncStatus=0;

         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_TIMESYNC_STATUS, (WPARAM)SyncStatus, 0);

         // Update status bar text.
         DrawStatusBarText(SyncStatus, UniqueProcName);        
         
         // Write a message to the log.
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Satellite fix OK."));
         WriteSystemLog(Message, 1);
         break;



         case TM_EXIT:                 
         //
         // The thread will cleanup and exit when this message is recieved.         
         //

         // Stop the reader thread.
         SerialRead=FALSE;
         status=WaitForSingleObject(hReaderThread, 2000);
         if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

         // Close the communication port.
         CloseHandle(hComm);

         // Clear the status bar.
         PostMessage(hWndFrame, TM_WRITE_STATUSBAR, 0, (LPARAM)TEXT(""));

         // End the message loop.
         ClearMessageQueue();  // Delete all messages in message queue.
         PostQuitMessage(0);   // Post WM_QUIT message to end the message loop.
         break;
      }

      // Return status values, and release calling process.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=RP;
         SetEvent((HANDLE)msg.wParam);
      }
   }


   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}





/* *********************************************************************** */
/*  Serial port reader procedure for the GPS device.                       */
/* *********************************************************************** */
DWORD WINAPI GPSReaderThread(PVOID StartupStruct)
{
   DCB dcb;
   INT pos;
   TCHAR Message[200], str50[51];
#ifndef WINCE
   // This string is not used on Windows CE
   TCHAR DevCntrlStr[64];
#endif
   TCHAR Sentence[83], SentenceID[7], CmpResult, str2[3];
   BYTE FixStatus, OldFixStatus, ReadStatus;
   DWORD BytesRead;
   DOUBLE TransTime, TDiff; 
   SYSTEMTIME GPSTime, CurrentTime;
   SYSTEMTIME ReadTime, EndReadTime;
   COMMTIMEOUTS ct={0, 0, 1500, 0, 0};
   STARTINFO_GPSREADER *SI;
   BOOL DataLost, SentenceRead, bStatus;

   
   // Create a pointer of correct type to 
   // startup structure in parent thread.
   SI=(STARTINFO_GPSREADER*)StartupStruct;

   // Initialize some variables.
   FixStatus=0;            // So far we have no satelite fix.
   OldFixStatus=0;         // We have no previous fix status.
   DataLost=TRUE;          // We have no data input.
   ReadStatus=FSOS;        // Start looking for StartOfSentence.
   SentenceRead=FALSE;     // No whole sentence have been read.
 
   //Initialize the communications parameters. 
   SetCommTimeouts(SI->hComm, &ct);                         // Enable communication timeouts.
   SetupComm(SI->hComm, 2048, 0);                           // Initialize the communications parameters. 
   GetCommState(SI->hComm, &dcb);                           // Get serial port mode.

#ifdef WINCE

   // The function BuildCommDCB is not available on
   // Windows CE, so we set the struct members manually.

   dcb.BaudRate     = 4800;
   dcb.fParity      = TRUE;
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;

   dcb.fInX         = FALSE;
   dcb.fOutX        = FALSE;
   dcb.fOutxDsrFlow = FALSE;
   dcb.fOutxCtsFlow = FALSE;  
   dcb.fDtrControl  = DTR_CONTROL_ENABLE;
   dcb.fRtsControl  = RTS_CONTROL_ENABLE;

#else
   wcscpy(DevCntrlStr, TEXT("baud=4800 parity=N data=8 stop=1"));
   BuildCommDCB(DevCntrlStr, &dcb);									// 4800,8,n,1 is expected.
#endif

   dcb.fAbortOnError=FALSE;                                 // Do not abort read on error.
   SetCommState(SI->hComm, &dcb);                           // Set serial port mode.
   PurgeComm(SI->hComm, PURGE_RXCLEAR);                     // Clear communication input buffer.
   
   // Start reading the serial port in a loop
   // until the SI->SerialRead variable goes false.
   // This variable is controlled by the parent thread.
   while(*SI->SerialRead) {
      //   
      // Try reading 2 bytes from the serial port.
      // If not read within 1500ms, the read will timeout.
      ReadFile(SI->hComm, (PVOID)str2, 2, &BytesRead, NULL); str2[2]=0;

      //
      // Process bytes read.
      //
      if (BytesRead==2) { // 2 bytes was successfully read.
         if (ReadStatus==FSOS) {
            // We are searching for the start of a message.
            // Check if str2 contains the $ character.
            if (StrFindChar(str2, '$', &pos)) {
               // We have just read the start of a sentence.
               GetSystemTime(&ReadTime);           // Save the time when start of sentence arrived.
               if (pos==0) wcscpy(Sentence, str2); // Copy start of sentence into sentence string.
               if (pos==1) wcscpy(Sentence, TEXT("$"));  // Insert only the $ sign into sentence string.
               ReadStatus=FEOS;                    // We will now search for EndOfSentence. 
            }
         } else if (ReadStatus==FEOS) {
            // We are searching for the end of a message.
            // Check if str2 contains the '\r'character.
            if (StrFindChar(str2, '\r', &pos)) {
               // We have just read the end of a sentence.
               GetSystemTime(&EndReadTime);                 // Save the time when end of sentence arrived.
               wcscat(Sentence, str2);                      // Add read bytes to sentence.
               if (Sentence[wcslen(Sentence)-1]!='\n') wcscat(Sentence, TEXT("\n"));
               TransTime=TimeDiff(&ReadTime, &EndReadTime); // Calculate sentence transfer time.
               SentenceRead=TRUE;                           // Indicate that this sentence is complete.
               ReadStatus=FSOS;                             // Next we will search for StartOfSentence.
            } else {
               // End of sentence was not found yet. Add
               // read bytes to current unfinished sentence.
               wcscat(Sentence, str2);
               // Check length of sentence read so far. If longer then 82
               // bytes,then we are not reading a valid NMEA0183 sentence.
               if (wcslen(Sentence)>82) {
                  // Abort sentence, and search for start of next sentence.
                  ReadStatus=FSOS;
                  // Write error message in the log.
                  wcscpy(Message, SI->UniqueProcName);
                  wcscat(Message, TEXT(": Sentence longer than 82 bytes read, this is not a valid NMEA-0183 sentence."));
                  WriteSystemLog(Message, 3);
               }
            }
         }

         //
         // If a sentence has been completed then process it.
         //
         if (SentenceRead) {
            // A complete sentence has been read into the 'Sentence' string.
            // Depending on which sentence has been read, take apropriate action.
            //
            // Get the sentence-ID.
            wcsncpy(SentenceID, Sentence, 6); SentenceID[6]=0;

            //
            // Handle the $GPRMC sentence.
            if (!wcscmp(SentenceID, TEXT("$GPRMC"))) {
               // Get time from sentence.
               bStatus=GetTimeFromGPRMCSentence(Sentence, &GPSTime);
               if (bStatus) {
                  AddToTime(&GPSTime, TIMECORR); // Apply timecorrection.  
                  if (MeanTimeBeforeGPRMC > 0.0) AddToTime(&GPSTime, MeanTimeBeforeGPRMC);         // Apply user specified timecorrection.
                  if (MeanTimeBeforeGPRMC < 0.0) SubstractFromTime(&GPSTime, -MeanTimeBeforeGPRMC); // Apply user specified timecorrection.
                  TDiff=TimeDiff(&GPSTime, &ReadTime); // Calculate time difference.
                  CmpResult=0;
                  if (TDiff>0.01 && TDiff<0.1) {
                     // Time difference between system time and
                     // GPS time is less then 0.1 second. We will
                     // adjust the systemtime 10ms so that it get
                     // closer to the GPS time.
                     CmpResult=TimeCmp(&GPSTime, &ReadTime);
                     GetSystemTime(&CurrentTime);
                     if (CmpResult== 1) AddToTime(&CurrentTime, 0.01);
                     if (CmpResult==-1) SubstractFromTime(&CurrentTime, 0.01);
                     SetSystemTime(&CurrentTime);    // Adjust system time + or - 10 ms.  
                  } 
                  if (TDiff>=0.1) {
                     // Timedifference is greater then 0.1 sec.
                     // We will just set the system time equal to GPS time.
                     AddToTime(&GPSTime, TransTime); // Compensate for sentence transfer time.
                     SetSystemTime(&GPSTime);        // Set the time.
                  }
               } else {
                  wcscpy(Message, SI->UniqueProcName);
                  wcscat(Message, TEXT(": Received a bad $GPRMC sentence."));
                  WriteSystemLog(Message, 2);
               }
            }

            //
            // Handle the $GPGGA sentence.
            if (!wcscmp(SentenceID, TEXT("$GPGGA"))) {
               // Get fix quality.
               GetFixQualityFromGPGGASentence(Sentence, &FixStatus);  
               // If fix quality has changed, inform parent process.
               if (FixStatus!=OldFixStatus) {
                  OldFixStatus=FixStatus;
                  if (FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_FIXOK, 0, 0);
                  if (!FixStatus) PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NOFIX, 0, 0);
               }
            }

            //
            // Write sentence to log if we debug modus.
            if (GPSDebug==2) {  
               wcscpy(Message, SI->UniqueProcName);
               wcscat(Message, TEXT(": Received sentence: "));
               wcscat(Message, Sentence);
               WriteSystemLog(Message, 1);
            }
         }

         // If data input was previusly lost, then inform
         // parent process that data input has been restored.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0);
         }

         // Write debug messages to the log if apropriate.
         if (GPSDebug && SentenceRead && !wcscmp(SentenceID, TEXT("$GPRMC"))) {
            wcscpy(Message, SI->UniqueProcName);
            wcscat(Message, TEXT(": ReadTime="));
            CreateTimeString(&ReadTime, str50);
            wcscat(Message, str50);
            wcscat(Message, TEXT(", GPS-Time="));            
            CreateTimeString(&GPSTime, str50);
            wcscat(Message, str50);
            swprintf(str50, TEXT(", Time difference %4f"), (DOUBLE)TDiff);
            wcscat(Message, str50);
            if (CmpResult==1) wcscat(Message, TEXT(", Clock adjusted +10ms"));
            if (CmpResult==-1) wcscat(Message, TEXT(", Clock adjusted -10ms"));
            WriteSystemLog(Message, 1);
         }

         // Reset 'SentenceRead' flag.
         if (SentenceRead) SentenceRead=FALSE;

      } else {
         // Timeout reading 2 bytes. The GPS may be 
         // disconnected, without power, or defective.
         if (!DataLost) {
            DataLost=TRUE;
            ReadStatus=FSOS;
            SentenceRead=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0);
         }

         // Wait one second before retrying the read.
         Sleep(1000); 
      }
   }
  
   // Thread ends here.
   ExitThread(0);
   return 0L;
   
}










/*****************************************************************/
/* This function pulls out a field (string) from a $GPRMC        */
/* sentence and returns the string in the FieldStr variable.     */
/*****************************************************************/
BOOL GetSentenceField(BYTE FieldNr, PTCHAR Sentence, PTCHAR FieldStr)
{
   BYTE CommasFound, Index, wcslength;
   BYTE IndexFirstComma, IndexNextComma;
   DWORD FieldSize;

   // Find the length of the sentence.
   // Return if string is empty.
   wcslength=(BYTE)wcslen(Sentence);
   if (!wcslength) return FALSE;

   // Find the index for the commas marking
   // the start and end of the requested field.
   CommasFound=0;
   IndexNextComma=0;
   IndexFirstComma=0;
   for (Index=6;Index<wcslength;Index++) {
      if (Sentence[Index]==',') {
         CommasFound++;
         if (CommasFound==FieldNr) IndexFirstComma=Index;
         if (CommasFound==FieldNr+1) IndexNextComma=Index;
      }
      if (Sentence[Index]=='\r') {
         // If we have reached the end of the sentence
         // and the second comma has not been found, 
         // then the '\r' character will mark the end
         // of the field.
         if (!IndexNextComma) IndexNextComma=Index;
      }
   }
   
   // If commas representing the start and end of
   // the field was not found, then return FALSE.
   if (!IndexFirstComma || !IndexNextComma) return FALSE;

   // Return the field string.
   FieldSize=IndexNextComma-IndexFirstComma-1;
   if (FieldSize==0) {
      wcscpy(FieldStr, TEXT(""));
   } else {
      CopyMemory((PVOID)FieldStr, (PVOID)&Sentence[IndexFirstComma+1], FieldSize);
      FieldStr[FieldSize]=0;
   }

   return TRUE;
}


/****************************************************************/
/* This function pulls out the time and date from a $GPRMC      */
/* sentence and inserts the values into a SYSTEMTIME structure. */
/****************************************************************/
BOOL GetTimeFromGPRMCSentence(PTCHAR Sentence, PSYSTEMTIME Time)
{
   TCHAR FieldStr[7], str2[3];
   
   // First pick out the time. This is the 
   // first field in the string. Assuming 
   // that this field is always 6 characters
   // long.
   GetSentenceField(1, Sentence, FieldStr);
   if (wcslen(FieldStr)!=6) return FALSE;

   // Insert time into SYSTEMTIME structure.
   ZeroMemory((PVOID)Time, (DWORD)sizeof(SYSTEMTIME));
   wcsncpy(str2, FieldStr, 2); str2[2]=0;
   Time->wHour=(WORD)_wtoi(str2);
   wcsncpy(str2, FieldStr+2, 2); str2[2]=0;
   Time->wMinute=(WORD)_wtoi(str2);
   wcsncpy(str2, FieldStr+4, 2); str2[2]=0;
   Time->wSecond=(WORD)_wtoi(str2);

   // Pick out the date. This is field #9 in the string.
   // Assuming that this field is always 6 characters long.
   GetSentenceField(9, Sentence, FieldStr);
   if (wcslen(FieldStr)!=6) return FALSE;

   // Insert date into SYSTEMTIME structure.
   wcsncpy(str2, FieldStr, 2); str2[2]=0;
   Time->wDay=(WORD)_wtoi(str2);
   wcsncpy(str2, FieldStr+2, 2); str2[2]=0;
   Time->wMonth=(WORD)_wtoi(str2);
   wcsncpy(str2, FieldStr+4, 2); str2[2]=0;
   Time->wYear=(WORD)_wtoi(str2);
   if (Time->wYear < 80) {
      // We have passed year 2000.
      Time->wYear=Time->wYear+2000;
   } else {
      // We have not yet passed year 2000.
      Time->wYear=Time->wYear+1900;
   }

   return TRUE;
}




/************************************************************************/
/* This function pulls out the fix quality from a $GPGGA sentence.      */
/************************************************************************/
VOID GetFixQualityFromGPGGASentence(PTCHAR Sentence, PBYTE FixQuality)
{
   TCHAR FieldStr[7];
   
   // Pick out the fix quality field. 
   // This is field #6 in the string.
   GetSentenceField(6, Sentence, FieldStr);

   // Return fix quality.
   if (wcslen(FieldStr)==1) {
      *FixQuality=(BYTE)_wtoi(FieldStr);
   } else {
      *FixQuality=0;
   }
}
