//#define STRICT

#include "stdafx.h"

#include <aygshell.h>
#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"

#define WINCE

//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   HANDLE hComm;              // Handle of communications device 
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   DOUBLE TimeCorrection;     // Time correction to apply (in seconds).
   BYTE SyncStatus;           // The current status of the time syncronization thread.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;             // An identifier for this particular model.
   BYTE NumChannels;          // Can be 1 or 3.
   TCHAR PortName[6];          // COM1, COM2 ... -> COM16
   DOUBLE TimeCorrection;     // Time correction in seconds.
};


//
// Global variables.
//
extern BYTE DebugLevel;
extern BOOL StartGPSSyncThread;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadEDM006(PVOID);
BOOL CALLBACK ConfigDlgProcEDM006(HWND, UINT, WPARAM, LPARAM);
BOOL EDM006BytesOK(PBYTE);





/* *********************************************************************** */
/*  Manager thread for device Earth Data EDM006. DeviceID=100.             */
/* *********************************************************************** */
DWORD WINAPI EDM006MngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   BOOL SerialRead, MemAlloc, ConfigRead;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;

   
   //
   // Perform thread initialization.
   //
   hReaderThread=NULL;                               // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=100;               // EDM006 uses device-id 100.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM1.
      DeviceConf.NumChannels=3;              // Assume digitizer has been configured for 3 channels.
      DeviceConf.TimeCorrection=0.0;         // Default time correction (seconds).

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("EDM006 at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);

   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);

   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial port: "));
   wcscat(DeviceResources, DeviceConf.PortName);

   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=(WORD)(225/DeviceConf.NumChannels);
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;


   // Open the serial port.
   if (!SI.ConfigOnly) {

#ifdef WINCE
      // All ports in Windows CE must end with ':'
      TCHAR CEPortName[7];
      wcscpy(CEPortName, DeviceConf.PortName);
      wcscat(CEPortName, TEXT(":"));
      hComm=CreateFile(CEPortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#else
      hComm=CreateFile(DeviceConf.PortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#endif
      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("Earth Data Ltd"));
   wcscpy(DS.Model, TEXT("EDM006"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   if ((DS.ChannelData[0]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if (DeviceConf.NumChannels==3) {
      if ((DS.ChannelData[1]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
      if ((DS.ChannelData[2]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   }
   
   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID) {
               // The system clock is syncronized by some thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0; 
            }

            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.hComm=hComm;
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL); 

            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadEDM006, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);            
            ResumeThread(hReaderThread); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID) UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);
         
            // Close the mutex object.
            CloseHandle(StartInfo.hSyncStatusMutex);

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_EDM006), NULL, 
                                  (DLGPROC)ConfigDlgProcEDM006, (LPARAM)&DevConfDlg);  

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Create a unique device instance name.
               wcscpy(UniqueDeviceName, TEXT("EDM006 at "));
               wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;

         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters not given in the device resource string to 
            // the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Time Correction="));
            swprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from the time syncronization
         // thread to inform about a change in the syncronization
         // status.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;

         
         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);
         }

         // Free memory buffers.
         if (DS.ChannelData[0]) LocalFree((HLOCAL)DS.ChannelData[0]); 
         if (DS.ChannelData[1]) LocalFree((HLOCAL)DS.ChannelData[1]); 
         if (DS.ChannelData[2]) LocalFree((HLOCAL)DS.ChannelData[2]); 


         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;


         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Not receiving data."));
         WriteSystemLog(Message, 2);
         break;


         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //

         // Send Notification to all subscribed threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Receiving data."));
         WriteSystemLog(Message, 1);
         break;


         case TM_DEVICEREADER_BADBLOCK:   // A bad block of data was received from the device.
         // Send Notification to all subscribed threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the Earth Data EDM006 device.         */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadEDM006(PVOID StartupStruct)
{
   DCB dcb;
#ifndef WINCE
   TCHAR DevCntrlStr[64];
#endif
   BOOL DataLost, ReadStatus;
   WORD BlockSize, NumChannels, NumBlocksPrChannel, SampleNr;
   DWORD BytesRead, BlockNr, OffsetCurrentSample, OffsetToBlockData;
   SYSTEMTIME SysTime;
	COMMTIMEOUTS ct={30, 0, 1200, 0, 0};
   BYTE Buffer[768], *bPtr, TimeQuality, SyncStatus;
   VOID *DataBlocksCh1, *DataBlocksCh2, *DataBlocksCh3; 
   STARTINFO_DEVREADER *SI;
   BLOCK_HEADER DBH;
   long int LIValue;
   BOOL BytesOK;


   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block channel 3.

   //Initialize the communications parameters. 
   SetupComm(SI->hComm, 768*3, 0); 
   GetCommState(SI->hComm, &dcb);

#ifdef WINCE
   dcb.BaudRate     = 19200;
   dcb.fParity      = TRUE;
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;

   dcb.fInX         = FALSE;
   dcb.fOutX        = FALSE;
   dcb.fOutxDsrFlow = FALSE;
   dcb.fOutxCtsFlow = FALSE;  
   dcb.fDtrControl  = DTR_CONTROL_ENABLE;
   dcb.fRtsControl  = RTS_CONTROL_ENABLE;
#else
   wcscpy(DevCntrlStr, TEXT("baud=19200 parity=N data=8 stop=1"));
   BuildCommDCB(DevCntrlStr, &dcb);
#endif
   
   dcb.fAbortOnError=FALSE; 
   SetCommState(SI->hComm, &dcb);
   SetCommTimeouts(SI->hComm, &ct);

   // Clear the input buffer.
   PurgeComm(SI->hComm, PURGE_RXCLEAR);

   // Start reading the serial port.
   BlockNr=0;  // Start filling data into first block. This variable is zero based.
   DataLost=TRUE;
   while(*SI->SerialRead) {
      
      // Try reading 768 bytes into buffer. Also save the time.
      ReadStatus=ReadFile(SI->hComm, (LPVOID)Buffer, 768, &BytesRead, NULL);
      GetSystemTime(&SysTime); // Get current time/date. This is UTC time.

      // Process incoming bytes.
      if (BytesRead==768) {  // 768 bytes was read.
         
         // Apply time correction.
         SubstractFromTime(&SysTime, 1.4);

         // Apply user specified time correction.
         if (SI->TimeCorrection>0.0) AddToTime(&SysTime, SI->TimeCorrection);
         if (SI->TimeCorrection<0.0) SubstractFromTime(&SysTime, -SI->TimeCorrection);

         // Check if bytes seems to be ok.
         BytesOK=EDM006BytesOK(Buffer);

         // Find the time quality for this block.
         TimeQuality=0;
         if (SysClkSyncThreadID) {
            WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
            SyncStatus=SI->SyncStatus;
            ReleaseMutex(SI->hSyncStatusMutex);
            if (SyncStatus==0) TimeQuality=2;
            if (SyncStatus==1) TimeQuality=1;
            if (SyncStatus==2) TimeQuality=3;
         }

         //
         // Update block headers.
         //
         if (NumChannels==1) { // Digitizer has 1 channel.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=SysTime;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=!BytesOK;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         } else {              // Digitizer has 3 channels.
            DBH.BlockID=BlockNr;
            DBH.TimeTag=SysTime;
            DBH.TimeQuality=TimeQuality;
            DBH.NullData=!BytesOK;
            DBH.DataLost=DataLost;
            DBH.EventDetected=FALSE;
            DBH.EventTime=0;
            bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
            bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         }

         // Calculate offset from start of data buffer, into data part of currrent block.
         OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);

         //
         // Insert sample values into block.
         //
         if (BytesOK) {
            if (NumChannels==1) { // Digitizer has 1 channel.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<225;SampleNr++) {   // There are 225 samples in this channel.
                  // Process sample pointed to by 'SampleNr'.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[95+SampleNr*3], Buffer[94+SampleNr*3], Buffer[93+SampleNr*3]);
                  *((long int*)bPtr)=LIValue;

                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }
            } else { // Digitizer has 3 channels.
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<75;SampleNr++) {    // There are 75 samples for each channel.
                  // Process sample pointed to by 'SampleNr' for channel 1.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[95+SampleNr*9], Buffer[94+SampleNr*9], Buffer[93+SampleNr*9]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 2.
                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[98+SampleNr*9], Buffer[97+SampleNr*9], Buffer[96+SampleNr*9]);
                  *((long int*)bPtr)=LIValue;

                  // Process sample pointed to by 'SampleNr' for channel 3.
                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  LIValue=CreateLongIntFrom3Bytes(Buffer[101+SampleNr*9], Buffer[100+SampleNr*9], Buffer[99+SampleNr*9]);
                  *((long int*)bPtr)=LIValue;

                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }
            }
         
         } else {
            // 768 bytes was read, but they don't seem
            // to be OK. Fill the block with nulldata.
            if (NumChannels==1) { // Digitizer has 1 channel.
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), 225*4);
            } else { // Digitizer has 3 channels.
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), 75*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh2+OffsetToBlockData), 75*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh3+OffsetToBlockData), 75*4);
            }

            // Inform parent thread.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0); 
         }
   
         // If data stream has just been restored,then inform parent thread.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;
      
      } else {
         // Failed to read 768 bytes.The digitizer has been
         // disconnected, or an other comm error has occured.
         if (DataLost==FALSE) {
            DataLost=TRUE;

            // Signal parent process that .
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
         }

         // Wait one second before trying a new read.
         if (!ReadStatus) Sleep(1000);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}




/*******************************************************************/
/*    Configuration dialog box procedure for EDM006.               */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcEDM006(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
   SHINITDLGINFO shidi;
#endif
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"),TEXT("COM2"), TEXT("COM3"), TEXT("COM4"), TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8")
                    , TEXT("COM9"),TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:

#ifdef WINCE
 	  // Create a Done button and size it. New in PocketPC version.
 	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);
#endif

      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;
   
      // Initialize the 'communication port' combo box.
#ifdef WINCE
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COM_PORT);
#else
	  hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
#endif
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'number of channels' combo box.
#ifdef WINCE
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_CHANNELS);
#else
	  hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
#endif

      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("1"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (DC->NumChannels==1) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("3"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->NumChannels==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
         
      // Initialize the 'time correction (seconds)' edit box.
      swprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
#ifdef WINCE
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);
#else
	  SetDlgItemText(hWndDlg, IDC_EDIT1, Str20);
#endif


      // Bring window to front.
      BringWindowToTop(hWndDlg);

      return TRUE;


      case WM_COMMAND:
      switch(wParam) {
         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'communication port' combo box.
#ifdef WINCE         
		 hwndDlgItem=GetDlgItem(hWndDlg, IDC_COM_PORT);
#else
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO1);
#endif

         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the 'number of channels' combo box.
#ifdef WINCE
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_CHANNELS);
#else
		 hwndDlgItem=GetDlgItem(hWndDlg, IDC_COMBO2);
#endif
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->NumChannels=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the time correction (seconds) edit box.
#ifdef WINCE
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
#else
         GetDlgItemText(hWndDlg, IDC_EDIT1, Str20, 20);
#endif
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_EDIT1));
            SendDlgItemMessage(hWndDlg, IDC_EDIT1, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}



/*******************************************************************/
/*    Checks if the string from the device looks OK.               */
/*******************************************************************/

BOOL EDM006BytesOK(PBYTE string)
{
   // Check that string starts with 'MOD\0'.
   if (wcscmp(TEXT("MOD"), (PTCHAR)string)) return FALSE;

   // Check test patterns.
   if (*(string+30)!=0x22 || *(string+31)!=0x22 || *(string+32)!=0x00) return FALSE;
   if (*(string+33)!=0x55 || *(string+34)!=0x55 || *(string+35)!=0x00) return FALSE;
   if (*(string+36)!=0x00 || *(string+37)!=0xFF || *(string+38)!=0xFF) return FALSE;

   // Check data header. Must start with "DAT\0".
   if (wcscmp(TEXT("DAT"), (PTCHAR)(string+87))) return FALSE;

   // No problems detected, return TRUE.
   return TRUE;
}