#include "stdafx.h"
//#define STRICT
#include <windows.h>
#include "seislog.h"
#include "msglist.h"


//
// Definition of subroutines used in class 'MsgList'.
//


MsgList::MsgList(void) {

   // Initialize variables.
   NumNodes=0;
   PtrFirstNode=PtrLastNode=(MSGNODE*)NULL;

}



MsgList::~MsgList(void) {

   // Destroy list.
   if (PtrFirstNode) DestroyList();

}



VOID MsgList::DestroyList(VOID) {
//
//  Releases memory for all nodes in the
//  nodechain pointed to by 'NodeListPtr'.
//
   MSGNODE *NextNode, *CurrentNode;

   CurrentNode=PtrFirstNode;
   while (CurrentNode!=NULL) {
      NextNode=CurrentNode->next_node;
      GlobalFree((HGLOBAL)CurrentNode);
      CurrentNode=NextNode;
   }
}



BOOL MsgList::Subscribe(SUBSCRIBE *SD) {
//
// Adds a node to the end of the notification list.
// 

   MSGNODE *pNewNode;

   // Allocate memory for new node.
   pNewNode=(MSGNODE*)GlobalAlloc(GMEM_FIXED, sizeof(MSGNODE));

   // Abort if memory could not be allocated.
   if (pNewNode==NULL) return FALSE;

   // Copy data into new node.
   pNewNode->uiMessage=SD->uiMessage;
   pNewNode->idThread=SD->idThread;
   pNewNode->hWnd=SD->hWnd;
   pNewNode->next_node=(MSGNODE*)NULL;
   
   // Append node to chain.
   if (PtrFirstNode==NULL) {
      // Insert as first and only node in chain.
      PtrFirstNode=PtrLastNode=pNewNode;
   } else {
      // Insert at end of chain.
      PtrLastNode->next_node=pNewNode;
      PtrLastNode=pNewNode;
   }
   
   return TRUE;
}



VOID MsgList::UnSubscribe(SUBSCRIBE *SD) {
//
//  Revoves a node from the list.
//

   BOOL bFound;
   MSGNODE *CN, *LN;

   // Find node to be removed.
   bFound=FALSE;
   LN=(MSGNODE*)NULL;
   CN=PtrFirstNode;
   while (CN!=NULL && !bFound) {
      if (CN->uiMessage==SD->uiMessage && CN->idThread==SD->idThread && CN->hWnd==SD->hWnd) {
         bFound=TRUE;
      } else {
         LN=CN;
         CN=CN->next_node;
      }
   }

   // Unlink node from chain.
   if (bFound) {
      if (LN==(MSGNODE*)NULL) {
         // Unlink the first node.
         PtrFirstNode=CN->next_node;
         GlobalFree((HGLOBAL)CN);
         if (PtrFirstNode==(MSGNODE*)NULL) PtrLastNode=(MSGNODE*)NULL;
      } else {
         // Unlink node from the middle or end of list.
         LN->next_node=CN->next_node;
         GlobalFree((HGLOBAL)CN);
         if (LN->next_node==(MSGNODE*)NULL) PtrLastNode=LN;
      }
   }

   return;
}



VOID MsgList::Notify(UINT ThreadMsg, WPARAM wParam, LPARAM lParam) {
//
//  Send notify message to all registered threads/windows.
//

   MSGNODE *CN;

   CN=PtrFirstNode;
   while (CN!=NULL) {
      if (CN->uiMessage==ThreadMsg) {
         // Send message to thread/window.
         if (!CN->idThread) {
            PostMessage(CN->hWnd, ThreadMsg, wParam, lParam);
         } else {
            PostThreadMessage(CN->idThread, ThreadMsg, wParam, lParam);
         }
      }
      CN=CN->next_node;
   }

   return;
}








