#include "stdafx.h"
#include <aygshell.h>

#define WINCE

//#define STRICT
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"


//
// This structure is used to pass startup- 
// information to the serial-reader thread.
//
struct STARTINFO_DEVREADER { 
   HANDLE hComm;              // Handle of communications device 
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *SerialRead;          // The serial-reader thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   DOUBLE TimeCorrection;     // Time correction to apply (in seconds).
   BYTE NumChannels;          // Number of device channels.
   BYTE DataCode;             // The output format type used by the device.
   WORD SampleRate;           // Device output sample-rate.
   TCHAR BaudRate[6];          // Communication speed.
   BYTE SyncStatus;           // The current status of the time syncronization thread.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
};


//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;             // An identifier for this particular model.
   BYTE DeviceModel;          // 3=RD3, 6=RD6.
   TCHAR PortName[6];          // Name of communication port (COMx).
   TCHAR BaudRate[6];          // Communication speed.
   BYTE OutputFormatType;     // Valid values are 1-4.
   WORD SampleRate;           // Device output sample-rate.
   DOUBLE TimeCorrection;     // Time correction in seconds.
};


//
// Global variables.
//
extern BYTE DebugLevel;
extern BOOL StartGPSSyncThread;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadNMRDx(PVOID);
BOOL CALLBACK ConfigDlgProcNMRDx(HWND, UINT, WPARAM, LPARAM);
BOOL NMRDxBytesOK(PBYTE, WORD);



/* *********************************************************************** */
/*  Manager thread for device Nanometrics RD3/6. DeviceID=300.             */
/* *********************************************************************** */
DWORD WINAPI NMRDxMngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65], Message[100];
   TCHAR DeviceResources[256], str10[11];
   BYTE Index;
   WORD BlockSize, NumBlocksPrChannel;
   BOOL SerialRead, MemAlloc, ConfigRead;
   HANDLE hReaderThread, hComm;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_DEVREADER StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, ReaderThreadID;
   DWORD ChannelSize, status, BlockNr;
   MSG_PARAM ID, OD;
   MsgList NotifyList;

#ifdef WINCE
   TCHAR CEPortName[7];
#endif
   
   //
   // Perform thread initialization.
   //
   hReaderThread=NULL;                               // The serial-reader thread is not running.
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));  // Copy startup information to local buffer.
   ParentThreadID=SI.ThreadID;                       // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();             // ThreadID for this thread.
   

   //
   // Handle device configuration.
   //
   // Read configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer configuration.
      DeviceConf.DeviceID=300;               // RD3/RD6 uses device-id 300.
      DeviceConf.DeviceModel=3;              // Assume RD3 model.
      wcscpy(DeviceConf.PortName, TEXT("COM1"));   // Assume digitizer on COM2.
      wcscpy(DeviceConf.BaudRate, TEXT("9600"));   // Assume a baud rate of 9600 bps.
      DeviceConf.OutputFormatType=3;         // Assume ouput format type 3.
      DeviceConf.SampleRate=50;              // Assume ouput sample rate of 50 s/s.
      DeviceConf.TimeCorrection=0.0;         // Default time correction (seconds).

      // Write settings to config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   if (DeviceConf.DeviceModel==3) wcscpy(UniqueDeviceName, TEXT("NMRD3 at "));
   if (DeviceConf.DeviceModel==6) wcscpy(UniqueDeviceName, TEXT("NMRD6 at "));
   wcscat(UniqueDeviceName, DeviceConf.PortName);

   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);

   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("Serial port: "));
   wcscat(DeviceResources, DeviceConf.PortName);

   //
   // Calculate blocksize, channelsize and number of blocks pr. channel.
   //
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(DeviceConf.SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;


   // Open the serial port.
   if (!SI.ConfigOnly) {

#ifdef WINCE
	   // Append ':' to the port name
	   wcscpy(CEPortName, DeviceConf.PortName);
	   wcscat(CEPortName, TEXT(":"));
	   hComm=CreateFile(CEPortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#else
	   hComm=CreateFile(DeviceConf.PortName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
#endif
      if (hComm==INVALID_HANDLE_VALUE) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Unable to open communication port "));
         wcscat(Message, DeviceConf.PortName);
         wcscat(Message, TEXT("."));
         WriteSystemLog(Message, 3);
      }
   }

   
   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("Nanometrics Inc"));
   if (DeviceConf.DeviceModel==3) wcscpy(DS.Model, TEXT("RD3"));
   if (DeviceConf.DeviceModel==6) wcscpy(DS.Model, TEXT("RD6"));

   // Insert samplerate.
   DS.SampleRate=DeviceConf.SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=DeviceConf.SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.DeviceModel;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   for (Index=1;Index<=DeviceConf.DeviceModel;Index++) {
      DS.ChannelData[Index-1]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize);
      if (DS.ChannelData[Index-1]==NULL) MemAlloc=FALSE;
   }
   
   // Check that memory allocation succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel>1) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start reading the serial port.
         //
         if (hReaderThread==NULL && hComm!=INVALID_HANDLE_VALUE && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID) {
               // The system clock is syncronized by some thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0; 
            }
            
            // Set up startup information for the serial reader thread routine.
            SerialRead=TRUE;
            StartInfo.hComm=hComm;
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.SerialRead=&SerialRead;
            StartInfo.DS=&DS;
            StartInfo.BlockNr=&BlockNr;
            StartInfo.TimeCorrection=DeviceConf.TimeCorrection;
            StartInfo.NumChannels=DeviceConf.DeviceModel;
            StartInfo.DataCode=DeviceConf.OutputFormatType;
            StartInfo.SampleRate=DeviceConf.SampleRate;
            wcscpy(StartInfo.BaudRate, DeviceConf.BaudRate);
            StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL); 

            // Start the thread.
            hReaderThread=CreateThread(NULL, 0, ReaderThreadNMRDx, (VOID*)&StartInfo, CREATE_SUSPENDED, &ReaderThreadID);
            SetThreadPriority(hReaderThread, THREAD_PRIORITY_ABOVE_NORMAL);            
            ResumeThread(hReaderThread); 
         }

         // Set return values. 
         if (!hReaderThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // reading the serial port. Returns the default return values.
         //
         if (hReaderThread) {
            // Make the serial thread loop exit, thereby stopping the thread.
            SerialRead=FALSE;

            // Wait up to 2 seconds for the thread to stop.
            status=WaitForSingleObject(hReaderThread, 2000);
            
            // If thread did not end, then terminate it.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0);

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID) UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hReaderThread);
            hReaderThread=NULL;
         }
         break;


         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (hReaderThread==NULL) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_NMRDx), NULL, (DLGPROC)ConfigDlgProcNMRDx, (LPARAM)&DevConfDlg);  

            // Save settings if requested.
            if (Status==1) {
               // Save the new configuration.
               DeviceConf=DevConfDlg;

               // Write configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));

               // Create a unique device instance name.
			   if (DeviceConf.DeviceModel==3) wcscpy(UniqueDeviceName, TEXT("NMRD3 at "));
			   if (DeviceConf.DeviceModel==6) wcscpy(UniqueDeviceName, TEXT("NMRD6 at "));
			   wcscat(UniqueDeviceName, DeviceConf.PortName);
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;

         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Communication speed="));
            wcscat(Message, DeviceConf.BaudRate);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Output format type="));
            wsprintf(str10,TEXT("%d"), DeviceConf.OutputFormatType);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Time correction="));
            wsprintf(str10,TEXT("%.3f"), DeviceConf.TimeCorrection);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         break;

         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         
         break;   


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from the time syncronization
         // thread to inform about a change in the syncronization
         // status.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;

         
         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;



         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the device-reader thread.
         if (hReaderThread) {
            SerialRead=FALSE;                                            // Signal the reader thread.
            status=WaitForSingleObject(hReaderThread, 2000);             // Wait for the thread to stop.
            if (status==WAIT_TIMEOUT) TerminateThread(hReaderThread, 0); // If thread wont stop, kill it.
            hReaderThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Close the communication port.
            CloseHandle(hComm);
         }

         // Free memory buffers.
         for (Index=0;Index<DeviceConf.DeviceModel;Index++) {
            if (DS.ChannelData[Index]) LocalFree((HLOCAL)DS.ChannelData[Index]); 
         }

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  // A new block of data has been inserted into buffer.
         // Send Notification to all registered threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;

         case TM_DEVICEREADER_NODATAINPUT:   
         //
         // The device reader has lost data input.
         //
         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_NODATAINPUT, WPARAM(CurrentThreadID), 0);

         // If running in debug mode, write a message to the log.
         // if (DebugLevel) {
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Not receiving data."));
            WriteSystemLog(Message, 2);
         // }
         break;

         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //

         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // If running in debug mode, write a message to the log.
         //if (DebugLevel) {
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Receiving data."));
            WriteSystemLog(Message, 1);
         //}
         break;

         case TM_DEVICEREADER_BADBLOCK:   // A bad block of data was received from the device.
         // Send Notification to all registered threads/windows.
         // The notofication is accompanied with this thread's thread-ID, and the blocknr of the bad block.
         NotifyList.Notify((UINT)TM_DEVICEREADER_BADBLOCK, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);

         // Write a message to the log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Received a bad block from device."));
         WriteSystemLog(Message, 2);
         break;
      }
      
      // If this is requested by the thread who sent the message
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Serial port reader procedure for the Nanometrics RD3/RD6 device.       */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadNMRDx(PVOID StartupStruct)
{
   DCB dcb;
   BOOL DataLost, ReadStatus;
   BYTE Exp1, Exp2, Exp3, Exp4, Exp5, Exp6, SyncStatus;
   BYTE Buffer[5250], *bPtr, TimeQuality, ChNr, str2[3];
   WORD BlockSize, NumChannels, NumBlocksPrChannel;
   WORD DeviceBlockSize, SampleNr, OffsetToExpByte;
   DWORD BytesRead, BlockNr, OffsetCurrentSample, OffsetToBlockData;
   DOUBLE ExpBase, TransferTime;
   COMMTIMEOUTS ct={20, 0, 1200, 0, 0};
   VOID *DataBlocksCh1, *DataBlocksCh2, *DataBlocksCh3;
   VOID *DataBlocksCh4, *DataBlocksCh5, *DataBlocksCh6;
   STARTINFO_DEVREADER *SI;
   SYSTEMTIME SysTime;
   BLOCK_HEADER DBH;
   __int16 Mantissa;
   long int LIValue;
   BOOL BytesOK;
#ifndef WINCE
   TCHAR DevCntrlStr[100];
#endif


   // Copy values from startup structure.
   SI=(STARTINFO_DEVREADER*)StartupStruct;         // Use pointer of correct type.
   NumChannels=SI->DS->NumChannels;                // Number of channels digitizer is using.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block in channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block in channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block in channel 3.
   DataBlocksCh4=SI->DS->ChannelData[3];           // Pointer to first block in channel 4.
   DataBlocksCh5=SI->DS->ChannelData[4];           // Pointer to first block in channel 5.
   DataBlocksCh6=SI->DS->ChannelData[5];           // Pointer to first block in channel 6.

   // Calculate size in bytes of each block sendt from device.
   if (SI->NumChannels==3) DeviceBlockSize=SI->SampleRate*7+24;
	if (SI->NumChannels==6) DeviceBlockSize=SI->SampleRate*14+24;

   // Calculate the the transfer time on the serial port.
   if (!wcscmp(SI->BaudRate, TEXT("9600"))) TransferTime=(DOUBLE)(DeviceBlockSize)/960.0;
   if (!wcscmp(SI->BaudRate, TEXT("14400"))) TransferTime=(DOUBLE)(DeviceBlockSize)/1440.0;
   if (!wcscmp(SI->BaudRate, TEXT("19200"))) TransferTime=(DOUBLE)(DeviceBlockSize)/1920.0;
   if (!wcscmp(SI->BaudRate, TEXT("38400"))) TransferTime=(DOUBLE)(DeviceBlockSize)/3840.0;

   //Initialize the communications parameters. 
   SetupComm(SI->hComm, DeviceBlockSize*3, 0); 
   GetCommState(SI->hComm, &dcb);


#ifdef WINCE
   dcb.BaudRate     = _wtoi(SI->BaudRate);
   dcb.fParity      = TRUE;
   dcb.Parity       = NOPARITY;
   dcb.StopBits     = ONESTOPBIT;
   dcb.ByteSize     = 8;

   dcb.fInX         = FALSE;
   dcb.fOutX        = FALSE;
   dcb.fOutxDsrFlow = FALSE;
   dcb.fOutxCtsFlow = FALSE;  
   dcb.fDtrControl  = DTR_CONTROL_ENABLE;
   dcb.fRtsControl  = RTS_CONTROL_ENABLE;
#else
   wcscpy(DevCntrlStr, TEXT("baud="));
   wcscat(DevCntrlStr, SI->BaudRate);
   wcscat(DevCntrlStr, TEXT(" parity=N data=8 stop=1"));
   BuildCommDCB(DevCntrlStr, &dcb);
#endif

   dcb.fAbortOnError=FALSE; 
   SetCommState(SI->hComm, &dcb);
   SetCommTimeouts(SI->hComm, &ct);

   // Clear the input buffer.
   PurgeComm(SI->hComm, PURGE_RXCLEAR);

   // Choose the correct exponent base
   // used to calculate the samples. 
   ExpBase=8.0;
   if (SI->DataCode==3) ExpBase=10.0;

   // Start reading the serial port.
   BlockNr=0;  // Start filling data into first block. This variable is zero based.
   DataLost=TRUE;
   while(*SI->SerialRead) {
      
      // Try reading SI->DeviceBlockSize number of bytes into buffer.
      ReadStatus=ReadFile(SI->hComm, (LPVOID)Buffer, (DWORD)DeviceBlockSize, &BytesRead, NULL);

      // Process incoming bytes.
      if (BytesRead==DeviceBlockSize) {  // Correct number of bytes was read.
         
         // Get current time/date. This is UTC time.
         GetSystemTime(&SysTime);

         // Apply time correction.
         SubstractFromTime(&SysTime, 2.103+TransferTime);
         if (SI->TimeCorrection>0.0) AddToTime(&SysTime, SI->TimeCorrection);
         if (SI->TimeCorrection<0.0) SubstractFromTime(&SysTime, -SI->TimeCorrection);

         // Check if bytes are ok.
         BytesOK=NMRDxBytesOK(Buffer, DeviceBlockSize);

         // Find the time quality for this block.
         TimeQuality=0;
         if (SysClkSyncThreadID) {
            WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
            SyncStatus=SI->SyncStatus;
            ReleaseMutex(SI->hSyncStatusMutex);
            if (SyncStatus==0) TimeQuality=2;
            if (SyncStatus==1) TimeQuality=1;
            if (SyncStatus==2) TimeQuality=3;
         }

         // Update block headers.
         DBH.BlockID=BlockNr;
         DBH.TimeTag=SysTime;
         DBH.TimeQuality=TimeQuality;
         DBH.NullData=!BytesOK;
         DBH.DataLost=DataLost;
         DBH.EventDetected=FALSE;
         DBH.EventTime=0;
         for (ChNr=0;ChNr<NumChannels;ChNr++) {
            bPtr=((BYTE*)SI->DS->ChannelData[ChNr])+(BlockSize*BlockNr);
            CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         }

         // Calculate offset from start of data buffer, into data part of currrent block.
         OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);


         //
         // Insert sample values into block.
         //
         if (BytesOK) {
            if (NumChannels==3) {
               //
               // This is an RD3 digitizer.
               //
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SI->SampleRate;SampleNr++) {

                  // Get the exponents for all 3 channels.
                  OffsetToExpByte=22+SampleNr*7+6;
                  Exp1=(Buffer[OffsetToExpByte]&48)/16;
                  Exp2=(Buffer[OffsetToExpByte]&12)/4;
                  Exp3=Buffer[OffsetToExpByte]&3;

                  // Process current sample for all 3 channels.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-5];
                  str2[1]=Buffer[OffsetToExpByte-6];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp1));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-3];
                  str2[1]=Buffer[OffsetToExpByte-4];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp2));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-1];
                  str2[1]=Buffer[OffsetToExpByte-2];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp3));
                  *((long int*)bPtr)=LIValue;

                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }

            } else { 
               //
               // This is an RD6 digitizer.
               //
               OffsetCurrentSample=OffsetToBlockData;
               for (SampleNr=0;SampleNr<SI->SampleRate;SampleNr++) {

                  // Get the exponents for the 3 first channels.
                  OffsetToExpByte=22+SampleNr*14+6;
                  Exp1=(Buffer[OffsetToExpByte]&48)/16;
                  Exp2=(Buffer[OffsetToExpByte]&12)/4;
                  Exp3=Buffer[OffsetToExpByte]&3;

                  // Process current sample for the three first channels.
                  bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-5];
                  str2[1]=Buffer[OffsetToExpByte-6];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp1));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-3];
                  str2[1]=Buffer[OffsetToExpByte-4];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp2));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-1];
                  str2[1]=Buffer[OffsetToExpByte-2];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp3));
                  *((long int*)bPtr)=LIValue;

                  // Get the exponents for the 3 last channels.
                  OffsetToExpByte=22+SampleNr*14+13;
                  Exp4=(Buffer[OffsetToExpByte]&48)/16;
                  Exp5=(Buffer[OffsetToExpByte]&12)/4;
                  Exp6=Buffer[OffsetToExpByte]&3;

                  // Process current sample for the three last channels.
                  bPtr=((BYTE*)DataBlocksCh4)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-5];
                  str2[1]=Buffer[OffsetToExpByte-6];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase, (DOUBLE)Exp4));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh5)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-3];
                  str2[1]=Buffer[OffsetToExpByte-4];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp5));
                  *((long int*)bPtr)=LIValue;

                  bPtr=((BYTE*)DataBlocksCh6)+OffsetCurrentSample;
                  str2[0]=Buffer[OffsetToExpByte-1];
                  str2[1]=Buffer[OffsetToExpByte-2];
                  Mantissa=*(__int16*)&str2;
                  LIValue=(long int)(Mantissa*pow(ExpBase,(DOUBLE)Exp6));
                  *((long int*)bPtr)=LIValue;

                  // Move to next sample.
                  OffsetCurrentSample=OffsetCurrentSample+4;
               }
            }
         } else {
            // Correct number of bytes was read, but they don't
            // seem to be OK. Fill the blocks with nulldata.
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh1+OffsetToBlockData), SI->SampleRate*4);
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh2+OffsetToBlockData), SI->SampleRate*4);
            ZeroMemory((PVOID)((PBYTE)DataBlocksCh3+OffsetToBlockData), SI->SampleRate*4);
            if (NumChannels==6) {
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh4+OffsetToBlockData), SI->SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh5+OffsetToBlockData), SI->SampleRate*4);
               ZeroMemory((PVOID)((PBYTE)DataBlocksCh6+OffsetToBlockData), SI->SampleRate*4);
            }

            // Inform parent thread.
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_BADBLOCK, 0, 0); 
         }
   
         // If data stream has just been restored,then inform parent thread.
         if (DataLost) {
            DataLost=FALSE;
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 
         }

         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;
      
      } else {
         // Failed to read correct number of bytes.The digitizer has been
         // disconnected, or another communication error has occured.
         if (DataLost==FALSE) {
            DataLost=TRUE;

            // Signal parent process that .
            PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_NODATAINPUT, 0, 0); 
         }

         // Wait one second and then try a new read.
         if (!ReadStatus) Sleep(1000);
      }
   }
   
   ExitThread(0);
   return 0L;
   
}




/*******************************************************************/
/*    Configuration dialog box procedure for RD3/RD6.              */
/*******************************************************************/
BOOL CALLBACK ConfigDlgProcNMRDx(HWND hWndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WINCE
	SHINITDLGINFO shidi;
#endif
   HWND hwndDlgItem;
   DWORD dwIndex;
   INT Result;
   static DEVICE_CONFIG *DC;
   TCHAR PortNr, Str20[21], Message[100];
   TCHAR Ports[16][6]={TEXT("COM1"), TEXT("COM2"), TEXT("COM3"), TEXT("COM4"),
	                   TEXT("COM5"), TEXT("COM6"), TEXT("COM7"), TEXT("COM8"),
                       TEXT("COM9"), TEXT("COM10"),TEXT("COM11"),TEXT("COM12"),
					   TEXT("COM13"),TEXT("COM14"),TEXT("COM15"),TEXT("COM16")};

   switch(message)  {
      case WM_INITDIALOG:

#ifdef WINCE
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hWndDlg;
	  SHInitDialog(&shidi);		  
#endif		  

      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;
   
      // Initialize the 'Device model' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_MODEL);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("RD3"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->DeviceModel==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("RD6"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 6);
      if (DC->DeviceModel==6) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      
      // Initialize the 'communication port' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_COM_PORT);
      for (PortNr=0;PortNr<16;PortNr++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)Ports[PortNr]);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, PortNr);
         if (!wcscmp(DC->PortName, Ports[PortNr])) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the '...baud-rate...' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_BAUD_RATE);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("9600"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 9600);
      if (!wcscmp(DC->BaudRate, TEXT("9600"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("14400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 14400);
      if (!wcscmp(DC->BaudRate, TEXT("14400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("19200"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 19200);
      if (!wcscmp(DC->BaudRate, TEXT("19200"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("38400"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 38400);
      if (!wcscmp(DC->BaudRate, TEXT("38400"))) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      
      // Initialize the 'Output format type' combo box.
      hwndDlgItem=GetDlgItem(hWndDlg, IDC_OUTPUT_FORMAT);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("1 (IIR filter)"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 1);
      if (DC->OutputFormatType==1) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("2 (IIR&FIR)"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (DC->OutputFormatType==2) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("3 (no filter)"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 3);
      if (DC->OutputFormatType==3) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)TEXT("4 (FIR filter)"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 4);
      if (DC->OutputFormatType==4) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Initialize the 'Output sample rate...' edit box.
      wsprintf(Str20, TEXT("%d"), DC->SampleRate);
      SetDlgItemText(hWndDlg, IDC_SAMPLE_RATE, Str20);

      // Initialize the 'time correction (seconds)' edit box.
      wsprintf(Str20, TEXT("%.3f"), DC->TimeCorrection);
      SetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20);

      // Bring window to front.
      BringWindowToTop(hWndDlg);

      return TRUE;


      case WM_COMMAND:
      switch(wParam) {
         case IDOK:  // User has pressed the OK button.
         // Read current choice from the 'DeviceModel' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_MODEL);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->DeviceModel=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'communication port' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_COM_PORT);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->PortName);

         // Read current choice from the '...baud-rate...' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_BAUD_RATE);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         SendMessage(hwndDlgItem, CB_GETLBTEXT, dwIndex, (LPARAM)&DC->BaudRate);

         // Read current choice from the 'Output format type' combo box.
         hwndDlgItem=GetDlgItem(hWndDlg, IDC_OUTPUT_FORMAT);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->OutputFormatType=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);       

         // Read current choice from the 'Output sample rate...' edit box.
         GetDlgItemText(hWndDlg, IDC_SAMPLE_RATE, Str20, 20);
         Result=swscanf(Str20, TEXT("%d"), &DC->SampleRate);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to an integer value is entered.
            wcscpy(Message, TEXT("The sample rate value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_SAMPLE_RATE));
            SendDlgItemMessage(hWndDlg, IDC_SAMPLE_RATE, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Read current choice from the time correction (seconds) edit box.
         GetDlgItemText(hWndDlg, IDC_TIME_CORR, Str20, 20);
         Result=swscanf(Str20, TEXT("%Lf"), &DC->TimeCorrection);
         if (Result==0 || Result==EOF) {
            // A string that cannot be converted to a float value is entered.
            wcscpy(Message, TEXT("The time correction value is missing or invalid.\n"));
            MessageBox(hWndDlg, Message, TEXT("Seislog message"), MB_OK|MB_ICONERROR);

            // Move the input focus to this field.
            SetFocus(GetDlgItem(hWndDlg, IDC_TIME_CORR));
            SendDlgItemMessage(hWndDlg, IDC_TIME_CORR, EM_SETSEL, 0, -1); 

            // Return without closing.
            return TRUE;            
         }

         // Close the dialog box
         EndDialog(hWndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hWndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}



/*******************************************************************/
/*    Checks if the string from the device looks OK.               */
/*******************************************************************/

BOOL NMRDxBytesOK(PBYTE string, WORD NumBytes)
{
   WORD CheckSum, StrPtr;
   
   // Check that block header has the SOM bytes.
   if (*(string+0)!=0xAA || *(string+1)!=0x16) return FALSE;

   // Calculate 16^2 modulo checksum for all bytes in the string
   // except the SOM bytes and the block footer bytes.
   CheckSum=0;
   for (StrPtr=2;StrPtr<(NumBytes-2);StrPtr++) CheckSum=CheckSum+*(string+StrPtr);

   // Add to the check sum the WORD given in block footer.
   // the checksum should then become zero.
   CheckSum=CheckSum+(*(string+NumBytes-2)*256);
   CheckSum=CheckSum+(*(string+NumBytes-1));
   if (CheckSum) return FALSE;

   // No problems detected, return TRUE.
   return TRUE;
}