//
//
// Author: yvind Natvik 1998.
//
//
//
//
//




#include "stdafx.h"
#include <aygshell.h>

//#define STRICT
#define HEADER_LINE_SIZE 1040
#define SIGNED_SHORT_MIN -32768
#define SIGNED_SHORT_MAX  32767  
#include <windows.h>
#include <stdio.h>
#include "seislog.h"
#include "RingBuffer.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "FifoBuffer.h"
#include "UnformPcFile.h"
#include "process.h"
#include "eventlog.h"
#include "DlgProcs.h"


#define WINCE

//
// Theese are global variables used by functions in this file
//
extern BYTE DebugLevel;


//
// Structures defined and used only in this file.
//
struct RB_FILENAME_DATA    // The info in this structure is used
                           // to create the name of an RB file.
{ 
   DWORD SequenceNr;       // File's sequence number in ringbuffer.
   SYSTEMTIME StartTime;   // Start time of first block in file.
   DWORD TimeSpan;         // The timespan of the file measured in whole seconds.
   BYTE TimeQuality;       // Time quality of the first block.
   TCHAR StationCode[6];    // Station code for this channel.
   TCHAR Component[5];      // Channel component.
};



//
// Prototype definitions of functions defined and used only in this file.
//
BOOL CreateRingBuffer(HINSTANCE, PTCHAR, PTCHAR, RINGBUFFER_CONFIG*, RINGBUFFER_STATUS*);
VOID CalculateRingBufferConfiguration(WORD, WORD, RINGBUFFER_CONFIG*);
BOOL CALLBACK ConfigRngBufDlgProc(HWND, UINT, WPARAM, LPARAM);
VOID CreateShortRBFilename(PTCHAR, DWORD);
VOID CreateRBFileName(RB_FILENAME_DATA*, PTCHAR);
BOOL GetBufferFilename(PTCHAR, DWORD, PTCHAR);
VOID ReadNameInfoFromRBFile(PTCHAR, RB_FILENAME_DATA*);
VOID ConvertLongToShort(PBYTE, DWORD);
BOOL UpdateFileheader(PTCHAR, DWORD, double);
BYTE WriteBufferToFile(RINGBUFFER_CONFIG*, RINGBUFFER_STATUS*, FifoBuffer*,
                       UnformPcFile*, PBYTE, RB_FILENAME_DATA*, PDWORD, PDWORD,
                       PDWORD, PTCHAR, STARTINFO_RINGBUFFER* , double);
BOOL WriteRBFileHeaders(UnformPcFile*, PTCHAR, PTCHAR, PTCHAR, BLOCK_HEADER, double, WORD, DWORD, BYTE);





/**************************************************************************/
/* Ring buffer thread. Thread handles one channel.                        */
/**************************************************************************/
DWORD WINAPI RingBufferThrd(PVOID StartupInfo) 
{
   MSG msg;
   DWORD CurrentThreadID, BytesActuallyWritten, BlocksWritten;
   DWORD CurFileNr, BufferLimitHigh, BlockNr, PrevBytesWritten;
   DWORD Bytes, SamplesWritten;
   HANDLE hEvent;
   MSG_PARAM RP, IP, MsgParams;
   STARTINFO_RINGBUFFER SI;
   BLOCK_HEADER *pBlockHeader;
   RINGBUFFER_STATUS RngBufStat;
   RINGBUFFER_CONFIG RngBufConf;
   SUBSCRIBE SubscrData;
   STARTUPINFO_DLG SIDLG;
   BYTE *pBlock, *pBlockData, WriteStatus;
   TCHAR ConfigFile[MAX_PATH+1], StatusFile[MAX_PATH+1], Message[200];
   TCHAR str6[7], str10[11], CurBuffFile[MAX_PATH+1], UniqueProcName[65];
   TCHAR OldRBFileName[MAX_PATH+1], NewRBFileName[MAX_PATH+1]; 
   TCHAR OldRBFile[MAX_PATH+1], RBFileName[MAX_PATH+1];
   INT Status;
   RB_FILENAME_DATA NameInfo, FirstBlockInfo;
   FifoBuffer *Buffer;
   UnformPcFile *RBFile;
   MsgList NotifyList;
   BOOL MemAllocOK, BufferRunning, BufferExists, FirstBlock, RBConfigReadFromFile;
   double TotalTime, SecondsPrBlock, SampleSizeInMS;
   BYTE BufferLimits[MAX_NUM_OF_CHANNELS]={ 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,
                                            5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,
                                            5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,
                                            5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20 };

   //
   // Initialize some variables.
   //
   CurrentThreadID=GetCurrentThreadId();  // Get and save the Thread-ID for this thread.
   BufferRunning=FALSE;                   // The ring-buffer has not been started yet. 


   //
   // Copy thread's startup information to a local buffer (SI).
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(STARTINFO_RINGBUFFER));


   //
   // Create a unique name for this instance of the ringbuffer process. 
   //
   wcscpy(UniqueProcName, TEXT("Channel"));
   swprintf(str6, TEXT("%02u"), SI.LogicalChNr);
   wcscat(UniqueProcName, str6);
   wcscat(UniqueProcName, TEXT(" ringbuffer"));


   //
   // Register this process in the process database.
   // Also register a dependency with the channel process.
   //
   RegisterProcess(UniqueProcName, 2, CurrentThreadID);
   RegisterDependency(CurrentThreadID, SI.ChnMngrThreadID);


   //
   // Read ring-buffer config-file if it exist. If not, create a default configuration.
   //
   AddPathAndFilename(ConfigFile, SI.ConfigFileDir, TEXT("RingBuffer.cnf"));
   if (ReadFileToMemory(ConfigFile, (PVOID)&RngBufConf, (DWORD)sizeof(RINGBUFFER_CONFIG))) {
      // Config file was succesfully read.
      RBConfigReadFromFile=TRUE;
   } else {
      // Config file is missing or corrupt.
      // Create a default ringbuffer configuration.
      RngBufConf.RngBufSizeInDays=2;
      RngBufConf.FileSizeInMinutes=10;
      RngBufConf.BytesPrSample=4;
      CalculateRingBufferConfiguration(SI.BlockSizeInMS, SI.SamplesPrBlock ,&RngBufConf);
      RBConfigReadFromFile=FALSE;
   }


   //
   // Read ringbuffer status file.
   //
   AddPathAndFilename(StatusFile, SI.RingBufferDir, TEXT("RingBuffer.sts"));
   if (ReadFileToMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS))) {
      // Status file was succesfully read. Assume buffer is already created.
      BufferExists=TRUE;
   } else {
      // Status file is missing or corrupt. Assume that no buffer exist yet.
      BufferExists=FALSE;

      // Write message in the log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Unable to read buffer status-file. Assuming that no buffer exist."));
      WriteSystemLog(Message, 2);
   }


   //
   // If there has been a power loss or abnormal program
   // stop, then try to rescue the last ringbuffer file.
   //
   if (BufferExists && RBConfigReadFromFile && RngBufStat.SampleBytesInFile) {
      // Calculate number of samples previously written to file.
      SamplesWritten=RngBufStat.SampleBytesInFile/RngBufConf.BytesPrSample;
      
      // Calculate timespan for one sample.
      SampleSizeInMS=(double)SI.BlockSizeInMS/(double)SI.SamplesPrBlock;
      
      // Create complete filename/path to existing bufferfile.
      GetBufferFilename(OldRBFileName, RngBufStat.LastFileWritten+1, SI.RingBufferDir);
      AddPathAndFilename(OldRBFile, SI.RingBufferDir, OldRBFileName);
      
      // Update file/channel headers to reflect the true number of samples in file.
      UpdateFileheader(OldRBFile, SamplesWritten, (double)((SamplesWritten*SampleSizeInMS)/1000));

      // Rename the file to the correct name according to it's contents.
      // The file will be opened and the event file header read. The
      // header contains information needed to create the filename.
      ReadNameInfoFromRBFile(OldRBFile, &NameInfo);               // Read and insert time and timespan.
      NameInfo.SequenceNr=RngBufStat.LastFileWritten+1;           // Insert file's sequence number.  
      NameInfo.TimeQuality=4;                                     // Insert time quality (unknown).    
      wcscpy(NameInfo.StationCode, SI.StationCode);               // Insert station code.   
      wcscpy(NameInfo.Component, SI.Component);                   // Insert component.     
      CreateRBFileName(&NameInfo, NewRBFileName);                 // Create the new filename.  
      RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName); // Rename file.

      // Jump to the next file.
      RngBufStat.LastFileWritten++;   // Jump to the next file.
      RngBufStat.SampleBytesInFile=0; // The next file is empty.
      WriteFileFromMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 

      // Write a note in the program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": RB-file '"));
      wcscat(Message, OldRBFileName);
      wcscat(Message, TEXT("' have been rescued."));
      WriteSystemLog(Message, 1);
   }
   
   
   //
   // Allocate memory/create class instances.
   //
   MemAllocOK=TRUE;

   // Create an instance of the FifoBuffer class.
   Buffer = new FifoBuffer;
   if (Buffer==NULL) MemAllocOK=FALSE;
   
   // Initialize the FifoBuffer class. Allocate a 22KB buffer.
   if (!Buffer->InitBuffer(1024*22)) MemAllocOK=FALSE;

   // Create new instance of the UnformPcFile class;
   // This class will allocate a small buffer.
   RBFile = new UnformPcFile(max(SI.SamplesPrBlock*4, HEADER_LINE_SIZE));
   if (RBFile==NULL) MemAllocOK=FALSE;

   // Allocate memory for a small transfer buffer 
   // big enough to hold one block of samples.
   pBlockData=(PBYTE)LocalAlloc(LPTR, (UINT)SI.BlockSize-sizeof(BLOCK_HEADER));
   if (pBlockData==NULL) MemAllocOK=FALSE;

   // Check that all memory/class allocations succseeded.
   if (!MemAllocOK) {
      // Write error message in the program log.
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Dynamic allocation of classes/memory failed."));
      WriteSystemLog(Message, 3);
   }
   

   //
   // If running in debug mode, write a startup message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   // 
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) {
      //
      // If debuglevel is 2 then log all messages recieved.
      //
      if (DebugLevel>1) {
         wcscpy(Message, UniqueProcName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }
      

      //
      // If this message is accompanied by a MSG_PARAM structure,
      // then copy this structure to a local buffer (IP). The message
      // is assumed to be accompanied by a structure if lParam is not 0.
      // The 'TM_BUFFERUPDATED' message is an exception.
      //
      if (msg.message!=TM_BUFFERUPDATED) {
         if (msg.lParam) IP=*((MSG_PARAM*)msg.lParam);
      }
      

      //
      // Initialize return structure parameters to default values.
      //
      ZeroMemory((PVOID)&RP, (DWORD)sizeof(MSG_PARAM));
      RP.idThread=CurrentThreadID;
      RP.bStatus=TRUE; 

      
      //
      // Process the incoming message.
      //
      switch (msg.message) {
         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         // Add window/thread to notify list.
         if (!NotifyList.Subscribe((SUBSCRIBE*)IP.vPointer)) {
            RP.bStatus=FALSE;
            RP.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         // Remove notification subscribtion from list.
         NotifyList.UnSubscribe((SUBSCRIBE*)IP.vPointer);
         break;   

         
         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure/create the ringbuffer.
         // The ringbuffer can only be configured while it is
         // not running.
         //
         if (!BufferRunning) { 
            // Initialize members in the dialogbox startup structure.
            SIDLG.RngBufConf=RngBufConf;
            SIDLG.ChannelNr=SI.ChannelNr;
            SIDLG.BlockSizeInMS=SI.BlockSizeInMS;
            SIDLG.SamplesPrBlock=SI.SamplesPrBlock;
            wcscpy(SIDLG.RingBufferDir, SI.RingBufferDir);

            // Open the 'Ringbuffer configuration' dialog box.
            Status=DialogBoxParam(SI.hInst, MAKEINTRESOURCE(IDD_CNF_RNGBUFF), NULL, (DLGPROC)ConfigRngBufDlgProc, (LPARAM)&SIDLG);  

            if (Status==-1) { // The DialogBoxParam function failed.
               RP.bStatus=FALSE;
               RP.wStatus=2;
            }

            if (Status==0) { // User pressed the 'Cancel' button.
               RP.bStatus=FALSE;
               RP.wStatus=100;
            }

            if (Status==1) { // User pressed the 'Apply' button.
               //
               // Save settings and create buffer.
               //
               // Save the new ringbuffer configuration.
               RngBufConf=SIDLG.RngBufConf;

               // Calculate filesizes/number-of-files etc.
               CalculateRingBufferConfiguration(SI.BlockSizeInMS, SI.SamplesPrBlock ,&RngBufConf);

               // Change cursor to a wait icon.
               SetCursor(LoadCursor(NULL, IDC_WAIT)); 

               // Create the ring-buffer.
               if (CreateRingBuffer(SI.hInst, SI.RingBufferDir, SI.ConfigFileDir, &RngBufConf, &RngBufStat)) {
                  // The buffer was successfully created.
                  BufferExists=TRUE;
               } else {
                  // Creation of ringbuffer failed.
                  // Delete all files that might have been created.
                  ClearDirectory(SI.RingBufferDir);
                  
                  // Indicate that buffer does not exist.
                  BufferExists=FALSE;

                  // Set return parameters.
                  RP.bStatus=FALSE;
                  RP.wStatus=150;
               }
               
               // Restore cursor back to normal.
               SetCursor(LoadCursor(NULL, IDC_ARROW));
            }
         }
         break;



         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start logging data to the ring buffer.
         //
         if (BufferExists && !BufferRunning && MemAllocOK) {
            // Find out which bufferfile should be opened first.
            CurFileNr=RngBufStat.LastFileWritten+1;
            if (CurFileNr>RngBufConf.NumOfFiles) CurFileNr=1;
            
            // Rename the bufferfile to indicate
            // it is currently beeing written to.
            GetBufferFilename(OldRBFileName, CurFileNr, SI.RingBufferDir);    // Get existing name of bufferfile.
            CreateShortRBFilename(NewRBFileName, CurFileNr);                  // Create new name for bufferfile.
            wcscat(NewRBFileName, TEXT("-0000-00-00-0000-00-000000-0-00000-0000"));
            RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName);       // Rename bufferfile.
            
            // Initialize the UnformPcFile class.This will open
            // and initialize the buffer file.
            AddPathAndFilename(CurBuffFile, SI.RingBufferDir, NewRBFileName);
            if (!RBFile->Open(CurBuffFile)) {
                  // Could not open the RB-file.
                  // Write an error message in the log.
                  wcscpy(Message, UniqueProcName);
                  wcscat(Message, TEXT(": Unable to open RB-file "));
                  wcscat(Message, CurBuffFile);
                  wcscat(Message, TEXT(". Ringbuffer operation has been halted.")); 
                  WriteSystemLog(Message, 3);

                  // Send message to all threads who has 
                  // subscribed to the TM_OPERATION_HALTED message.
                  NotifyList.Notify((UINT)TM_OPERATION_HALTED, (WPARAM)CurrentThreadID, 0);
               break;
            }

            // Update internal status flag.
            BufferRunning=TRUE;

            // Initiate some variables.
            FirstBlock=TRUE;
            SamplesWritten=0;       // No data written so far.
            BytesActuallyWritten=0; // No data written so far.
            BlocksWritten=0;        // No blocks written so far.
            SecondsPrBlock=SI.BlockSizeInMS/1000;
            TotalTime=(double)(RngBufConf.BlocksInFile*SecondsPrBlock); // Timespan in seconds for each file.

            // Clear the FIFO buffer.
            Buffer->ClearBuffer();
            
            // Calculate a value for the buffer limit. This number will decide how
            // many bytes can be inserted into the FIFO buffer before we start to
            // write it's contents to the ringbuffer file. Value is in bytes. The 
            // buffer limit should be in the range from 5-20KB. If the maxvalue of 
            // 20KB is adjusted upwards, then the value given in the call to 
            // 'Buffer.InitBuffer(...)' MUST be adjusted upwards accordingly.
            // The buffer limit selected is based on the logical channel number for this thread.
            // Note that this limit is only used the first time the buffer is filled
            // up. After that, the buffer will be used to its full capasity (20kb).
            // This is done to avoid many ringbuffer threads trying to write to the
            // ringbuffer files simultainielsy (which will cause high CPU load peaks).
            BufferLimitHigh=1024*BufferLimits[SI.LogicalChNr-1];
         }
         break;

         

         case TM_STOP:
         //
         // This message is sendt from a higher level thread
         // to stop the logging of samples to the ring-buffer.
         // The contents of the FIFO buffer will be written to
         // one or several RB-files, which will be properly closed.
         //
         if (BufferRunning) {
            // Update internal status flag.
            BufferRunning=FALSE;

            //
            // Write the FIFO buffer contents to ringbuffer file before closing.
            //
            WriteStatus=0;
            while (WriteStatus!=2 && WriteStatus!=4) {  // Write until the buffer is emptied.
               WriteStatus=WriteBufferToFile(&RngBufConf, &RngBufStat, Buffer, RBFile, pBlockData,
                                             &FirstBlockInfo, &BlocksWritten, &SamplesWritten,
                                             &BytesActuallyWritten, StatusFile, &SI, TotalTime);

               // WriteStatus=1 - The file fills up.
               // WriteStatus=2 - The buffer goes empty.
               // WriteStatus=3 - We encounter a data lost situation. 
               // WriteStatus=4 - An error occured while writing to the RB-file.

               if (WriteStatus==4) {
                  //
                  // Something whent wrong. Stop the ringbuffer 
                  // operation and write error message in the log.
                  //
                  BufferRunning=FALSE;
                  RBFile->Close();

                  // Update the bufferstatus file.
                  RngBufStat.LastFileWritten=CurFileNr-1;
                  RngBufStat.SampleBytesInFile=0;
                  WriteFileFromMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 

                  // Write an error message in the log.
                  wcscpy(Message, UniqueProcName);
                  wcscat(Message, TEXT(": An error occurred while writing to RB-file "));
                  wcscat(Message, CurBuffFile);
                  wcscat(Message, TEXT(". Ringbuffer operation has been halted.")); 
                  WriteSystemLog(Message, 3);

                  // Send message to all threads who has subscribed to this message.
                  NotifyList.Notify((UINT)TM_OPERATION_HALTED, (WPARAM)CurrentThreadID, 0);

               } else {

                  // Close the buffer file.
                  RBFile->WriteLeftovers(&Bytes);
                  RBFile->Close();

                  // Update file/channel headers to reflect the
                  // true number of samples in file if needed.
                  if (SamplesWritten<RngBufConf.SamplesInFile) {
                     UpdateFileheader(CurBuffFile, SamplesWritten, (double)(BlocksWritten*SecondsPrBlock));
         
                     // If running in debug mode, write a message to the log.
                     if (DebugLevel) {
                        CreateShortRBFilename(str6, CurFileNr);
                        wcscpy(Message, UniqueProcName);
                        wcscat(Message, TEXT(": Closing RB file '"));
                        wcscat(Message, str6);
                        wcscat(Message, TEXT("' prematurely."));
                        WriteSystemLog(Message, 1);
                     }

                  }

                  // Rename the file to it's correct name.
                  CreateShortRBFilename(OldRBFileName, CurFileNr);               // Create filename of existing file.
                  wcscat(OldRBFileName, TEXT("-0000-00-00-0000-00-000000-0-00000-0000"));
                  FirstBlockInfo.SequenceNr=CurFileNr;                           // Insert file's sequence number.
                  FirstBlockInfo.TimeSpan=(DWORD)(BlocksWritten*SecondsPrBlock); // Insert file's time span.
                  CreateRBFileName(&FirstBlockInfo, NewRBFileName);              // Create the new filename.
                  RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName);    // Rename file.
                  wcscpy(RBFileName, NewRBFileName);

                  // Update the bufferstatus file.
                  RngBufStat.LastFileWritten=CurFileNr;
                  RngBufStat.SampleBytesInFile=0;
                  WriteFileFromMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 

                  // If there are still more data to be written, open the next file.
                  if (WriteStatus!=2) {

                     // Find filenr for next file.
                     CurFileNr++;
                     if (CurFileNr>RngBufConf.NumOfFiles) CurFileNr=1;

                     // Create a temporary filename for next buffer
                     // file and rename it to this temporary name.
                     GetBufferFilename(OldRBFileName, CurFileNr, SI.RingBufferDir);
                     CreateShortRBFilename(NewRBFileName, CurFileNr);
                     wcscat(NewRBFileName, TEXT("-0000-00-00-0000-00-000000-0-00000-0000"));
                     RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName);


                     // Open and initialize the next file.
                     AddPathAndFilename(CurBuffFile, SI.RingBufferDir, NewRBFileName);
                     RBFile->Open(CurBuffFile);

                     // So far, nothing is written to the file.
                     SamplesWritten=0;
                     BytesActuallyWritten=0;
                     BlocksWritten=0;

                     // If running in debug mode, write a message to the log.
                     if (DebugLevel) {
                        wcscpy(Message, UniqueProcName);
                        wcscat(Message, TEXT(": Finished writing '"));
                        wcscat(Message, RBFileName);
                        wcscat(Message, TEXT("', jumping to next RB file."));
                        WriteSystemLog(Message, 1);
                     }
                  }
               }
            }
         }
         break;
         
         
         case TM_BUFFERUPDATED:
         //
         // The channel-thread indicates that a new block of data is ready.
         //
         if (BufferRunning) {
            // Get the block number for the new block.
            BlockNr=(DWORD)msg.lParam;

            // Create pointers to the block's header.
            pBlock=(PBYTE)SI.pMemBuffer+(BlockNr*SI.BlockSize);
            pBlockHeader=(BLOCK_HEADER*)pBlock;

            // Insert complete block into FIFO buffer.
            Buffer->InsertBytes((DWORD)SI.BlockSize, pBlock);

            //
            // Check the status of the FIFO buffer.
            // If the buffer is full, then the contents
            // must be written to ring-buffer file(s).
            //

            // See if the buffer limit has been reached.
            if (Buffer->GetNumBytes()>=BufferLimitHigh) {
               // Buffer limit has been reached. Write FIFO buffer to ringbuffer file(s).
               WriteStatus=0;
               while (WriteStatus!=2 && WriteStatus!=4) {
                  PrevBytesWritten=BytesActuallyWritten;
                  WriteStatus=WriteBufferToFile(&RngBufConf, &RngBufStat, Buffer, RBFile, pBlockData,
                                                &FirstBlockInfo, &BlocksWritten, &SamplesWritten,
                                                &BytesActuallyWritten, StatusFile, &SI, TotalTime);
                  
                  // Write until one of 4 conditions are met:
                  //
                  // WriteStatus=1 - Ringbuffer file is full.
                  // WriteStatus=2 - The FIFO buffer is empty.
                  // WriteStatus=3 - The write stopped because of a data lost situation. 
                  // WriteStatus=4 - An error occured while writing to the RB-file.

                  // After the FIFO buffer has been emptied the
                  // first time, use the whole buffer capasity.
                  if (WriteStatus==2) BufferLimitHigh=1024*20;

                  if (WriteStatus==4) {
                     //
                     // Something whent wrong. Stop the ringbuffer 
                     // operation and write error message in the log.
                     //
                     BufferRunning=FALSE;
                     RBFile->Close();

                     // Unsubscribe to the TM_BUFFERUPDATED message.
                     SubscrData.uiMessage=TM_BUFFERUPDATED;
                     SubscrData.idThread=CurrentThreadID;
                     SubscrData.hWnd=(HWND)NULL;
                     MsgParams.vPointer=(PVOID)&SubscrData;
                     hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
                     PostThreadMessage(SI.ChnMngrThreadID, TM_UNSUBSCRIBE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
                     WaitForSingleObject(hEvent, INFINITE);
                     CloseHandle(hEvent);

                     // Update the bufferstatus file.
                     RngBufStat.LastFileWritten=CurFileNr-1;
                     RngBufStat.SampleBytesInFile=0;
                     WriteFileFromMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 

                     // Write an error message in the log.
                     wcscpy(Message, UniqueProcName);
                     wcscat(Message, TEXT(": An error occurred while writing to RB-file "));
                     wcscat(Message, CurBuffFile);
                     wcscat(Message, TEXT(". Ringbuffer operation has been halted.")); 
                     WriteSystemLog(Message, 3);

                     // Send message to all threads who has subscribed to this message.
                     NotifyList.Notify((UINT)TM_OPERATION_HALTED, (WPARAM)CurrentThreadID, 0);
                  }

               
                  //
                  // Signal that bytes have been written to disk.
                  //
                  if (WriteStatus!=4 && BytesActuallyWritten>PrevBytesWritten) {
                     PostThreadMessage(CurrentThreadID, TM_DISK_WRITE, 0, 0);
                  }

                  
                  //
                  // If file is full, or a datalost situation
                  // was detected, then we must jump to the next 
                  // file in the ring-buffer.
                  //
                  if (WriteStatus==1 || WriteStatus==3) {
                     //
                     // This file is done. Close it.
                     // 
                     RBFile->WriteLeftovers(&Bytes);
                     RBFile->Close();

                     // Update file/channel headers to reflect the
                     // true number of samples in file if needed.
                     if (SamplesWritten<RngBufConf.SamplesInFile) {
                        UpdateFileheader(CurBuffFile, SamplesWritten, (double)(BlocksWritten*SecondsPrBlock));

                        // If running in debug mode, write a message to the log.
                        if (DebugLevel) {
                           CreateShortRBFilename(str6, CurFileNr);
                           wcscpy(Message, UniqueProcName);
                           wcscat(Message, TEXT(": Closing RB file '"));
                           wcscat(Message, str6);
                           wcscat(Message, TEXT("' prematurely."));
                           WriteSystemLog(Message, 1);
                        }
                     }

                     // Rename the file to it's correct name.
                     FirstBlockInfo.SequenceNr=CurFileNr;                           // Insert file's sequence number.
                     FirstBlockInfo.TimeSpan=(DWORD)(BlocksWritten*SecondsPrBlock); // Insert file's time span.
                     CreateRBFileName(&FirstBlockInfo, NewRBFileName);              // Create the new filename.
                     CreateShortRBFilename(OldRBFileName, CurFileNr);               // Create filename of existing file.
                     wcscat(OldRBFileName, TEXT("-0000-00-00-0000-00-000000-0-00000-0000"));
                     RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName);    // Rename file.
                     wcscpy(RBFileName, NewRBFileName);                             // Save this name.

                     // Update the bufferstatus file.
                     RngBufStat.LastFileWritten=CurFileNr;
                     RngBufStat.SampleBytesInFile=0;
                     WriteFileFromMemory(StatusFile, (PVOID)&RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 


                     //
                     // Now change to the next file in the buffer.
                     //
                     // Find filenr for next file.
                     CurFileNr++;
                     if (CurFileNr>RngBufConf.NumOfFiles) CurFileNr=1;

                     // Create a temporary filename for next buffer
                     // file and rename it to this temporary name.
                     GetBufferFilename(OldRBFileName, CurFileNr, SI.RingBufferDir);
                     CreateShortRBFilename(NewRBFileName, CurFileNr);
                     wcscat(NewRBFileName, TEXT("-0000-00-00-0000-00-000000-0-00000-0000"));
                     RenameFile(SI.RingBufferDir, OldRBFileName, NewRBFileName);
               
                     // Open and initialize the next file.
                     AddPathAndFilename(CurBuffFile, SI.RingBufferDir, NewRBFileName);
                     RBFile->Open(CurBuffFile);

                     // So far, nothing is written to the file.
                     SamplesWritten=0;
                     BytesActuallyWritten=0;
                     BlocksWritten=0;

                     // Signal that we have jumped to the next rb-file.
                     PostThreadMessage(CurrentThreadID, TM_RB_FILECHANGE, 0, 0);

                     // If running in debug mode, write a message to the log.
                     if (DebugLevel) {
                        wcscpy(Message, UniqueProcName);
                        wcscat(Message, TEXT(": Finished writing '"));
                        wcscat(Message, RBFileName);
                        wcscat(Message, TEXT("', jumping to next RB file."));
                        WriteSystemLog(Message, 1);
                     }
                  }
               }
            }
         }

         // Avoid returning of status values.
         msg.lParam=msg.wParam=NULL;
         break;


         case TM_DISK_WRITE:  
         //
         // Data has just been written to the RB file.
         //
         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_DISK_WRITE, (WPARAM)CurrentThreadID, 0);
         break;


         case TM_RB_FILECHANGE:
         //
         // The ringbuffer has jumped to the next rb-file.
         //
         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_RB_FILECHANGE, (WPARAM)CurrentThreadID, (LPARAM)CurFileNr);
         break;


         case TM_GET_RBFILE:
         //
         // This message is used to get the name of the current rb-file. 
         //
         if (BufferRunning) {
            CreateShortRBFilename(str6, CurFileNr);
            wcscpy(RP.cStatus, str6);
         } else {
            wcscpy(RP.cStatus, TEXT(""));
         }
         break;   



         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;

         
         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //
         if (BufferRunning) {  // Abort exit.
            // Post TM_STOP message to stop the buffer before exiting.
            PostThreadMessage(CurrentThreadID, TM_STOP, 0, 0); 

            // Repost the TM_DEPTHREADS_STOPPED message so that this 
            // thread will receive a new TM_DEPTHREADS_STOPPED message 
            // after having processed TM_STOP message.
            PostThreadMessage(CurrentThreadID, TM_DEPTHREADS_STOPPED, 0, 0); 
         } else {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);

            // Release memory for transfer buffer.
            LocalFree(pBlockData);

            // Delete the FifoBuffer object.
            // This will free up all memory allocated by the class.
            delete Buffer;

            // Delete the UnformPcFile object.
            // This will free up all memory allocated by the class.
            delete RBFile;

            // End the message loop.
            ClearMessageQueue();  // Delete all messages in message queue.
            PostQuitMessage(0);   // Post WM_QUIT message.
         }
         break;
      }
      
      // Return status values, and release calling process.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=RP;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Thread dies here.
   ExitThread(0);
   return 0L;
   
}










/*********************************************************************/
/* Writes the contents of the FIFO buffer to the ringbuffer file.    */
/* Write are abandoned when the buffer empties, the file gets full,  */
/* or we come across a block that indicate that data has been lost.  */
/* Headers are also written at start of each new file.               */
/*********************************************************************/
BYTE WriteBufferToFile(RINGBUFFER_CONFIG *RngBufConf, RINGBUFFER_STATUS *RngBufStat, 
                       FifoBuffer *Buffer, UnformPcFile *RBFile, PBYTE BlockData, 
                       RB_FILENAME_DATA *FirstBlockInfo, PDWORD BlocksWritten, 
                       PDWORD SamplesWritten, PDWORD BytesActuallyWritten, 
                       PTCHAR StatusFile, STARTINFO_RINGBUFFER *SI, double TotalTime)
{
   DWORD BlocksLeft, Bytes;
   BLOCK_HEADER BlockHeader;
   BOOL DataLost;

   //
   // Write samples from FIFO buffer to file. 
   // Bytes will be written to file in chunks of one block pr. write.
   // Write until one of three situations occur:
   //    1 - The file fills up.
   //    2 - The buffer goes empty.
   //    3 - We encounter a data lost situation.
   //
   //

   
   // Find out how many blocks there are still room for in the file.
   BlocksLeft=RngBufConf->BlocksInFile-(*BlocksWritten);

   // Start writing to file.
   DataLost=FALSE;
   while (Buffer->GetNumBytes() && BlocksLeft && !DataLost) {
      // Get next block from FIFO buffer.
      Buffer->RemoveBytes(sizeof(BLOCK_HEADER), (PBYTE)&BlockHeader);      // Get the block header.
      Buffer->RemoveBytes(SI->BlockSize-sizeof(BLOCK_HEADER), BlockData);  // Get the block samples.

      // Check if this is a new file that has not yet been written to.
      // If so, then create and write eventfile headers in start of file,
      // and save information required for the naming of the file.
      if (*BlocksWritten==0) {
         // This is a new file. Write eventfile headers
         if (!WriteRBFileHeaders(RBFile, SI->NetworkName, SI->StationCode, SI->Component, 
                                 BlockHeader, TotalTime, SI->SampleRate, RngBufConf->SamplesInFile, 
                                 RngBufConf->BytesPrSample)) return 4;

         // Save all information needed to create the filename when the file is full.
         FirstBlockInfo->StartTime=BlockHeader.TimeTag;
         FirstBlockInfo->TimeQuality=BlockHeader.TimeQuality;
         wcscpy(FirstBlockInfo->StationCode, SI->StationCode);
         wcscpy(FirstBlockInfo->Component, SI->Component);
      }

      // Check for lost data.
      DataLost=BlockHeader.DataLost;
      if (DataLost && *BlocksWritten==0) DataLost=FALSE;
      if (!DataLost) {
         //
         // Write this block to file.
         //
         // Convert 4-byte samples to 2-byte samples if needed.
         if (RngBufConf->BytesPrSample==2) ConvertLongToShort(BlockData, SI->SamplesPrBlock);
      
         // Write block to ring-buffer file. 
         if (!RBFile->Write(SI->SamplesPrBlock*RngBufConf->BytesPrSample, 128, BlockData, &Bytes)) return 4; 

         // Update number of bytes actually written to file.
         *BytesActuallyWritten=(*BytesActuallyWritten)+Bytes;
      
         // Update the number of blocks written to file.
         *BlocksWritten=(*BlocksWritten)+1;

         // Update the number of samples written to file.
         *SamplesWritten=(*SamplesWritten)+SI->SamplesPrBlock;

         // Calculate how much free room is still left in the file.
         BlocksLeft=RngBufConf->BlocksInFile-(*BlocksWritten);
      } else {
         // This block must go in the next file.
         // Put block back into the buffer, otherwise it will be lost.
         Buffer->InsertBytesFirst(SI->BlockSize-sizeof(BLOCK_HEADER), BlockData);
         Buffer->InsertBytesFirst(sizeof(BLOCK_HEADER), (PBYTE)&BlockHeader);
      }
   }

   // Make sure blocks are written to file right now
   // by flushing internal OS buffers. This is neccesarry
   // for the file-rescue feature to function properly.
   RBFile->Flush();

   // Update bufferstatus file. This file will now hold 
   // the number of sample bytes actually written to disk.
   RngBufStat->SampleBytesInFile=*BytesActuallyWritten;
   WriteFileFromMemory(StatusFile, (PVOID)RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS)); 

   // Return with correct statusvalue.
   if (!BlocksLeft) return 1;
   if (DataLost) return 3;
   return 2;

}






/*********************************************************************/
/* Creates all the files in a ring-buffer directory.                 */
/* All existing files in the directory will be deleted.              */
/*********************************************************************/
BOOL CreateRingBuffer(HINSTANCE hInst, TCHAR *RingBufferDir, 
                      TCHAR *ConfigFileDir, RINGBUFFER_CONFIG *RngBufCnf, 
                      RINGBUFFER_STATUS *RngBufStat) 
{
   WORD Index;
   HANDLE hFile;
   TCHAR FileName[MAX_PATH+1], str6[7];
   HWND hwndDialog;
   PROGBARDLG_STARTUPINFO DlgStartup;
   BOOL Error=FALSE;

   //
   // Create progress bar window. 
   //
   wcscpy(DlgStartup.WindowText, TEXT("Creating ringbuffer."));
   wcscpy(DlgStartup.Static1, TEXT(" ")); 
   wcscpy(DlgStartup.Static2, TEXT("Removing existing buffer..."));  
   wcscpy(DlgStartup.Static3, TEXT(" ")); 
   DlgStartup.ProgBarMaxRange=(WORD)(RngBufCnf->NumOfFiles+2);
   DlgStartup.ProgBarStepIncr=1;
   hwndDialog=CreateDialogParam(hInst, MAKEINTRESOURCE(IDD_PROGBAR), NULL, (DLGPROC)ProgBarDlgProc, (LPARAM)&DlgStartup); 

   
   // 
   // Remove all files in the ring-buffer directory.
   //
   ClearDirectory(RingBufferDir);
   


   //
   // Create ring-buffer files.
   //
   // Change texts in progress bar window.
   wcscpy(DlgStartup.Static2, TEXT("Creating new ringbuffer..."));   
   wcscpy(DlgStartup.Static3, TEXT("Please wait... "));  
   SendMessage(hwndDialog, WM_PROGRESSBARWND_SETTEXTS, 0, (LPARAM)&DlgStartup);

   // Create files.
   for (Index=1;Index<=RngBufCnf->NumOfFiles;Index++) {
      // Create the full filename including path.
      CreateShortRBFilename(str6, Index);
      AddPathAndFilename(FileName, RingBufferDir, str6);

      // Create the file.
      hFile=CreateFile(FileName, GENERIC_READ|GENERIC_WRITE, 0, NULL, 
                       CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_WRITE_THROUGH, NULL);
      
      // Check that the file was created.
      if (hFile==INVALID_HANDLE_VALUE) Error=TRUE;

      // Give file correct size, and then close it.
      if (SetFilePointer(hFile, (LONG)RngBufCnf->FileSize, NULL, FILE_BEGIN)==0xFFFFFFFF) Error=TRUE;
      if (!SetEndOfFile(hFile)) Error=TRUE; 
      CloseHandle(hFile);

      // Update progress bar.
      SendMessage(hwndDialog, WM_STEPPROGRESSBAR, 0, 0);

      // Abort if errors was detected.
      if (Error) {
         // Close progress bar window.
         SendMessage(hwndDialog, WM_CLOSEDIALOG, 0, 0);
         return FALSE;
      }
   }


   //
   // Create the ring-buffer status-file.
   //
   // Initialize structure members.
   RngBufStat->LastFileWritten=0;
   RngBufStat->SampleBytesInFile=0;
   
   // Create the full filename including path.
   AddPathAndFilename(FileName, RingBufferDir, TEXT("RingBuffer.sts"));

   // Write the new status file.
   if (!WriteFileFromMemory(FileName, (PVOID)RngBufStat, (DWORD)sizeof(RINGBUFFER_STATUS))) Error=TRUE; 

   // Update progress bar.
   SendMessage(hwndDialog, WM_STEPPROGRESSBAR, 0, 0);

   // Abort if errors was detected.
   if (Error) {
      // Close progress bar window.
      SendMessage(hwndDialog, WM_CLOSEDIALOG, 0, 0);
      return FALSE;
   }


   //
   // Create the ring-buffer config-file.
   //
   //
   // Create the full filename including path.
   AddPathAndFilename(FileName, ConfigFileDir, TEXT("RingBuffer.cnf"));

   // Remove the old config file.
   DeleteFile(FileName);
   
   // Write the new config file.
   if (!WriteFileFromMemory(FileName, (PVOID)RngBufCnf, (DWORD)sizeof(RINGBUFFER_CONFIG))) Error=TRUE; 

   // Update progress bar.
   SendMessage(hwndDialog, WM_STEPPROGRESSBAR, 0, 0);

   // Close progress bar window.
   SendMessage(hwndDialog, WM_CLOSEDIALOG, 0, 0);

   if (Error) return FALSE;
   return TRUE;
}




/************************************************************************/
/* Calculates and filles out some unknown variables in a                */
/* RINGBUFFER_CONFIG structure. Some members must already               */ 
/* be filled in with valid values before calling this subroutine.       */ 
/* Theese are: FileSizeInMinutes, RngBufSizeInDays and BytesPrSample.   */
/************************************************************************/
VOID CalculateRingBufferConfiguration(WORD BlockSizeInMS, WORD SamplesPrBlock, 
                                      RINGBUFFER_CONFIG *RngBufConf) {

      DWORD NumFortranExtraBytes, DataExtrabytes, SampleBytes;
      double BlockSizeInSeconds;

      // Calculate number of blocks in each file.
      BlockSizeInSeconds=BlockSizeInMS/1000;
      RngBufConf->BlocksInFile=(DWORD)((RngBufConf->FileSizeInMinutes*60)/BlockSizeInSeconds);

      // Calculate number of samples in each file. 
      RngBufConf->SamplesInFile=RngBufConf->BlocksInFile*SamplesPrBlock;

      // Calculate how much space all samples occupies (in bytes). 
      SampleBytes=RngBufConf->SamplesInFile *RngBufConf->BytesPrSample;

      // Calculate number of files in the buffer.
#ifdef WINCE
	  RngBufConf->NumOfFiles=(60*RngBufConf->RngBufSizeInDays)/RngBufConf->FileSizeInMinutes;
#else
      RngBufConf->NumOfFiles=(1440*RngBufConf->RngBufSizeInDays)/RngBufConf->FileSizeInMinutes;
#endif 
      
      // Calculate the size of each file in bytes.
      RngBufConf->FileSize=2000+SampleBytes;
      NumFortranExtraBytes=1+24+16+2;           // This is extrabytes for the headers.
      DataExtrabytes=(SampleBytes/128)*2+2;     // This is extrabytes for the samples.
      NumFortranExtraBytes=NumFortranExtraBytes+DataExtrabytes;
      RngBufConf->FileSize=RngBufConf->FileSize+NumFortranExtraBytes;

}




/*******************************************************************/
/*    Configuration dialog box procedure for ring buffer.          */
/*******************************************************************/
BOOL CALLBACK ConfigRngBufDlgProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   SHINITDLGINFO shidi; // New in PocketPC version

   HWND hwndDlgItem;
   DWORD dwIndex;
   WORD BufferSize, FileSize;
   static RINGBUFFER_CONFIG *RBC;
   static STARTUPINFO_DLG *SIDLG;
   float fValue;
   TCHAR str2[3], str4[5], str15[16], str64[65];

   switch(message)  {
      case WM_INITDIALOG:

	  // Create a Done button and size it. New in PocketPC version.
	  shidi.dwMask = SHIDIM_FLAGS;
	  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
	  shidi.hDlg = hwndDlg;
	  SHInitDialog(&shidi);		  
	


      // Create pointer to the startup structure.
      SIDLG=(STARTUPINFO_DLG*)lParam;

      // Create pointer to the config structure.
      RBC=&SIDLG->RngBufConf;

      // Set the static text.
      GetDlgItemText(hwndDlg, IDC_STATIC1, str64, 64);
      _itow((int)SIDLG->ChannelNr, str2, 10);
      wcscat(str64, str2);
      SetDlgItemText(hwndDlg, IDC_STATIC1, str64);

      // Initialize the 'File size' combo box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
      for (FileSize=1;FileSize<=60;FileSize++) {
         _itow((int)FileSize, str2, 10);
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)str2);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, FileSize);
         if (FileSize==RBC->FileSizeInMinutes) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // Initialize the 'Total buffersize' combo box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
      for (BufferSize=1;BufferSize<=365;BufferSize++) {
         _itow((int)BufferSize, str4, 10);
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)str4);
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, BufferSize);
         if (BufferSize==RBC->RngBufSizeInDays) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
         
      // Initialize the 'Number of bytes pr. sample' combo box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("2"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 2);
      if (RBC->BytesPrSample==2) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)TEXT("4"));
      SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, 4);
      if (RBC->BytesPrSample==4) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);

      // Update statictext numbers.
      PostMessage(hwndDlg, WM_COMMAND, MAKEWPARAM(IDC_COMBO1,LBN_SELCHANGE), 0);

      // Bring window to front.
      BringWindowToTop(hwndDlg);

	  SetForegroundWindow(hwndDlg);

      return TRUE; 


      case WM_COMMAND:
      switch(LOWORD(wParam)) {
         case IDC_COMBO1:
         case IDC_COMBO2:
         case IDC_COMBO3:
         switch (HIWORD(wParam)) {  
            case LBN_SELCHANGE: // User has made a selection.
            // Read current choice from the 'File size' combo box.
            hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
            dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
            RBC->FileSizeInMinutes=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
            // Read current choice from the 'Total buffersize' combo box.
            hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
            dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
            RBC->RngBufSizeInDays=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
            // Read current choice from the 'Number of bytes pr. sample' combo box.
            hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3); 
            dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
            RBC->BytesPrSample=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

            // Calculate filesizes etc.
            CalculateRingBufferConfiguration(SIDLG->BlockSizeInMS, SIDLG->SamplesPrBlock ,RBC);
            
            //
            // Update static texts.
            //
            // Size of each buffer file.
            fValue=(float)RBC->FileSize/1024;
            swprintf(str15, TEXT("%.1f"), fValue);
            wcscat(str15, TEXT(" KB"));
            SetDlgItemText(hwndDlg, IDC_STATIC2, str15);
            // Number of files in buffer.
            _itow((int)RBC->NumOfFiles, str15, 10);
            SetDlgItemText(hwndDlg, IDC_STATIC3, str15);
            // Total buffer size.
            fValue=(float)RBC->FileSize/1024;
            fValue=(fValue*RBC->NumOfFiles)/1024;
            swprintf(str15, TEXT("%.1f"), fValue);
            wcscat(str15, TEXT(" MB"));
            SetDlgItemText(hwndDlg, IDC_STATIC4, str15);
         }
         return TRUE;

         case IDOK:  // User has pressed the Apply button.
         // Read current choice from the 'File size' combo box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         RBC->FileSizeInMinutes=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
   
         // Read current choice from the 'Total buffersize' combo box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         RBC->RngBufSizeInDays=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Number of bytes pr. sample' combo box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3); 
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         RBC->BytesPrSample=(BYTE)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
         
         // Check that the number of files is not to big.
         if (RBC->NumOfFiles>9999) {
            MessageBox(hwndDlg, TEXT("Total number of files in buffer exceeds system limit of 9999 files."), TEXT("Invalid configuration:"),  MB_OK);
            return TRUE;
         }

         // Close the dialog box
         EndDialog(hwndDlg , 1);
         return TRUE;

         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hwndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}


/*******************************************************************************/
/* Creates the short name of a buffer file ('RXXXXX') given a sequence number. */
/*******************************************************************************/
VOID CreateShortRBFilename(TCHAR *str6, DWORD SeqNr)
{
   TCHAR str5[6];

   wcscpy(str6, TEXT("R"));
   swprintf(str5, TEXT("%05u"), SeqNr);
   wcscat(str6, str5);
}



/*******************************************************************************/
/* Creates the full name of a buffer file given the                            */     
/* information in the RB_FILENAME_DATA structure.                              */
/*******************************************************************************/
VOID CreateRBFileName(RB_FILENAME_DATA *NameInfo, PTCHAR LongName)
{
   TCHAR str6[7];
   BYTE Index;

   CreateShortRBFilename(LongName, NameInfo->SequenceNr); // Generate the six first charaters
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%04d"), NameInfo->StartTime.wYear);   
   wcscat(LongName, str6);                   // Add year.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%02d"), NameInfo->StartTime.wMonth);   
   wcscat(LongName, str6);                   // Add month.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%02d"), NameInfo->StartTime.wDay);   
   wcscat(LongName, str6);                   // Add day.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%02d"), NameInfo->StartTime.wHour);   
   wcscat(LongName, str6);                   // Add hour.
   swprintf(str6, TEXT("%02d"), NameInfo->StartTime.wMinute);   
   wcscat(LongName, str6);                   // Add minute.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%02d"), NameInfo->StartTime.wSecond);   
   wcscat(LongName, str6);                   // Add second.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%06d"), NameInfo->TimeSpan);   
   wcscat(LongName, str6);                   // Add timespan (length of data).
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   swprintf(str6, TEXT("%1d"), NameInfo->TimeQuality);   
   wcscat(LongName, str6);                   // Add timequality.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   wcscpy(str6, TEXT("     "));
   for (Index=0;Index<wcslen(NameInfo->StationCode);Index++) str6[Index]=NameInfo->StationCode[Index];
   wcscat(LongName, str6);                   // Add station code.
   wcscat(LongName, TEXT("-"));                    // Add a minus sign.
   wcscat(LongName, NameInfo->Component);    // Add component.
   StrSpace2Underscore(LongName);            // Replace spaces with underscores.
}



/************************************************************************/
/* Given the filenumber of the RB file, and the ringbuffer directory,   */
/* this procedure will return the full name of the ringbuffer file.     */
/************************************************************************/
BOOL GetBufferFilename(PTCHAR LongName, DWORD FileNr, PTCHAR RBDir)
{
   TCHAR FileSpec[MAX_PATH+1], ShortName[7];
   HANDLE SearchHandle;
   WIN32_FIND_DATA FileData;

   // Generate the six first charaters in the filename.
   CreateShortRBFilename(ShortName, FileNr);

   // Build file-search-specification string.
   AddPathAndFilename(FileSpec, RBDir, ShortName);
   wcscat(FileSpec, TEXT("*"));
   
   // Search for file.
   SearchHandle=FindFirstFile(FileSpec, &FileData);
   if (SearchHandle==INVALID_HANDLE_VALUE) {
      // Return if file could not be found.
      FindClose(SearchHandle);
      return FALSE;
   }
   
   // Return full filename.
   wcscpy(LongName, FileData.cFileName);
   
   // Close handle and return.
   FindClose(SearchHandle);
   return TRUE;
}



/********************************************************************/
/* Given the full path to an RB file, this procedure will fill out  */
/* the two fields StartTime and TimeSpan in the NameInfo structure. */
/********************************************************************/
VOID ReadNameInfoFromRBFile(PTCHAR RBFile, RB_FILENAME_DATA *NameInfo)
{
   CHAR line[84], str10[11];
   FLOAT fValue;

   // Read line 1 from the event file header into memory.
   ReadFileToMemory(RBFile, (PVOID)line, 83);

   // Extract the time from the line.
   CopyMemory((PVOID)str10, (PVOID)(line+35), 3);
   str10[3]=0;
   NameInfo->StartTime.wYear=atoi(str10)+1900;
   CopyMemory((PVOID)str10, (PVOID)(line+43), 2);
   str10[2]=0;
   NameInfo->StartTime.wMonth=atoi(str10);
   CopyMemory((PVOID)str10, (PVOID)(line+46), 2);
   str10[2]=0;
   NameInfo->StartTime.wDay=atoi(str10);
   CopyMemory((PVOID)str10, (PVOID)(line+49), 2);
   str10[2]=0;
   NameInfo->StartTime.wHour=atoi(str10);
   CopyMemory((PVOID)str10, (PVOID)(line+52), 2);
   str10[2]=0;
   NameInfo->StartTime.wMinute=atoi(str10);
   CopyMemory((PVOID)str10, (PVOID)(line+55), 6);
   str10[6]=0;
   sscanf(str10, "%f", &fValue);
   NameInfo->StartTime.wSecond=(WORD)fValue;
   NameInfo->StartTime.wMilliseconds=0;

   // Extract timespan from line.
   CopyMemory((PVOID)str10, (PVOID)(line+62), 9);
   str10[9]=0;
   sscanf(str10, "%f", &fValue);
   NameInfo->TimeSpan=(DWORD)fValue;

}





/*********************************************************************/
/* Converts 'NumOfLongs' number of signed long's, to signed int's.   */
/*********************************************************************/
VOID ConvertLongToShort(PBYTE Buffer, DWORD NumOfLongs) 
{
   signed long *LongPtr, LongValue;
   signed short *ShortPtr;
   DWORD index;

   LongPtr=(signed long*)Buffer;
   ShortPtr=(signed short*)Buffer;

   for (index=1;index<=NumOfLongs;index++) {
      LongValue=*LongPtr;
      if (LongValue>SIGNED_SHORT_MAX) LongValue=SIGNED_SHORT_MAX;
      if (LongValue<SIGNED_SHORT_MIN) LongValue=SIGNED_SHORT_MIN; 
      *ShortPtr=(signed short)LongValue;
      LongPtr++;
      ShortPtr++;
   }
}



/**********************************************************************************/
/* Creates and writes the channel headers in a ringbuffer file.                   */
/**********************************************************************************/
BOOL WriteRBFileHeaders(UnformPcFile *RBFile, PTCHAR NetworkName, PTCHAR StationCode,
                        PTCHAR Component, BLOCK_HEADER BlockHeader, double TotalTime,
                        WORD SampleRate, DWORD SamplesInFile, BYTE BytesPrSample)
{
   BYTE line[HEADER_LINE_SIZE], *LinePtr;
   CHAR str20[21];
   DWORD Bytes;
   BOOL WriteStatus;


   //
   // Create and write the first header line.
   //
   WriteStatus=TRUE;

   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);

   

   // Insert network name.
   // The UNICODE string first is converted to a ANSI string.
   CHAR _NetworkName[30];
   wcstombs(_NetworkName, NetworkName, 30);

   LinePtr=line+1;
   CopyMemory((PVOID)LinePtr, (PVOID)_NetworkName, strlen(_NetworkName));

   // Insert number of stations.
   line[32]='1';

   // Insert year.
   sprintf(str20, "%03d", BlockHeader.TimeTag.wYear-1900);
   LinePtr=line+33;
   CopyMemory((PVOID)LinePtr, (PVOID)(str20), 3);

   // Insert day of year.
   LinePtr=line+37;
   sprintf(str20, "%3d", GetDayOfYear(&BlockHeader.TimeTag));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 3);

   // Insert month.
   LinePtr=line+41;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert day.
   LinePtr=line+44;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert hour.
   LinePtr=line+47;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert minute.
   LinePtr=line+50;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert seconds.
   LinePtr=line+53;
   sprintf(str20, "%6.3f", (float)(BlockHeader.TimeTag.wSecond+(BlockHeader.TimeTag.wMilliseconds/1000)));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);

   // Insert total time window.
   LinePtr=line+60;
   sprintf(str20, "%9.3f", TotalTime);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 9);

   // Write the first header line.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;

   //
   // Create and write the second header line. This line should be empty.
   //
   FillMemory((PVOID)line, 80, 32);
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;


   //
   // Create and write the third header line.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);

   
  
   // Insert station code.
   // The UNICODE string first is converted to a ANSI string.
   
   CHAR _StationCode[6];
   wcstombs(_StationCode, StationCode, 6);

  if (strlen(_StationCode)<5) {
      LinePtr=line+1;
      CopyMemory((PVOID)LinePtr, (PVOID)_StationCode, strlen(_StationCode));
   } else {
      LinePtr=line+1;
      CopyMemory((PVOID)LinePtr, (PVOID)_StationCode, 4);
      LinePtr=line+9;
      CopyMemory((PVOID)LinePtr, (PVOID)(_StationCode+4), 1);
   }

   // Insert Component.
   // The UNICODE string first is converted to a ANSI string.
   
   CHAR _Component[5];
   wcstombs(_Component, Component, 5);
   LinePtr=line+5;
   CopyMemory((PVOID)LinePtr, (PVOID)_Component, strlen(_Component));

   // Insert start time relative to eventfile start time.
   LinePtr=line+10;
   CopyMemory((PVOID)LinePtr, (PVOID)"   0.00", 7);

   // Insert station data interval length.
   LinePtr=line+18;
   sprintf(str20, "%8.2f", TotalTime);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 8);

   // Write the third header line.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;


   //
   // Create and write header lines 4-12. These lines are empty.
   //
   FillMemory((PVOID)line, 80, 32);
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 4.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 5.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 6.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 7.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 8.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 9.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 10.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 11.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;   // Write line nr 12.


   //
   // Write channel header. This line is 1040 bytes long.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, HEADER_LINE_SIZE, 32);

   // Insert station code.
   // The UNICODE string first is converted to a ANSI string.

   wcstombs(_StationCode, StationCode, 6);
   LinePtr=line;
   CopyMemory((PVOID)LinePtr, (PVOID)_StationCode, strlen(_StationCode));

   // Insert Component.
   // The UNICODE string first is converted to a ANSI string.
   wcstombs(_Component, Component, 5);
   LinePtr=line+5;
   CopyMemory((PVOID)LinePtr, (PVOID)_Component, strlen(_Component));

   // Insert year.
   sprintf(str20, "%03d", BlockHeader.TimeTag.wYear-1900);
   LinePtr=line+9;
   CopyMemory((PVOID)LinePtr, (PVOID)(str20), 3);

   // Insert day of year.
   LinePtr=line+13;
   sprintf(str20, "%3d", GetDayOfYear(&BlockHeader.TimeTag));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 3);

   // Insert month.
   LinePtr=line+17;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert day.
   LinePtr=line+20;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert hour.
   LinePtr=line+23;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert minute.
   LinePtr=line+26;
   sprintf(str20, "%2d", BlockHeader.TimeTag.wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);

   // Insert timequality factor.
   LinePtr=line+28;
   sprintf(str20, "%1d", (int)BlockHeader.TimeQuality);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 1);

   // Insert seconds.
   LinePtr=line+29;
   sprintf(str20, "%6.3f", (float)(BlockHeader.TimeTag.wSecond+(BlockHeader.TimeTag.wMilliseconds/1000)));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);

   // Insert Sample rate.
   LinePtr=line+36;
   sprintf(str20, "%7.2f", (float)SampleRate);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 7);

   // Insert number of samples.
   LinePtr=line+44;
   sprintf(str20, "%6d", SamplesInFile);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);

   // Insert sample size.
   if (BytesPrSample==2) line[76]='2';
   if (BytesPrSample==4) line[76]='4';

   // Write channel header line.
   if (!RBFile->Write(HEADER_LINE_SIZE, 128, line, &Bytes)) WriteStatus=FALSE;
   if (!RBFile->WriteLeftovers(&Bytes)) WriteStatus=FALSE;

   return WriteStatus;
}









/*********************************************************************/
/* Opens a ringbuffer file and changes the 'total time window' field */
/* and the 'Number of samples' field                                 */
/*********************************************************************/

BOOL UpdateFileheader(PTCHAR FileName, DWORD SamplesInFile, double TotalTime)
{
   HANDLE hFile;
   DWORD BytesWritten, PointerLow;
   CHAR str20[21];

   // Open the file.
   hFile=CreateFile(FileName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile==INVALID_HANDLE_VALUE) return FALSE;  // If CreateFile failed then exit.

   //
   // Update the event file header
   //
   // Update the 'Total time window' field.
   PointerLow=SetFilePointer(hFile, 61, 0, FILE_BEGIN); 
   if (PointerLow == 0xFFFFFFFF) return FALSE;
   sprintf(str20, "%9.3f", TotalTime);
   WriteFile(hFile, str20, 9, &BytesWritten, NULL);
   if (BytesWritten!=9) return FALSE;  // Check that bytes where written.

   // Update the 'Station data interval length' field.
   PointerLow=SetFilePointer(hFile, 184, 0, FILE_BEGIN); 
   if (PointerLow == 0xFFFFFFFF) return FALSE;
   sprintf(str20, "%8.2f", TotalTime);
   WriteFile(hFile, str20, 8, &BytesWritten, NULL);
   if (BytesWritten!=8) return FALSE;  // Check that bytes where written.

   //
   // Update the event file channel header.
   //
   // Update the 'Number of samples' field.
   PointerLow=SetFilePointer(hFile, 1030, 0, FILE_BEGIN); 
   if (PointerLow == 0xFFFFFFFF) return FALSE;
   sprintf(str20, "%6d", SamplesInFile);
   WriteFile(hFile, str20, 6, &BytesWritten, NULL);
   if (BytesWritten!=6) return FALSE;  // Check that bytes where written.

   // Close the file.
   CloseHandle(hFile);

   return TRUE;
}




