#include "stdafx.h"
#include <aygshell.h>  // New in PocketPC version

#define WINCE

//#define STRICT
#define HALFPI       1.570796327
#define PI           3.141592654
#define TREEHALFPI   4.712388981
#define TWOPI        6.283185307
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "seislog.h"
#include "library.h"
#include "msglist.h"
#include "resource.h"
#include "process.h"
#include "eventlog.h"

#include <commctrl.h>

//
// This structure is read from the binary device config file.
//
struct DEVICE_CONFIG {
   WORD DeviceID;             // An identifier for this particular model.
   BYTE NumChannels;          // Can only be 4.
   WORD Frequency1;           // Wave frequency.
   WORD EventInterval1;       // Simulate event every 'EventInterval' second.
   WORD EventLength1;         // How long the event will last (in seconds).
   WORD Frequency2;
   WORD EventInterval2;
   WORD EventLength2;
   WORD Frequency3;
   WORD EventInterval3;
   WORD EventLength3;
   WORD Frequency4;
   WORD EventInterval4;
   WORD EventLength4;
};


//
// This structure is used to pass startup- 
// information to the data-generation thread.
//
struct STARTINFO_GENTHREAD { 
   DWORD ThreadID;            // Thread-ID for parent thread.
   BOOL *DataGen;             // The thread will exit if this becomes FALSE.
   DEVMNGR_INTERF_STRUCT *DS; // Pointer to the exported structure.
   DEVICE_CONFIG *DC;         // Pointer to device Configuration structure.
   DWORD* BlockNr;            // Pointer to variable who are to receive block numbers.
   BYTE SyncStatus;           // The current status of the time syncronization thread.
   HANDLE hSyncStatusMutex;   // Handle to a mutex object used to control access to the
                              // SyncStatus variable in this structure.
};


//
// Global variables.
//
extern BYTE DebugLevel;
extern BOOL StartGPSSyncThread;
extern DWORD SysClkSyncThreadID;


//
// Function prototyping.
//
DWORD WINAPI ReaderThreadNullDev(PVOID);
LRESULT CALLBACK ConfigDlgProcNullDevice(HWND, UINT, WPARAM, LPARAM);
 

// [PocketPC] Function prototyping

BOOL ConfigNullDeviceDialogBox(HINSTANCE, HWND, LPARAM);
BOOL CALLBACK ConfigNullDevicePropSheetProc(HWND, UINT, LPARAM);
BOOL CALLBACK ConfigNullDeviceDlgProc1(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK ConfigNullDeviceDlgProc2(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK ConfigNullDeviceDlgProc3(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK ConfigNullDeviceDlgProc4(HWND, UINT, WPARAM, LPARAM);


HWND hConfigDeviceDlg; // PocketPC





/*************************************************************************/
/* Manager thread for the wave generator. DeviceID=10.                   */
/*************************************************************************/
DWORD WINAPI NullDevMngrThrd(PVOID StartupInfo) 
{
   MSG msg;
   int Status;
   TCHAR ConfigFile[MAX_PATH+1], ChNr, UniqueDeviceName[65];
   TCHAR Message[200], DeviceResources[256], str10[11];
   WORD BlockSize, NumBlocksPrChannel, SampleRate;
   HANDLE hDtaGenThread;
   STARTINFO_DEVMANAGER SI;
   STARTINFO_GENTHREAD StartInfo;
   DEVICE_CONFIG DeviceConf, DevConfDlg;
   DEVMNGR_INTERF_STRUCT DS;
   DWORD ParentThreadID, CurrentThreadID, status;
   DWORD ChannelSize, BlockNr, DtaGenThreadID;
   MSG_PARAM ID, OD;
   MsgList NotifyList;
   BOOL MemAlloc, ConfigRead, DataGen;
   
   //
   // Perform thread initialization
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(SI));   // Copy startup inform_wtoin to local buffer.
   ParentThreadID=SI.ThreadID;                        // ThreadID for parent thread.
   CurrentThreadID=GetCurrentThreadId();              // ThreadID for this thread.
   hDtaGenThread=NULL;                                // Handle to data-gener_wtoin thread.
   

   //
   // Handle device Configuration.
   //
   // Read Configuration file.
   ConfigRead=FALSE;
   wcscpy(ConfigFile, SI.DeviceDir); 
   wcscat(ConfigFile, TEXT("\\device.cnf"));
   if (ExistFile(SI.DeviceDir, TEXT("device.cnf"))) {
      // Read device config from config file.
      ConfigRead=ReadFileToMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }
   if (!ConfigRead) {
      // A valid config file could not be read.
      // Set up a default digitizer Configuration.
      ZeroMemory((PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
      DeviceConf.DeviceID=10;             // WaveGen device uses device-id 10.
      DeviceConf.NumChannels=4;           // 4 channels.
      DeviceConf.Frequency1=1;            // Wave frequency.
      DeviceConf.EventInterval1=0;        // Simulate event every 'EventInterval' second.
      DeviceConf.EventLength1=5;          // How long the event will last (in seconds).
      DeviceConf.Frequency2=2;
      DeviceConf.EventInterval2=0;
      DeviceConf.EventLength2=5;
      DeviceConf.Frequency3=3;
      DeviceConf.EventInterval3=0;
      DeviceConf.EventLength3=5;
      DeviceConf.Frequency4=4;
      DeviceConf.EventInterval4=0;
      DeviceConf.EventLength4=5;
      
      // Write settings to a new config file.
      WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DeviceConf));
   }


   // 
   // Create a unique device instance name.
   //
   wcscpy(UniqueDeviceName, TEXT("Internal Wave-Generator"));

   //
   // Register this process in the process database.
   //
   if (!SI.ConfigOnly) RegisterProcess(UniqueDeviceName, 1, CurrentThreadID);

   //
   // Create device resource string.
   // 
   wcscpy(DeviceResources, TEXT("None"));
   
   //
   // Calculate sample-rate, blocksize, channelsize and number of blocks pr. channel.
   //
   SampleRate=120; // Emulate a 120hz samplerate.
   BlockSize=(WORD)(sizeof(BLOCK_HEADER)+(SampleRate*4));
   NumBlocksPrChannel=60;
   ChannelSize=NumBlocksPrChannel*BlockSize;

   //
   // Initialize the interface structure.
   //
   // Insert manufacturer name and model.
   wcscpy(DS.Manufacturer, TEXT("N/A"));
   wcscpy(DS.Model, TEXT("N/A"));

   // Insert samplerate.
   DS.SampleRate=SampleRate;

   // Insert number of samples in each block.
   DS.SamplesPrBlock=SampleRate;

   // Insert number of channels
   DS.NumChannels=DeviceConf.NumChannels;

   // Insert block size, block header included.
   DS.BlockSize=BlockSize;

   // Insert block size measured in milliseconds.
   DS.BlockSizeInMS=1000;

   // Insert number of blocks pr channel.
   DS.NumBlocksPrChannel=NumBlocksPrChannel;

   // Make sure all pointers to channel data is NULL.
   for (ChNr=0;ChNr<MAX_CHANNELS_PR_DEVICE;ChNr++) DS.ChannelData[ChNr]=(PVOID)NULL;
   
   
   //
   // Allocate memory for the channel buffers, and set up the pointers to the buffers.
   //
   MemAlloc=TRUE;
   if ((DS.ChannelData[0]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if ((DS.ChannelData[1]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if ((DS.ChannelData[2]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;
   if ((DS.ChannelData[3]=(PVOID)LocalAlloc(LPTR, (UINT)ChannelSize))==NULL) MemAlloc=FALSE;

   
   // Check that memory allocationi succeded.
   if (!MemAlloc) {
         // Update program log.
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(":"));
         wcscat(Message, TEXT(" Could not allocate memory."));
         WriteSystemLog(Message, 3);
   }
   

   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }
   

   //
   // Signal that the thread is started.
   //
   SetEvent(SI.StartEvent);


   //
   // Enter the thread message loop. The thread will now start to process messages.
   //
   while (GetMessage(&msg, (HWND)NULL, 0, 0)) 
   { 
   
      // If debuglevel is 2 then log all messages recieved.
      if (DebugLevel==2) {
         TCHAR str10[11];
         wcscpy(Message, UniqueDeviceName);
         wcscat(Message, TEXT(": Recieved message "));
         _itow(msg.message, str10, 10);
         wcscat(Message, str10);
         WriteSystemLog(Message, 1);
      }

      // Save message parameters.
      ZeroMemory((PVOID)&ID, (DWORD)sizeof(ID));
      if (msg.lParam) ID=*((MSG_PARAM*)msg.lParam);
      
      // Initialize return-message parameter to default values.
      ZeroMemory((PVOID)&OD, (DWORD)sizeof(OD));
      OD.bStatus=TRUE;
      OD.idThread=CurrentThreadID;


      // Process message.
      switch (msg.message) {
         case TM_START:                            
         //
         // This message is sendt from a higher level 
         // thread to start generating data.
         //
         if (!hDtaGenThread && MemAlloc && !SI.ConfigOnly) {
            if (SysClkSyncThreadID) {
               // The system clock is syncronized by some thread.
               // Subscribe to the TM_TIMESYNC_STATUS message from thread.
               Subscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, (HWND)NULL);

               // Get the current syncronization status.
               StartInfo.SyncStatus=GetCurrentTimeSyncStatus(SysClkSyncThreadID);
            } else {
               // The system clock is not syncronized and
               // there is no other syncronization possibilities.
               StartInfo.SyncStatus=0; 
            }

            // Set up startup information for the serial reader thread routine.
            DataGen=TRUE;
            StartInfo.ThreadID=CurrentThreadID;
            StartInfo.DataGen=&DataGen;
            StartInfo.DS=&DS;
            StartInfo.DC=&DeviceConf;
            StartInfo.BlockNr=&BlockNr;
            StartInfo.hSyncStatusMutex=CreateMutex(NULL, FALSE, NULL); 
 
            // Start the thread.
            hDtaGenThread=CreateThread(NULL, 0, ReaderThreadNullDev, (VOID*)&StartInfo, CREATE_SUSPENDED, &DtaGenThreadID);
            SetThreadPriority(hDtaGenThread, THREAD_PRIORITY_ABOVE_NORMAL); 
            ResumeThread(hDtaGenThread); 
         }

         // Set return values. 
         if (!hDtaGenThread) {
            OD.bStatus=FALSE;
            OD.wStatus=1;
         }
         break;


         
         case TM_STOP:
         //
         // This message is sendt from a higher level thread to stop
         // generating data. Returns the default return values.
         //
         if (hDtaGenThread) {
            // Make the thread loop exit, thereby stopping the thread.
            DataGen=FALSE;

            // Wait up to 1 second for the thread to stop.
            status=WaitForSingleObject(hDtaGenThread, INFINITE);

            // The thread is now stopped, invalidate the thread handle.
				CloseHandle(hDtaGenThread);
            hDtaGenThread=NULL;

            // Unsubscribe to the TM_TIMESYNC_STATUS message as needed.
            if (SysClkSyncThreadID) UnSubscribe(TM_TIMESYNC_STATUS, SysClkSyncThreadID, CurrentThreadID, NULL);

            // Delete the mutex object.
            CloseHandle(StartInfo.hSyncStatusMutex);
         }
         break;



         case TM_CONFIGURE:            
         //
         // This message is sendt from a higher level thread
         // when it wants to configure the digitizer settings.
         //
         if (!hDtaGenThread) {
            // Let user work on a copy of the current device settings.
            DevConfDlg=DeviceConf;

            // Display the dialog box.
            
#ifdef WINCE
			Status=ConfigNullDeviceDialogBox(SI.hInstance, (HWND)NULL, (LPARAM)&DevConfDlg);
#else
			Status=DialogBoxParam(SI.hInstance, MAKEINTRESOURCE(IDD_CNF_WAVEGEN), (HWND)NULL, (DLGPROC)ConfigDlgProcNullDevice, (LPARAM)&DevConfDlg);    
#endif

            // Save settings if requested.
            if (Status==1) {
               // Save the new Configuration.
               DeviceConf=DevConfDlg;

               // Write Configuration to disk.
               WriteFileFromMemory(ConfigFile, (PVOID)&DeviceConf, (DWORD)sizeof(DEVICE_CONFIG));
            }

            // Set return values. 
            if (Status==0) {
               OD.bStatus=FALSE;
               OD.wStatus=100;
            }
         }
         break;


         case TM_SHOW_CONFIG:
            //
            // When this message is received, dump all Configuration
            // parameters that is not given in the device resource 
            // string to the system log.
            //
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 1 wave frequency="));
            swprintf(str10,TEXT("%d"), DeviceConf.Frequency1);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 1 event interval="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventInterval1);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 1 event length="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventLength1);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 2 wave frequency="));
            swprintf(str10,TEXT("%d"), DeviceConf.Frequency2);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);


            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 2 event interval="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventInterval2);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 2 event length="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventLength2);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 3 wave frequency="));
            swprintf(str10,TEXT("%d"), DeviceConf.Frequency3);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
            
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 3 event interval="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventInterval3);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 3 event length="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventLength3);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 4 wave frequency="));
            swprintf(str10,TEXT("%d"), DeviceConf.Frequency4);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 4 event interval="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventInterval4);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);

            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(" Configuration: Channel 4 event length="));
            swprintf(str10,TEXT("%d"), DeviceConf.EventLength4);
            wcscat(Message, str10);
            WriteSystemLog(Message, 1);
         break;


         case TM_GET_INTERFACE:
         //
         // This message is used to get a pointer to the interface structure.
         //
         OD.vPointer=(PVOID)&DS;
         break;   


         case TM_GET_UNIQUE_DEVICE_NAME:
         //
         // This message is used to get a string, 
         // identifying this unique device instance.
         //
         wcscpy(OD.cStatus, UniqueDeviceName);
         break;   


         case TM_GET_RESOURCE_STRING:
         //
         // This message is used to get a string, 
         // identifying the system resources used by this device.
         //
         wcscpy(OD.cStatus, DeviceResources);
         break;   


         case TM_SUBSCRIBE:
         //
         // This message is sendt from a window or thread
         // Who wants to be informed about an event.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Add window/thread to notify list.
         if (ID.vPointer) if (!NotifyList.Subscribe((SUBSCRIBE*)ID.vPointer)) {
            OD.bStatus=FALSE;
            OD.wStatus=3;
         }
         break;   


         case TM_UNSUBSCRIBE:
         //
         // This message is sendt from a window or thread who
         // wants to remove a notification from the notify list.
         // This message must be accompanied by a MSG_PARAM structure.
         // The MSG_PARAM structure must have a pointer to a valid
         // SUBSCRIBE structure.
         //
         //
         // Remove notification from list.
         if (ID.vPointer) NotifyList.UnSubscribe((SUBSCRIBE*)ID.vPointer);
         break;   


         case TM_TIMESYNC_STATUS:
         //
         // This message is sendt from the time syncroniz_wtoin
         // thread to inform about a change in the syncronization
         // status.
         WaitForSingleObject(StartInfo.hSyncStatusMutex, INFINITE); 
         StartInfo.SyncStatus=(BYTE)msg.wParam;
         ReleaseMutex(StartInfo.hSyncStatusMutex);
         break;


         case TM_EXIT:                 
         //
         // This is a request for the thread to end.
         // Close down all processes that are depandent on this thread.
         // The 'StopDependantThreads' function will return immediatly,
         // and will send a 'TM_DEPTHREADS_STOPPED' message back when
         // all dependant threads have been stopped.
         StopDependantThreads();
         break;


         case TM_DEPTHREADS_STOPPED:
         //
         // This message indicates that all dependent threads have been stopped.
         // The thread will cleanup and exit when this message is recieved.         
         //

         // First stop the data-generation thread.
         if (hDtaGenThread) {
            DataGen=FALSE;                                            // Signal the data-gener_wtoin thread.
            status=WaitForSingleObject(hDtaGenThread, INFINITE);      // Wait for the thread to stop.
            hDtaGenThread=NULL;
         }

         if (!SI.ConfigOnly) {
            // Unregister this process from the process database.
            UnregisterProcess(CurrentThreadID);
         }

         // Free memory buffers.
         if (DS.ChannelData[0]) LocalFree((HLOCAL)DS.ChannelData[0]); 
         if (DS.ChannelData[1]) LocalFree((HLOCAL)DS.ChannelData[1]); 
         if (DS.ChannelData[2]) LocalFree((HLOCAL)DS.ChannelData[2]); 
         if (DS.ChannelData[3]) LocalFree((HLOCAL)DS.ChannelData[3]); 

         // End the message loop.
         ClearMessageQueue(); // Delete all messages in message queue.
         PostQuitMessage(0);  // Post WM_QUIT message.   
         break;
   
         
         //
         // Handle messages from device reader thread.
         //
         case TM_BUFFERUPDATED:  
         //   
         // A new block of data has been inserted into buffer.
         //
         // Send notification to all registered threads/windows.
         // The notification is accompanied with this thread's thread-ID, and the blocknr of the updated block.
         NotifyList.Notify((UINT)TM_BUFFERUPDATED, (WPARAM)CurrentThreadID, (LPARAM)BlockNr);
         break;

         case TM_DEVICEREADER_DATAINPUTOK:   
         //
         // The device reader has regained data input.
         //

         // Send Notification to all registered threads/windows.
         NotifyList.Notify((UINT)TM_DEVICEREADER_DATAINPUTOK, WPARAM(CurrentThreadID), 0);

         // If running in debug mode, write a message to the log.
         if (DebugLevel) {
            wcscpy(Message, UniqueDeviceName);
            wcscat(Message, TEXT(": Receiving data."));
            WriteSystemLog(Message, 1);
         }
         break;
         
      }
      
      // If this is requested by the thread who sent the message,
      // then return status values, and release the caller thread.
      if (((MSG_PARAM*)msg.lParam)!=NULL && msg.wParam!=NULL) {
         *(MSG_PARAM*)msg.lParam=OD;
         SetEvent((HANDLE)msg.wParam);
      }
   } 


   // Free up memeory allocated in the MsgList class object.
   NotifyList.~MsgList();
   
   // If running in debug mode, write a message to the log.
   if (DebugLevel) {
      wcscpy(Message, UniqueDeviceName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }

   // Exit this thread
   ExitThread(0);
   return 0L;
   
}




/* *********************************************************************** */
/*  Data-generation thread for the internal wave generator device.         */
/* *********************************************************************** */
DWORD WINAPI ReaderThreadNullDev(PVOID StartupStruct)
{
   WORD BlockSize, NumBlocksPrChannel, SampleNr;
   WORD SecondCntr1,SecondCntr2,SecondCntr3,SecondCntr4;
   DWORD BlockNr, OffsetCurrentSample, OffsetToBlockData;
   SYSTEMTIME SysTime, OldSysTime;
   DOUBLE Radians1, Radians2, Radians3, Radians4;
   DOUBLE RadiansStep1, RadiansStep2, RadiansStep3, RadiansStep4;
   BYTE *bPtr, TimeQuality, SyncStatus;
   BOOL Event1, Event2, Event3, Event4; 
   VOID *DataBlocksCh1, *DataBlocksCh2;
   VOID *DataBlocksCh3, *DataBlocksCh4;
   BLOCK_HEADER DBH;
   STARTINFO_GENTHREAD *SI;
   long int LIValue;
   extern BOOL StartGPSSyncThread;


   // Copy values from startup structure.
   SI=(STARTINFO_GENTHREAD*)StartupStruct;         // Use pointer of correct type.
   NumBlocksPrChannel=SI->DS->NumBlocksPrChannel;  // Number of 1 sec. blocks in buffer.
   BlockSize=SI->DS->BlockSize;                    // Size of 1 sec. block header included
   DataBlocksCh1=SI->DS->ChannelData[0];           // Pointer to first block channel 1.
   DataBlocksCh2=SI->DS->ChannelData[1];           // Pointer to first block channel 2.
   DataBlocksCh3=SI->DS->ChannelData[2];           // Pointer to first block channel 3.
   DataBlocksCh4=SI->DS->ChannelData[3];           // Pointer to first block channel 4.
    
   // Initialze some variables.
   BlockNr=0;  // Start filling data into first block. This variable is zero based.

   // Get current time/date. This is UTC time.
   GetSystemTime(&OldSysTime);

   // Inform parent thread that we are receiving (creating) data
   PostThreadMessage(SI->ThreadID, TM_DEVICEREADER_DATAINPUTOK, 0, 0); 

   // Prepare some more variables.
   RadiansStep1=(SI->DC->Frequency1*TWOPI)/121.0;
   RadiansStep2=(SI->DC->Frequency2*TWOPI)/121.0;
   RadiansStep3=(SI->DC->Frequency3*TWOPI)/121.0;
   RadiansStep4=(SI->DC->Frequency4*TWOPI)/121.0;
   SecondCntr1=SecondCntr2=SecondCntr3=SecondCntr4=0;
   Event1=Event2=Event3=Event4=FALSE;
   
   // Loop forever until SI->DataGen goes false.
   while(*SI->DataGen) {
      // Get current time/date.
      GetSystemTime(&SysTime);

      // Check if a new second has arrived.
      if (SysTime.wSecond!=OldSysTime.wSecond) {
         //
         // A second has gone since the last gener_wtoin.
         // Generate a new set of data and inform parent thread.
         //

         // Increase second counters.
         SecondCntr1++; SecondCntr2++; SecondCntr3++; SecondCntr4++;

         // Turn on/off events for channel 1.
         if (SI->DC->EventInterval1) {
            if (SecondCntr1==SI->DC->EventInterval1+1) Event1=TRUE;           // Check for event start.
            if (SecondCntr1==SI->DC->EventInterval1+SI->DC->EventLength1+1) { // Check for event stop.
               Event1=FALSE; 
               SecondCntr1=SI->DC->EventLength1+1;
            }
         }

         // Turn on/off events for channel 2.
         if (SI->DC->EventInterval2) {
            if (SecondCntr2==SI->DC->EventInterval2+1) Event2=TRUE;
            if (SecondCntr2==SI->DC->EventInterval2+SI->DC->EventLength2+1) {
               Event2=FALSE; 
               SecondCntr2=SI->DC->EventLength2+1;
            }
         }

         // Turn on/off events for channel 3.
         if (SI->DC->EventInterval3) {
            if (SecondCntr3==SI->DC->EventInterval3+1) Event3=TRUE;
            if (SecondCntr3==SI->DC->EventInterval3+SI->DC->EventLength3+1) {
               Event3=FALSE; 
               SecondCntr3=SI->DC->EventLength3+1;
            }
         }

         // Turn on/off events for channel 4.
         if (SI->DC->EventInterval4) {
            if (SecondCntr4==SI->DC->EventInterval4+1) Event4=TRUE;
            if (SecondCntr4==SI->DC->EventInterval4+SI->DC->EventLength4+1) {
               Event4=FALSE; 
               SecondCntr4=SI->DC->EventLength4+1;
            }
         }


         // Find the time quality for this block.
         TimeQuality=0;
         if (SysClkSyncThreadID) {
            WaitForSingleObject(SI->hSyncStatusMutex, INFINITE); 
            SyncStatus=SI->SyncStatus;
            ReleaseMutex(SI->hSyncStatusMutex);
            if (SyncStatus==0) TimeQuality=2;
            if (SyncStatus==1) TimeQuality=1;
            if (SyncStatus==2) TimeQuality=3;
         }


         // Update block headers.
         DBH.BlockID=BlockNr;
         SysTime.wMilliseconds=0; 
         DBH.TimeTag=OldSysTime;
         DBH.TimeQuality=TimeQuality;
         DBH.NullData=FALSE;
         DBH.DataLost=FALSE;
         DBH.EventDetected=FALSE;
         DBH.EventTime=0;
         bPtr=((BYTE*)DataBlocksCh1)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         bPtr=((BYTE*)DataBlocksCh2)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         bPtr=((BYTE*)DataBlocksCh3)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));
         bPtr=((BYTE*)DataBlocksCh4)+(BlockSize*BlockNr);
         CopyMemory((VOID*)bPtr, (VOID*)&DBH, (DWORD)sizeof(BLOCK_HEADER));


         // Insert sample values into block.
         Radians1=Radians2=Radians3=Radians4=0.0;
         OffsetToBlockData=(BlockNr*BlockSize)+sizeof(BLOCK_HEADER);
         OffsetCurrentSample=OffsetToBlockData;
         for (SampleNr=0;SampleNr<120;SampleNr++) {      // There are 120 samples for each channel.
            // Process sample pointed to by 'SampleNr' for channel 1.
            bPtr=((BYTE*)DataBlocksCh1)+OffsetCurrentSample;
            LIValue=(long int)(DblRound(sin(Radians1)*(10.0)));
            if (Event1) LIValue=LIValue*3;
            *((long int*)bPtr)=LIValue;

            // Process sample pointed to by 'SampleNr' for channel 2.
            bPtr=((BYTE*)DataBlocksCh2)+OffsetCurrentSample;
            LIValue=(long int)(DblRound(sin(Radians2+HALFPI)*(10.0)));
            if (Event2) LIValue=LIValue*3;
            *((long int*)bPtr)=LIValue;

            // Process sample pointed to by 'SampleNr' for channel 3.
            bPtr=((BYTE*)DataBlocksCh3)+OffsetCurrentSample;
            LIValue=(long int)(DblRound(sin(Radians3+PI)*(10.0)));
            if (Event3) LIValue=LIValue*3;
            *((long int*)bPtr)=LIValue;

            // Process sample pointed to by 'SampleNr' for channel 4.
            bPtr=((BYTE*)DataBlocksCh4)+OffsetCurrentSample;
            LIValue=(long int)(DblRound(sin(Radians4+TREEHALFPI)*(10.0)));
            if (Event4) LIValue=LIValue*3;
            *((long int*)bPtr)=LIValue;

            // Increase radian angle.
            Radians1=Radians1+RadiansStep1;
            Radians2=Radians2+RadiansStep2;
            Radians3=Radians3+RadiansStep3;
            Radians4=Radians4+RadiansStep4;

            // Move to next sample.
            OffsetCurrentSample=OffsetCurrentSample+4;
         }


         // Return block nr for updated block.
         *SI->BlockNr=BlockNr;

         // Signal parent process that buffer is updated.
         PostThreadMessage(SI->ThreadID, TM_BUFFERUPDATED, 0, 0); 

         // Advance to the next block.
         BlockNr++; 
         if (BlockNr==NumBlocksPrChannel) BlockNr=0;

         // Save the current time.
         OldSysTime=SysTime;
      }

      // Wait 100 ms.
      Sleep(100);
   }


   ExitThread(0);
   return 0L;
   
}




/*******************************************************************/
/*    Configuration dialog box procedure for this device.          */
/*******************************************************************/
LRESULT CALLBACK ConfigDlgProcNullDevice(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex, Index;
   TCHAR str4[5];
   static DEVICE_CONFIG *DC;
   

   switch(message)  {
      case WM_INITDIALOG:
      // Create pointer to the device config structure.
      DC=(DEVICE_CONFIG*)lParam;

      // 
      // Initialize combo box values for channel 1.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // 
      // Initialize combo box values for channel 2.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO4);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO5);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO6);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

      // 
      // Initialize combo box values for channel 3.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO7);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO8);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO9);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
   
      // 
      // Initialize combo box values for channel 4.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO10);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO11);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO12);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      
      // Bring window to front.
      BringWindowToTop(hwndDlg);
		SetForegroundWindow(hwndDlg);

      return TRUE;


      case WM_COMMAND:
      switch(wParam) {
         case IDOK:  // User has pressed the OK button.
         // 
         // Read current choices for channel 1.
         //
         // Read current choice from the Wave Frequency box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->Frequency1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Simulate event every XX second' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventInterval1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Event should last for XX seconds' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventLength1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // 
         // Read current choices for channel 2.
         //
         // Read current choice from the Wave Frequency box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO4);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->Frequency2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Simulate event every XX second' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO5);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventInterval2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Event should last for XX seconds' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO6);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventLength2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // 
         // Read current choices for channel 3.
         //
         // Read current choice from the Wave Frequency box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO7);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->Frequency3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Simulate event every XX second' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO8);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventInterval3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Event should last for XX seconds' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO9);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventLength3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // 
         // Read current choices for channel 4.
         //
         // Read current choice from the Wave Frequency box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO10);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->Frequency4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Simulate event every XX second' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO11);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventInterval4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

         // Read current choice from the 'Event should last for XX seconds' box.
         hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO12);
         dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
         DC->EventLength4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
         
         // Close the dialog box
         EndDialog(hwndDlg , 1);
         return TRUE;
         
         
         case IDCANCEL:  // User has pressed the Cancel button.
         // Do nothing, just exit.
         EndDialog(hwndDlg , 0);
         return TRUE;      
      }
      return FALSE;

   }
   return FALSE;
}


#ifdef WINCE
//The next five functons are only used in the PocketPC version

/*******************************************************************/
/* Opens the Configure Internal Wavegen property sheet             */
/*******************************************************************/

BOOL ConfigNullDeviceDialogBox(HINSTANCE hInst, HWND hWnd, LPARAM lParam)
{
	PROPSHEETHEADER  psh;    // The entire Property Sheet
	PROPSHEETPAGE    psp[4]; // The property pages
	INT              i;

	// Init page structures with generic information.
	memset (&psp, 0, sizeof (psp));      // Zero out all unused values.

	for (i = 0; i < 4; i++) {
		psp[i].dwSize = sizeof (PROPSHEETPAGE);
		psp[i].dwFlags = PSP_DEFAULT | PSP_PREMATURE;
		psp[i].hInstance = hInst;         // Instance handle where the
	                                      // dialog templates are located
		psp[i].lParam  = lParam; 
	}
	// Now do the page specific stuff.
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_CNF_WAVEGEN_1); // Name of dialog resource for page 1
	psp[0].pfnDlgProc = ConfigNullDeviceDlgProc1;    // Pointer to dialog proc for page 1

	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_CNF_WAVEGEN_2); // Name of dialog resource for page 2
	psp[1].pfnDlgProc = ConfigNullDeviceDlgProc2;    // Pointer to dialog proc for page 2

	psp[2].pszTemplate = MAKEINTRESOURCE(IDD_CNF_WAVEGEN_3); // Name of dialog resource for page 2
	psp[2].pfnDlgProc = ConfigNullDeviceDlgProc3;    // Pointer to dialog proc for page 2

	psp[3].pszTemplate = MAKEINTRESOURCE(IDD_CNF_WAVEGEN_4); // Name of dialog resource for page 2
	psp[3].pfnDlgProc = ConfigNullDeviceDlgProc4;    // Pointer to dialog proc for page 2


	// Init property sheet header structure.
	psh.dwSize = sizeof (PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USECALLBACK;    // We are using templates not handles.
	psh.hwndParent = hWnd;              // Handle of the owner window
	psh.hInstance = hInst;              // Instance handle of the application
	psh.pszCaption = TEXT ("Device configuration");
	psh.nPages = 4;                    // Number of pages
	psh.nStartPage = 0;                  // Index of page to be shown first
	psh.ppsp = psp;                     // Pointer to page structures
	psh.pfnCallback = ConfigNullDevicePropSheetProc;   

	// Create property sheet. This returns when the user dismisses the sheet
	// by tapping OK or the Close button.

	return PropertySheet (&psh);
}


/*******************************************************************/
/* This function is called when the poperty sheet is opened        */
/*******************************************************************/

BOOL CALLBACK ConfigNullDevicePropSheetProc(HWND hwndDlg, UINT uMsg, LPARAM lParam )
{
	
	SHINITDLGINFO shidi;

	switch (uMsg)
	{
		
	case PSCB_INITIALIZED: // Note: This may be the wrong message to do this!
		 
		// Initialize handle to the Property Sheet
		hConfigDeviceDlg = hwndDlg;
	
		// Create a Done button and size it.  
		shidi.dwMask = SHIDIM_FLAGS;
		shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
		shidi.hDlg = hwndDlg;
		SHInitDialog(&shidi);
		
		return TRUE;
	}
	return TRUE;
}


/*******************************************************************/
/* Dialog box procedure for the first property page                */
/* of the Configure Internal Wavegenerator property page           */
/*******************************************************************/

BOOL CALLBACK ConfigNullDeviceDlgProc1(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex, Index;
   TCHAR str4[5];
   static DEVICE_CONFIG *DC;
   

   switch(message)  {
      case WM_INITDIALOG:


      // Create pointer to the device config structure.
     
	  DC=(DEVICE_CONFIG*) ((LPPROPSHEETPAGE)lParam)->lParam;

     // 
      // Initialize combo box values for channel 1.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength1==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

   
      return TRUE;

      case WM_COMMAND:
 		 switch(wParam) {
		 case IDCANCEL:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_CANCEL);
			 return TRUE;
		 case IDOK:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_OK);
			 return TRUE;
		  }
		 return FALSE;

      case WM_NOTIFY:

	  switch (((NMHDR *)lParam)->code)
	  {

		  case PSN_QUERYCANCEL:
			  return TRUE;

		  case PSN_APPLY:
		  {
			  // 
			 // Read current choices for channel 1.
			 //
			 // Read current choice from the Wave Frequency box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO1);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->Frequency1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Simulate event every XX second' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO2);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventInterval1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Event should last for XX seconds' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO3);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventLength1=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hwndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

		  }
		  return FALSE;
	  }
      return FALSE;
   }
   return FALSE;
}



/*******************************************************************/
/* Dialog box procedure for the second property page               */
/* of the Configure Internal Wavegenerator property page           */
/*******************************************************************/
BOOL CALLBACK ConfigNullDeviceDlgProc2(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex, Index;
   TCHAR str4[5];
   static DEVICE_CONFIG *DC;
   

   switch(message)  {
      case WM_INITDIALOG:


      // Create pointer to the device config structure.
     
	  DC=(DEVICE_CONFIG*) ((LPPROPSHEETPAGE)lParam)->lParam;

     // 
      // Initialize combo box values for channel 2.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO4);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO5);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO6);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength2==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }

   
      return TRUE;

      case WM_COMMAND:
 		 switch(wParam) {
		 case IDCANCEL:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_CANCEL);
			 return TRUE;
		  
		 case IDOK:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_OK);
			 return TRUE;
		 }
		 return FALSE;

      case WM_NOTIFY:

	  switch (((NMHDR *)lParam)->code)
	  {

		  case PSN_QUERYCANCEL:
			  return TRUE;

		  case PSN_APPLY:
		  {
			// 
			 // Read current choices for channel 2.
			 //
			 // Read current choice from the Wave Frequency box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO4);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->Frequency2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Simulate event every XX second' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO5);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventInterval2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Event should last for XX seconds' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO6);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventLength2=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hwndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;

		  }
		  return FALSE;
	  }
      return FALSE;
   }
   return FALSE;
}


/*******************************************************************/
/* Dialog box procedure for the third property page                */
/* of the Configure Internal Wavegenerator property page           */
/*******************************************************************/
BOOL CALLBACK ConfigNullDeviceDlgProc3(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex, Index;
   TCHAR str4[5];
   static DEVICE_CONFIG *DC;
   

   switch(message)  {
      case WM_INITDIALOG:


      // Create pointer to the device config structure.
     
	  DC=(DEVICE_CONFIG*) ((LPPROPSHEETPAGE)lParam)->lParam;

       // 
      // Initialize combo box values for channel 3.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO7);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO8);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO9);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength3==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
   
      return TRUE;

      case WM_COMMAND:
 		 switch(wParam) {
		 case IDCANCEL:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_CANCEL);
			 return TRUE;
		 case IDOK:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_OK);
			 return TRUE;
		 
		  }
		 return FALSE;

      case WM_NOTIFY:

	  switch (((NMHDR *)lParam)->code)
	  {

		  case PSN_QUERYCANCEL:
			  return TRUE;

		  case PSN_APPLY:
		  {
			// 
			 // Read current choices for channel 3.
			 //
			 // Read current choice from the Wave Frequency box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO7);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->Frequency3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Simulate event every XX second' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO8);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventInterval3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Event should last for XX seconds' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO9);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventLength3=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hwndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;
		  }
		  return FALSE;
	  }
      return FALSE;
   }
   return FALSE;
}


/*******************************************************************/
/* Dialog box procedure for the fourth property page                */
/* of the Configure Internal Wavegenerator property page           */
/*******************************************************************/
BOOL CALLBACK ConfigNullDeviceDlgProc4(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
   HWND hwndDlgItem;
   DWORD dwIndex, Index;
   TCHAR str4[5];
   static DEVICE_CONFIG *DC;
   

   switch(message)  {
      case WM_INITDIALOG:


      // Create pointer to the device config structure.
     
	  DC=(DEVICE_CONFIG*) ((LPPROPSHEETPAGE)lParam)->lParam;

     // 
      // Initialize combo box values for channel 4.
      //
      // Initialize the Wave Frequency box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO10);
      for (Index=1;Index<=30;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->Frequency4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Simulate event every XX second' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO11);
      for (Index=0;Index<=120;Index=Index+10) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventInterval4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
      // Initialize the 'Event should last for XX seconds' box.
      hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO12);
      for (Index=1;Index<=10;Index++) {
         dwIndex=SendMessage(hwndDlgItem, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)_itow((int)Index, str4, 10));
         SendMessage(hwndDlgItem, CB_SETITEMDATA, dwIndex, Index);
         if (DC->EventLength4==Index) SendMessage(hwndDlgItem, CB_SETCURSEL, dwIndex, 0);
      }
   
      return TRUE;

      case WM_COMMAND:
 		 switch(wParam) {
		 case IDCANCEL:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_CANCEL);
			 return TRUE;
		 case IDOK:
			 PropSheet_PressButton(hConfigDeviceDlg, PSBTN_OK);
			 return TRUE;

		  }
		 return FALSE;

      case WM_NOTIFY:

	  switch (((NMHDR *)lParam)->code)
	  {

		  case PSN_QUERYCANCEL:
			  return TRUE;

		  case PSN_APPLY:
		  {
			 // 
			 // Read current choices for channel 4.
			 //
			 // Read current choice from the Wave Frequency box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO10);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->Frequency4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Simulate event every XX second' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO11);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventInterval4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);

			 // Read current choice from the 'Event should last for XX seconds' box.
			 hwndDlgItem=GetDlgItem(hwndDlg, IDC_COMBO12);
			 dwIndex=SendMessage(hwndDlgItem, CB_GETCURSEL, 0, 0);
			 DC->EventLength4=(WORD)SendMessage(hwndDlgItem, CB_GETITEMDATA, dwIndex, 0);
         
			 // [PocketPC] OK to close the dialog box
			 SetWindowLong (hwndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
			 return TRUE;
		  }
		  return FALSE;
	  }
      return FALSE;
   }
   return FALSE;
}

#endif



