//
//
// Author: yvind Natvik 1998.
//
//
//
//
//

#include "stdafx.h"

//#define STRICT
#define HEADER_LINE_SIZE 1040
#include <windows.h>
#include <stdio.h>
#include "Seislog.h"
#include "WriteEvent.h"
#include "channel.h"
#include "library.h"
#include "Process.h"
#include "EventLog.h"
#include "UnformPcFile.h"


//
// Theese are global variables used by functions in this file
//
extern BYTE DebugLevel;
extern TCHAR NetworkCode[];
extern TCHAR NetworkName[];


//
// Definitions of structures used only by functions in this file.
//
struct CHNINFO {
   TCHAR        StationCode[6];      // Station code name.
   TCHAR        Component[5];        // Component name.
   DOUBLE      RelStartTime;        // Start time relative to event file time.
   DOUBLE      DataIntLength;       // Length in seconds of event.
   SYSTEMTIME  StartTimeFrstBlck;   // Start time for the first block in event.
   BUFF_PROP   BufferProperties;    // Properties of the channel buffer.
   DWORD       NumSamplesInEvent;   // Number of samples in the event.
   DWORD       NumBlocksInEvent;    // Number of blocks in the event.
   LONG        FirstBlockNr;        // The first channel block to be dumped.
   LONG        LastBlockNr;         // The last channel block to be dumped.
   BOOL        SkipChannel;         // If TRUE then this channel will not be dumped.
}; 


//
// Prototype definitions of functions defined and used only in this file..
//
BOOL WriteEventFileHeader(UnformPcFile*, PTCHAR, BYTE, BYTE, SYSTEMTIME*, double, CHNINFO[]);
BOOL WriteChannelHeader(UnformPcFile*, BYTE, CHNINFO[]);
BOOL ConvertComponentCode(PTCHAR);



/**************************************************************************/
/* WriteEvent thread.                                                     */
/**************************************************************************/
DWORD WINAPI WriteEventThrdProc(PVOID StartupInfo) 
{
   DWORD CurrentThreadID, ChannelThreadID, BlockNr;
   DWORD LastBlockWritten, DataBytesPrBlock;
   DWORD NumberOfBytesWritten;
   DOUBLE TotalTimeWindow, duration, SecondsToWait;
   WORD LargestBlockSize;
   BYTE *pBlock, Index, line[83], NumDumpChannels;
   HANDLE hEvent, hFile;
   MSG_PARAM MsgParams;
   TCHAR Message[200], ProcName[65], EFileName[MAX_PATH+1], SFileName[MAX_PATH+1];
   TCHAR EventFile[MAX_PATH+1], SFile[MAX_PATH+1], str31[32], *LinePtr;
   BUFF_PROP BufProp;
   CHNINFO ChnlInfo[MAX_NUM_OF_CHANNELS];
   STARTINFO_WRITEEVENT SI;
   BLOCK_RANGE RangeInfo;
   SYSTEMTIME EventFileTime,CurrentTime;
   UnformPcFile *RBFile;
   BOOL WriteOK;
   
   CHAR _str31[32], *_LinePtr;  // Temporary string and pointer
   //
   // Initialize some variables.
   //
   CurrentThreadID=GetCurrentThreadId();  // Get and save the Thread-ID for this thread.


   //
   // Copy thread's startup information to a local buffer (SI).
   //
   CopyMemory((PVOID)&SI, StartupInfo, sizeof(STARTINFO_WRITEEVENT));


   //
   // Release the memory for the startup information if requested.
   //
   if (SI.ReleaseMemory) LocalFree(StartupInfo);


   //
   // Create a name for the process. 
   //
   wcscpy(ProcName, TEXT("WriteEvent"));
   

   //
   // Register this process in the process database.
   //
   RegisterProcess(ProcName, 6, CurrentThreadID);


   //
   // Register dependency with the dump channels
   //
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      RegisterDependency(CurrentThreadID, SI.DumpChannels[Index]);
   }


   //
   // If running in debug mode, write a startup message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Thread started."));
      WriteSystemLog(Message, 1);
   }


   //
   // The event end-time might not have arrived yet.
   // If so then we must wait for end-time to come
   // before we can continue. This is usually the
   // case if the user has set a post event time 
   // greater then zero. After the wait, all the 
   // needed samples should hopefully be present
   // in the channel buffers and ready to be dumped.
   //
   GetSystemTime(&CurrentTime);
   if (TimeCmp(&CurrentTime, &SI.EventStopTime)==-1) {
      SecondsToWait=TimeDiff(&CurrentTime, &SI.EventStopTime);
      SecondsToWait=SecondsToWait+1.5;
      Sleep(1000*(DWORD)SecondsToWait);
   }


   //
   // For all dump channels, find the range of
   // blocks corresponding as close as possible
   // to the given event window. This code section
   // will initialize the following members of the
   // ChnlInfo table: 
   //             StartTimeFrstBlck
   //             FirstBlockNr
   //             LastBlockNr
   //             NumBlocksInEvent
   //             SkipChannel
   //
   NumDumpChannels=0;
   ZeroMemory((PVOID)&ChnlInfo, (DWORD)sizeof(ChnlInfo));
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      // Get the start an stop block and the corresponding times.
      RangeInfo.StartTime=SI.EventStartTime;
      RangeInfo.EndTime=SI.EventStopTime;
      MsgParams.vPointer=(VOID*)&RangeInfo;
      ChannelThreadID=SI.DumpChannels[Index];
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(ChannelThreadID, TM_GET_BLOCK_RANGE, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);

      // Save returned values if the TM_GET_BLOCK_RANGE message succeeded.
      if (MsgParams.bStatus) {
         NumDumpChannels++;
         ChnlInfo[Index].StartTimeFrstBlck=RangeInfo.StartTime;
         ChnlInfo[Index].FirstBlockNr=RangeInfo.FirstBlock;
         ChnlInfo[Index].LastBlockNr=RangeInfo.LastBlock;
         ChnlInfo[Index].NumBlocksInEvent=RangeInfo.NumBlocksInRange;
         ChnlInfo[Index].SkipChannel=FALSE;
      } else {
         ChnlInfo[Index].SkipChannel=TRUE;
      }
   }


   //
   // If no channels can be dumped then exit this thread here.
   //
   if (!NumDumpChannels) {
      UnregisterProcess(CurrentThreadID);
      ExitThread(0);
   }
   
  
   //
   // Initialize the rest of the fields in the ChnlInfo
   // table except the 'RelStartTime' field. 
   //
   hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      if (!ChnlInfo[Index].SkipChannel) {
         // Get thread-id for the thread running this channel.
         ChannelThreadID=SI.DumpChannels[Index];
      
         // Insert Station Code.
         PostThreadMessage(ChannelThreadID, TM_GET_STATION, (WPARAM)hEvent, (LPARAM)&MsgParams); 
         WaitForSingleObject(hEvent, INFINITE);
         wcscpy(ChnlInfo[Index].StationCode, MsgParams.cStatus);
      
         // Insert Component.
         PostThreadMessage(ChannelThreadID, TM_GET_COMPONENT, (WPARAM)hEvent, (LPARAM)&MsgParams); 
         WaitForSingleObject(hEvent, INFINITE);
         wcscpy(ChnlInfo[Index].Component, MsgParams.cStatus);
      
         // Insert buffer properties of channel buffer.
         MsgParams.vPointer=&BufProp;
         PostThreadMessage(ChannelThreadID, TM_GET_BUFFER_PROPERTIES, (WPARAM)hEvent, (LPARAM)&MsgParams); 
         WaitForSingleObject(hEvent, INFINITE);
         ChnlInfo[Index].BufferProperties=BufProp;

         // Calculate and insert number of samples in event.
         ChnlInfo[Index].NumSamplesInEvent=ChnlInfo[Index].NumBlocksInEvent*BufProp.SamplesPrBlock;
      
         // Calculate station data interval length.
         ChnlInfo[Index].DataIntLength=(DOUBLE)(ChnlInfo[Index].NumBlocksInEvent*BufProp.BlockSizeInMS)/1000.0;
      }
   }
   CloseHandle(hEvent);

  
   //
   // Find the event file time.
   //
   EventFileTime=SI.EventStartTime;
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      if (!ChnlInfo[Index].SkipChannel) {
         if (TimeCmp(&ChnlInfo[Index].StartTimeFrstBlck, &EventFileTime)==-1) {
            EventFileTime=ChnlInfo[Index].StartTimeFrstBlck;
         }
      }
   }
   
   
   //
   // Now that we have found the event file time, we can calculate and 
   // insert the values for the 'RelStartTime' field in the ChnlInfo table.
   // Also find the total timewindow that all channels span over.
   //
   TotalTimeWindow=0.0;
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      if (!ChnlInfo[Index].SkipChannel) {
         ChnlInfo[Index].RelStartTime=TimeDiff(&EventFileTime, &ChnlInfo[Index].StartTimeFrstBlck);
         if (ChnlInfo[Index].RelStartTime+ChnlInfo[Index].DataIntLength > TotalTimeWindow) {
            TotalTimeWindow=ChnlInfo[Index].RelStartTime+ChnlInfo[Index].DataIntLength;
         }
      }
   }
   
      
   //
   // Create new instance of the UnformPcFile class;
   // This class will allocate a small buffer.
   //
   // First scan trough the table and find the largest block size (in bytes).
   LargestBlockSize=0;
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      if (!ChnlInfo[Index].SkipChannel) {
         if (ChnlInfo[Index].BufferProperties.BlocksizeInBytes>LargestBlockSize) {
            LargestBlockSize=ChnlInfo[Index].BufferProperties.BlocksizeInBytes;
         }
      }
   }
   
   // Create the class instance
   RBFile = new UnformPcFile(max(LargestBlockSize, HEADER_LINE_SIZE));
   if (RBFile==NULL) {
      // Write message to the system log.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Memory allocation failed. Thread will terminate."));
      WriteSystemLog(Message, 3);
      
      // End this thread.
      ExitThread(1);
      return 1;
   }
   
   
   //
   // Create the name of the event file.
   //
   LinePtr=EFileName;
   swprintf(str31, TEXT("%4d"), EventFileTime.wYear);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 4*sizeof(TCHAR));       // Insert year.

   LinePtr=EFileName+4;
   *LinePtr='-';                                                    // Insert '-'.
   
   LinePtr=EFileName+5;
   swprintf(str31, TEXT("%02d"), EventFileTime.wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert month.

   LinePtr=EFileName+7;
   *LinePtr='-';                                                    // Insert '-'.
   
   LinePtr=EFileName+8;
   swprintf(str31, TEXT("%02d"), EventFileTime.wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert day.

   LinePtr=EFileName+10;
   *LinePtr='-';                                      // Insert '-'.
   
   LinePtr=EFileName+11;
   swprintf(str31, TEXT("%02d"), EventFileTime.wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert hour.

   LinePtr=EFileName+13;
   swprintf(str31, TEXT("%02d"), EventFileTime.wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert minute.

   LinePtr=EFileName+15;
   *LinePtr='-';                                      // Insert '-'.

   LinePtr=EFileName+16;
   swprintf(str31, TEXT("%02d"), EventFileTime.wSecond);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert second.

   LinePtr=EFileName+18;
   CopyMemory((PVOID)LinePtr, (PVOID)TEXT("S."), 2*sizeof(TCHAR));        // Insert 'S.'.
   
   // Insert network code.
   // This field must be 5 characters long.
   // Spaces should be replaced with underscores.
   LinePtr=EFileName+20;
   wcscpy(str31, TEXT("     "));
   for (Index=0;Index<wcslen(NetworkCode);Index++) str31[Index]=NetworkCode[Index];
   StrSpace2Underscore(str31);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 5*sizeof(TCHAR)); 

   LinePtr=EFileName+25;
   *LinePtr='_';                                      // Insert '_'.

   LinePtr=LinePtr+1;
   swprintf(str31, TEXT("%03d"), NumDumpChannels);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 3*sizeof(TCHAR));       // Insert number of channels.

   LinePtr=LinePtr+3;
   *LinePtr=0;                                        // Terminate filename string.

   
   
   // Try to create a new file with this name.
   AddPathAndFilename(EventFile, SI.EFileDir, EFileName);
   hFile=CreateFile(EventFile, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile==INVALID_HANDLE_VALUE) {
      // The creation of the file failed.
      // A file with this name probably already exist.
      // Add a letter ranging from A-Z to the end of 
      // the file name and try creating the file again.
      Index=wcslen(EFileName);
      wcscat(EFileName, TEXT(" "));
      EFileName[Index]=('A'-1);
      do {
         EFileName[Index]++;
         AddPathAndFilename(EventFile, SI.EFileDir, EFileName);
         hFile=CreateFile(EventFile, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
      } while (hFile==INVALID_HANDLE_VALUE && EFileName[Index]!='Z');
   }
   if (hFile==INVALID_HANDLE_VALUE) {
      // Could not create the RB-file. 
      // Write an error message in the log and exit.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Unable to create event file '"));
      wcscat(Message, EventFile);
      wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
      WriteSystemLog(Message, 3);
      delete RBFile;    // Free the class instance.
      ExitThread(1);    // Exit the thread.
   } else {
      CloseHandle(hFile);
   }
   
   
   
   //
   // Open the event file.
   //
   if (!RBFile->Open(EventFile)) {
      // Could not open the event file. 
      // Write an error message in the log.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Unable to open event file '"));
      wcscat(Message, EventFile);
      wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
      WriteSystemLog(Message, 3);
      delete RBFile;    // Free the class instance.
      ExitThread(1);    // Exit the thread.
   }
   
   
   //
   // Write the event file header to the event file.
   //
   WriteOK=WriteEventFileHeader(RBFile, NetworkName, SI.NumOfDFileChannels, NumDumpChannels, 
                                                   &EventFileTime, TotalTimeWindow, ChnlInfo);
   if (!WriteOK) {
      // An error occurred during the write.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": An error occured while writing the event file '"));
      wcscat(Message, EventFile);
      wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
      WriteSystemLog(Message, 3);
      RBFile->Close();        // Close the file.
      DeleteFile(EventFile);  // Delete the file.
      delete RBFile;          // Free the class instance.
      ExitThread(1);          // Exit the thread.
   }
   
   
   //
   // Write eventfile channel headers and data.
   // 
   for (Index=0;Index<SI.NumOfDFileChannels;Index++) {
      if (!ChnlInfo[Index].SkipChannel) {      
         //
         // Write channel header for this channel.
         WriteOK=WriteChannelHeader(RBFile, Index, ChnlInfo);
      
         //
         // Write channel data.
         if (WriteOK) {
            // Write all blocks in the event.
            BlockNr=ChnlInfo[Index].FirstBlockNr;
            DataBytesPrBlock=(DWORD)(ChnlInfo[Index].BufferProperties.SamplesPrBlock*4);
            do {
               // Create pointer to the block samples.
               pBlock=(PBYTE)ChnlInfo[Index].BufferProperties.pBuffer;   
               pBlock=pBlock + BlockNr*ChnlInfo[Index].BufferProperties.BlocksizeInBytes;
               pBlock=pBlock + sizeof(BLOCK_HEADER);
            
               // Write block to event file.
               WriteOK=RBFile->Write(DataBytesPrBlock, 128, pBlock, NULL);
               LastBlockWritten=BlockNr;
            
               // Jump to the next block.
               BlockNr++;
               if (BlockNr==ChnlInfo[Index].BufferProperties.NumBlocksInBuffer) BlockNr=0;
            
            } while (LastBlockWritten!=(DWORD)ChnlInfo[Index].LastBlockNr);
         
            // Write any leftover bytes in the class.
            RBFile->WriteLeftovers(NULL);
         }
      
         //
         // Check that writing was successful.
         if (!WriteOK) {
            // An error occurred during the write.
            wcscpy(Message, ProcName);
            wcscat(Message, TEXT(": An error occured while writing the event file '"));
            wcscat(Message, EventFile);
            wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
            WriteSystemLog(Message, 3);
            RBFile->Close();        // Close the file.
            DeleteFile(EventFile);  // Delete the file.
            delete RBFile;          // Free the class instance.
            ExitThread(1);          // Exit the thread.
         }
      }
   }
   
   
   //
   // Close the event file.
   //
   RBFile->Close();

   //
   // Delete the UnformPcFile object.
   // This will free up all memory allocated by the class.
   delete RBFile;


   //
   // Last we create and dump the S-file. We have all the information we need
   // from the startup information structure, except for the station name and 
   // the component code, which we will have to get from the channel thread.
   //
   //
   // Create the name of the S-file.
   LinePtr=SFileName;
   swprintf(str31, TEXT("%02d"), EventFileTime.wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert day.
   LinePtr=SFileName+2;
   *LinePtr='-';                                      // Insert minus sign.
   LinePtr=SFileName+3;
   swprintf(str31, TEXT("%02d"), EventFileTime.wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert hour.
   LinePtr=SFileName+5;
   swprintf(str31, TEXT("%02d"), EventFileTime.wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert minute.
   LinePtr=SFileName+7;
   *LinePtr='-';                                      // Insert minus sign.
   LinePtr=SFileName+8;
   swprintf(str31, TEXT("%02d"), EventFileTime.wSecond);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert seconds.
   LinePtr=SFileName+10;
   *LinePtr='L';                                      // Insert 'L'.
   LinePtr=SFileName+11;
   *LinePtr='.';                                      // Insert '.'.
   LinePtr=SFileName+12;
   *LinePtr='S';                                      // Insert 'S'.
   LinePtr=SFileName+13;
   swprintf(str31, TEXT("%4d"), EventFileTime.wYear);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 4*sizeof(TCHAR));       // Insert year.
   LinePtr=SFileName+17;
   swprintf(str31, TEXT("%02d"), EventFileTime.wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str31, 2*sizeof(TCHAR));       // Insert month.
   LinePtr=SFileName+19;
   *LinePtr=0;                                        // Terminate filename string.
  
   
   // Try to create a new file with this name.
   AddPathAndFilename(SFile, SI.SFileDir, SFileName);
   hFile=CreateFile(SFile, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile==INVALID_HANDLE_VALUE) {
      // The creation of the file failed.
      // A file with this name probably already exist.
      // Add a letter ranging from A-Z to the end of 
      // the file name and try creating the file again.
      Index=wcslen(SFileName);
      wcscat(SFileName, TEXT(" "));
      SFileName[Index]=('A'-1);
      do {
         SFileName[Index]++;
         AddPathAndFilename(SFile, SI.SFileDir, SFileName);
         hFile=CreateFile(SFile, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
      } while (hFile==INVALID_HANDLE_VALUE && SFileName[Index]!='Z');
   }
   if (hFile==INVALID_HANDLE_VALUE) {
      // Could not create the S-File. 
      // Write an error message in the log and exit.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Unable to create S-File '"));
      wcscat(Message, SFile);
      wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
      WriteSystemLog(Message, 3);
      ExitThread(1);    // Exit the thread.
   } else {
      CloseHandle(hFile);
   }
   
   
   // Open the S-file.
   hFile=CreateFile(SFile, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile==INVALID_HANDLE_VALUE) {
      // Could not open the S-File. 
      // Write an error message in the log and exit.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Unable to open S-File '"));
      wcscat(Message, SFile);
      wcscat(Message, TEXT("'. Thread will terminate, event not dumped."));   
      WriteSystemLog(Message, 3);
      ExitThread(1);    // Exit the thread.
   }





   //
   // Create and write type 1 line to S-file.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);
  
   // Insert year.
   _LinePtr=(PCHAR)(line+1);
   sprintf(_str31, "%4d", EventFileTime.wYear);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 4);
   
   // Insert month.
   _LinePtr=(PCHAR)(line+6);
   sprintf(_str31, "%2d", EventFileTime.wMonth);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);
   
   // Insert day.
   _LinePtr=(PCHAR)(line+8);
   sprintf(_str31, "%2d", EventFileTime.wDay);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);
   
   // Insert hour.
   _LinePtr=(PCHAR)(line+11);
   sprintf(_str31, "%2d", EventFileTime.wHour);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);
   
   // Insert minute.
   _LinePtr=(PCHAR)(line+13);
   sprintf(_str31, "%2d", EventFileTime.wMinute);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);
   
   // Insert seconds.
   _LinePtr=(PCHAR)(line+16);
   sprintf(_str31, "%4.1f", (float)(EventFileTime.wSecond+(EventFileTime.wMilliseconds/1000.0)));
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 4);
   
   // Insert distance indicator 'L'.
   _LinePtr=(PCHAR)(line+21);
   *_LinePtr='L';
   
    // Insert event id.
   _LinePtr=(PCHAR)(line+22);
   if (SI.ManualTrig) {
      *_LinePtr='M';
   } else {
      // Insert trigger set number.
      sprintf(_str31, "%1d", SI.TrgSetNumber);
      *_LinePtr=_str31[0];
   }
  
   // Insert network name.
   // Network name will be cut down to 3 characters.

   // UNICODE: Convert Network code to ANSI string
   CHAR _NetworkCode[6];  
   wcstombs(_NetworkCode, NetworkCode, 6);

   _LinePtr=(PCHAR)(line+45);
   CopyMemory((PVOID)_LinePtr, (PVOID)_NetworkCode, 3);
   
    // Mark line as type 1.
   _LinePtr=(PCHAR)(line+79);
   *_LinePtr='1';
   
   // Insert newline/carridge return.
   _LinePtr=(PCHAR)(line+80);
   *_LinePtr=13;
   _LinePtr=(PCHAR)(line+81);
   *_LinePtr=10;
   
   // Write line to file.
   WriteOK=TRUE;
   WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
   if (NumberOfBytesWritten!=82) WriteOK=FALSE;



   // 
   // Create and write type 6 line to S-file.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);
   
   // Write waveform filename.

   CHAR _EFileName[MAX_PATH+1];          // Convert UNICODE string 
   wcstombs(_EFileName, EFileName, MAX_PATH+1);   // to ANSI string

   _LinePtr=(PCHAR)(line+1);
   CopyMemory((PVOID)_LinePtr, (PVOID)_EFileName, (DWORD)strlen(_EFileName));
   
   // Mark line as type 6.
   _LinePtr=(PCHAR)(line+79);
   *_LinePtr='6';
   
   // Insert newline/carridge return.
   _LinePtr=(PCHAR)(line+80);
   *_LinePtr=13;
   _LinePtr=(PCHAR)(line+81);
   *_LinePtr=10;
   
   // Write line to file.
   WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
   if (NumberOfBytesWritten!=82) WriteOK=FALSE;

 
  // 
   // Create and write type I line to S-file.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);
   
   // Insert help text for action indicator and last action done.
   _LinePtr=(PCHAR)(line+1);
   CopyMemory((PVOID)_LinePtr, (PVOID)"ACTION:NEW", 10);
   
   // Insert Date and time of last action (current time).
   GetSystemTime(&CurrentTime);
   _LinePtr=(PCHAR)(line+12);
   sprintf(_str31, "%4d", CurrentTime.wYear);
   CopyMemory((PVOID)_LinePtr, (PVOID)(_str31+2), 2);   // Insert year.
   _LinePtr=(PCHAR)(line+14);
   *_LinePtr='-';                                      // Insert '-'.
   _LinePtr=(PCHAR)(line+15);
   sprintf(_str31, "%02d", CurrentTime.wMonth);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);       // Insert month.
   _LinePtr=(PCHAR)(line+17);
   *_LinePtr='-';                                      // Insert '-'.
   _LinePtr=(PCHAR)(line+18);
   sprintf(_str31, "%02d", CurrentTime.wDay);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);       // Insert day.
   _LinePtr=(PCHAR)(line+21);
   sprintf(_str31, "%02d", CurrentTime.wHour);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);       // Insert hour.
   _LinePtr=(PCHAR)(line+23);
   *_LinePtr=':';                                      // Insert ':'.
   _LinePtr=(PCHAR)(line+24);
   sprintf(_str31, "%02d", CurrentTime.wMinute);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);       // Insert minute.
   
   // Insert help text for operator and operator code.
   _LinePtr=(PCHAR)(line+27);
   CopyMemory((PVOID)_LinePtr, (PVOID)"OP:SEIS", 7);
   
   // Insert help text for status.
   _LinePtr=(PCHAR)(line+35);
   CopyMemory((PVOID)_LinePtr, (PVOID)"STATUS:", 7);
   
   // Insert help text for ID.
   _LinePtr=(PCHAR)(line+57);
   CopyMemory((PVOID)_LinePtr, (PVOID)"ID:", 3);
   
   // Insert ID.
   _LinePtr=(PCHAR)(line+60);
   sprintf(_str31, "%4d", EventFileTime.wYear);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 4);     // Insert year.
   _LinePtr=(PCHAR)(line+64);
   sprintf(_str31, "%02d", EventFileTime.wMonth);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);     // Insert month.
   _LinePtr=(PCHAR)(line+66);
   sprintf(_str31, "%02d", EventFileTime.wDay);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);     // Insert day.
   _LinePtr=(PCHAR)(line+68);
   sprintf(_str31, "%02d", EventFileTime.wHour);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);     // Insert hour.
   _LinePtr=(PCHAR)(line+70);
   sprintf(_str31, "%02d", EventFileTime.wMinute);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);     // Insert minute.
   _LinePtr=(PCHAR)(line+72);
   sprintf(_str31, "%02d", EventFileTime.wSecond);
   CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);     // Insert seconds.
   
   // Mark line as type I.
   _LinePtr=(PCHAR)(line+79);
   *_LinePtr='I';
   
   // Insert newline/carridge return.
   _LinePtr=(PCHAR)(line+80);
   *_LinePtr=13;
   _LinePtr=(PCHAR)(line+81);
   *_LinePtr=10;
   
   // Write line to file.
   WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
   if (NumberOfBytesWritten!=82) WriteOK=FALSE;
 

   // 
   // Create and write type 7 line to S-file.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);

   // Insert help lines and line code.
   _LinePtr=(PCHAR)(line+1);
   CopyMemory((PVOID)_LinePtr, (PVOID)"STAT SP IPHASW D HRMM SECON CODA AMPLIT PERI AZIMU VELO SNR AR TRES W  DIS CAZ7", 79);
   
   // Insert newline/carridge return.
   _LinePtr=(PCHAR)(line+80);
   *_LinePtr=13;
   _LinePtr=(PCHAR)(line+81);
   *_LinePtr=10;
   
   // Write line to file.
   WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
   if (NumberOfBytesWritten!=82) WriteOK=FALSE;

   // 
   // Create and write type 4 lines to S-file. One line for each channel.
   //
   for (Index=0;Index<SI.NumOfSFileChannels;Index++) {
      // Fill line with blanks.
      FillMemory((PVOID)line, 80, 32);

      // Insert station name.
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(SI.SChData[Index].ThreadID, TM_GET_STATION, (WPARAM)hEvent, (LPARAM)&MsgParams); 
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);
      _LinePtr=(PCHAR)(line+1);

	  CHAR _StationCode[6];                          // UNICODE 
	  wcstombs(_StationCode, MsgParams.cStatus, 6);  // Spesific

      CopyMemory((PVOID)_LinePtr, (PVOID)_StationCode, strlen(_StationCode));

      // Insert component.
      hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
      PostThreadMessage(SI.SChData[Index].ThreadID, TM_GET_COMPONENT, (WPARAM)hEvent, (LPARAM)&MsgParams);
      WaitForSingleObject(hEvent, INFINITE);
      CloseHandle(hEvent);
      ConvertComponentCode(MsgParams.cStatus);
      _LinePtr=(PCHAR)(line+6);

	  CHAR _ComponentCode[5];                          // UNICODE 
	  wcstombs(_ComponentCode, MsgParams.cStatus, 5);  // Spesific
     
	  CopyMemory((PVOID)_LinePtr, (PVOID)_ComponentCode, 2);

      // Insert quality indicator/phase-id.
      _LinePtr=(PCHAR)(line+9);
      CopyMemory((PVOID)_LinePtr, (PVOID)"IP", 2);

      // Insert hour/minute.
      _LinePtr=(PCHAR)(line+18);
      sprintf(_str31, "%02d", SI.SChData[Index].ChEventStartTime.wHour);
      CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);
      _LinePtr=(PCHAR)(line+20);
      sprintf(_str31, "%02d", SI.SChData[Index].ChEventStartTime.wMinute);
      CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 2);

      // Insert seconds.
      _LinePtr=(PCHAR)(line+23);
      sprintf(_str31, "%f", (double)(SI.SChData[Index].ChEventStartTime.wSecond+(SI.SChData[Index].ChEventStartTime.wMilliseconds/1000.0)));
      CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 5);

      // Insert Duration.
      _LinePtr=(PCHAR)(line+29);
      duration=TimeDiff(&SI.SChData[Index].ChEventStartTime, &SI.SChData[Index].ChEventStopTime); 
      sprintf(_str31, "%4d", (int)duration);
      CopyMemory((PVOID)_LinePtr, (PVOID)_str31, 4);
      
      // Insert newline/carridge return.
      _LinePtr=(PCHAR)(line+80);
      *_LinePtr=13;
      _LinePtr=(PCHAR)(line+81);
      *_LinePtr=10;

      // Write line to file.
      WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
      if (NumberOfBytesWritten!=82) WriteOK=FALSE;
   }

   // 
   // Create a last blank line and write to S-file.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);

   // Insert newline/carridge return.
   _LinePtr=(PCHAR)(line+80);
   *_LinePtr=13;
   _LinePtr=(PCHAR)(line+81);
   *_LinePtr=10;
   
   // Write line to file.
   WriteFile(hFile, (PVOID)line, 82, &NumberOfBytesWritten, NULL);
   if (NumberOfBytesWritten!=82) WriteOK=FALSE;
   
   //
   // Close the S-file
   //
   CloseHandle(hFile);

   // Check if S-File was successfully written.
   if (!WriteOK) {
      // Write message in the log.
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": An error occured while writing to S-File '"));
      wcscat(Message, SFile);
      wcscat(Message, TEXT("'. Thread will terminate, S-File not written.")); 
      WriteSystemLog(Message, 3);
      DeleteFile(SFile);
   } else {
      if ( SI.ManualTrig) WriteSystemLog(TEXT("Seislog: Manual event dumped."), 1);
      if (!SI.ManualTrig) WriteSystemLog(TEXT("Seislog: Event dumped."), 1);
   }

   
   //
   // Unregister this process from the process database.
   //
   UnregisterProcess(CurrentThreadID);
   
   
   //
   // If running in debug mode, write a message to the log.
   //
   if (DebugLevel) {
      wcscpy(Message, ProcName);
      wcscat(Message, TEXT(": Thread stopped."));
      WriteSystemLog(Message, 1);
   }
   
   
   //
   // Thread dies here.
   //
   ExitThread(0);
   return 0;
   
}






/*********************************************************************************************/
/* Writes the event file header to an event file.                                            */ 
/*********************************************************************************************/
BOOL WriteEventFileHeader(UnformPcFile *RBFile, PTCHAR NetworkName, BYTE NumDFileChannels, 
                          BYTE NumStations, SYSTEMTIME *EventFileTime, double TotalTimeWindow,
                          CHNINFO ChInfo[])
{
   BYTE line[81], *LinePtr, Index, PosInLine, LinesWritten;
   CHAR str20[21];
   DWORD Bytes;
   BOOL WriteStatus, DataInLine;


   //
   // Create and write the first header line.
   //
   WriteStatus=TRUE;
   
   // Fill line with blanks.
   FillMemory((PVOID)line, 80, 32);
   
   // Insert network name.
   // OJ: The UNICODE string is first converted to a ANSI string
   CHAR _NetworkName[30];
   wcstombs(_NetworkName, NetworkName, 30);

   LinePtr=line+1;
   CopyMemory((PVOID)LinePtr, (PVOID)_NetworkName, strlen(_NetworkName));
   
   // Insert number of stations.
   LinePtr=line+30;
   sprintf(str20, "%2d", NumStations);
   CopyMemory((PVOID)LinePtr, (PVOID)(str20), 2);
   
   // Insert year.   
   sprintf(str20, "%03d", EventFileTime->wYear-1900);
   LinePtr=line+33;
   CopyMemory((PVOID)LinePtr, (PVOID)(str20), 3);   

   // Insert day of year.
   LinePtr=line+37;
   sprintf(str20, "%3d", GetDayOfYear(EventFileTime));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 3);
   
   // Insert month.
   LinePtr=line+41;
   sprintf(str20, "%2d", EventFileTime->wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert day.
   LinePtr=line+44;
   sprintf(str20, "%2d", EventFileTime->wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert hour.
   LinePtr=line+47;
   sprintf(str20, "%2d", EventFileTime->wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert minute.
   LinePtr=line+50;
   sprintf(str20, "%2d", EventFileTime->wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert seconds.
   LinePtr=line+53;
   sprintf(str20, "%6.3f", (float)(EventFileTime->wSecond+(EventFileTime->wMilliseconds/1000.0)));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);
   
   // Insert total time window.
   LinePtr=line+60;
   sprintf(str20, "%9.3f", TotalTimeWindow);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 9);
   
   // Write the first header line.
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;
   
   //
   // Create and write the second header line. This line should be empty.
   //
   FillMemory((PVOID)line, 80, 32);
   if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;
   
   
   //
   // Create and write header lines 3-XX where XX depends on the number of channels.
   // One line hold data for up to 3 channels (stations).
   //
   PosInLine=1;
   LinesWritten=0;
   DataInLine=FALSE;
   for (Index=0;Index<NumDFileChannels;Index++) {
      if (!ChInfo[Index].SkipChannel) {
         // Fill line with blanks.
         if (PosInLine==1) FillMemory((PVOID)line, 80, 32);

	     // OJ: The UNICODE string is first converted to a ANSI string
	     CHAR _StationCode[6];
	     wcstombs(_StationCode, ChInfo[Index].StationCode, 6);

         // Insert station code.
         if (strlen(_StationCode)<5) {           //OJ: Changed
            if (PosInLine==1) LinePtr=line+1;
            if (PosInLine==2) LinePtr=line+27;
            if (PosInLine==3) LinePtr=line+53;
            CopyMemory((PVOID)LinePtr, (PVOID)(_StationCode), strlen(_StationCode)); //OJ: Changed
         } else {
            if (PosInLine==1) LinePtr=line+1;
            if (PosInLine==2) LinePtr=line+27;
            if (PosInLine==3) LinePtr=line+53;
            CopyMemory((PVOID)LinePtr, (PVOID)(_StationCode), 4); //OJ: Changed
            if (PosInLine==1) LinePtr=line+9;
            if (PosInLine==2) LinePtr=line+35;
            if (PosInLine==3) LinePtr=line+61;
            CopyMemory((PVOID)LinePtr, (PVOID)((_StationCode)+4), 1);
         }

		 // OJ: The UNICODE string is first converted to a ANSI string
	     CHAR _Component[5];
	     wcstombs(_Component, ChInfo[Index].Component, 5);
		 

         // Insert Component.
         if (PosInLine==1) LinePtr=line+5;
         if (PosInLine==2) LinePtr=line+31;
         if (PosInLine==3) LinePtr=line+57;
         CopyMemory((PVOID)LinePtr, (PVOID)(_Component), strlen(_Component));
      
         // Insert start time relative to eventfile start time.
         if (PosInLine==1) LinePtr=line+10;
         if (PosInLine==2) LinePtr=line+36;
         if (PosInLine==3) LinePtr=line+62;
         sprintf(str20, "%7.2f", ChInfo[Index].RelStartTime);
         CopyMemory((PVOID)LinePtr, (PVOID)str20, 7);
      
         // Insert station data interval length.
         if (PosInLine==1) LinePtr=line+18;
         if (PosInLine==2) LinePtr=line+44;
         if (PosInLine==3) LinePtr=line+70;
         sprintf(str20, "%8.2f", ChInfo[Index].DataIntLength);
         CopyMemory((PVOID)LinePtr, (PVOID)str20, 8);

         // This line is not empty.
         DataInLine=TRUE; 
         PosInLine++;

         // Write line if needed.
         if (PosInLine==4) {
            // The current line is full. Write to disk an start new line.
            PosInLine=1;
            LinesWritten++;
            DataInLine=FALSE; 
            if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;
         }
      }
   }

   // If there are data left in the line then write them to disk.
   if (DataInLine) {
      LinesWritten++;
      if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;
   }
   
   
   //
   // The fileformat requires that there must be at least 12 lines in
   // the event file header, so we fill in with blank lines as needed.
   //
   if (LinesWritten+2<12) {
      FillMemory((PVOID)line, 80, 32);
      for (Index=LinesWritten+3;Index<=12;Index++) {
         if (!RBFile->Write(80, 80, line, &Bytes)) WriteStatus=FALSE;
      }
   }
   
   return WriteStatus;
}





/***********************************************************************************/
/* Writes the event file channel header to an event file.                          */ 
/***********************************************************************************/
BOOL WriteChannelHeader(UnformPcFile *RBFile, BYTE TblIndex, CHNINFO ChInfo[])
{
   BYTE line[HEADER_LINE_SIZE], *LinePtr;
   CHAR str20[21];
   DWORD Bytes;
   BOOL WriteStatus = TRUE;
   SYSTEMTIME BlockTime;
   

   //
   // Write channel header. This line is 1040 bytes long.
   //
   // Fill line with blanks.
   FillMemory((PVOID)line, HEADER_LINE_SIZE, 32);

   // Insert station code.
   LinePtr=line;
   wcstombs(str20, ChInfo[TblIndex].StationCode, 21);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, strlen(str20));
   
   // Insert Component.
   LinePtr=line+5;
   wcstombs(str20, ChInfo[TblIndex].Component, 21);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, strlen(str20));
   
   // Get the time of the first block.
   BlockTime=ChInfo[TblIndex].StartTimeFrstBlck;
   
   // Insert year.
   LinePtr=line+9;
   sprintf(str20, "%03d", BlockTime.wYear-1900);
   CopyMemory((PVOID)LinePtr, (PVOID)(str20), 3);
   
   // Insert day of year.
   LinePtr=line+13;
   sprintf(str20, "%3d", GetDayOfYear(&BlockTime));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 3);
   
   // Insert month.
   LinePtr=line+17;
   sprintf(str20, "%2d", BlockTime.wMonth);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert day.
   LinePtr=line+20;
   sprintf(str20, "%2d", BlockTime.wDay);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert hour.
   LinePtr=line+23;
   sprintf(str20, "%2d", BlockTime.wHour);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert minute.
   LinePtr=line+26;
   sprintf(str20, "%2d", BlockTime.wMinute);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 2);
   
   // Insert seconds.
   LinePtr=line+29;
   sprintf(str20, "%6.3f", (float)(BlockTime.wSecond+(BlockTime.wMilliseconds/1000)));
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);
   
   // Insert Sample rate.
   LinePtr=line+36;
   sprintf(str20, "%7.2f", (float)ChInfo[TblIndex].BufferProperties.SampleRate);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 7);
   
   // Insert number of samples.
   LinePtr=line+44;
   sprintf(str20, "%6d", ChInfo[TblIndex].NumSamplesInEvent);
   CopyMemory((PVOID)LinePtr, (PVOID)str20, 6);
   
   // Insert sample size.
   line[76]='4';
   
   // Write channel header line.
   if (!RBFile->Write(HEADER_LINE_SIZE, 128, line, &Bytes)) WriteStatus=FALSE;
   if (!RBFile->WriteLeftovers(&Bytes)) WriteStatus=FALSE;
   
   return WriteStatus;
}





/***********************************************************************************/
/* Converts from 4 TTCHAR component code to 2 char component code for use in S-files.*/
/* NOTE:Changes made to this function, should also be                              */
/*      made to the 'IsValidCompCode' function.                                    */
/***********************************************************************************/
BOOL ConvertComponentCode(PTCHAR CompCode)
{
   // Return if component code is
   // shorter then 4 characters.
   if (wcslen(CompCode)<4) return FALSE;
   
   if (!wcscmp(CompCode, TEXT("S  Z"))) wcscpy(CompCode, TEXT("SZ"));
   if (!wcscmp(CompCode, TEXT("S  N"))) wcscpy(CompCode, TEXT("SN"));
   if (!wcscmp(CompCode, TEXT("S  E"))) wcscpy(CompCode, TEXT("SE"));
   if (!wcscmp(CompCode, TEXT("L  Z"))) wcscpy(CompCode, TEXT("LZ"));
   if (!wcscmp(CompCode, TEXT("L  N"))) wcscpy(CompCode, TEXT("LN"));
   if (!wcscmp(CompCode, TEXT("L  E"))) wcscpy(CompCode, TEXT("LE"));
   if (!wcscmp(CompCode, TEXT("AL Z"))) wcscpy(CompCode, TEXT("AZ"));
   if (!wcscmp(CompCode, TEXT("AL N"))) wcscpy(CompCode, TEXT("AN"));
   if (!wcscmp(CompCode, TEXT("AL E"))) wcscpy(CompCode, TEXT("AE"));
   if (!wcscmp(CompCode, TEXT("AH Z"))) wcscpy(CompCode, TEXT("AZ"));
   if (!wcscmp(CompCode, TEXT("AH N"))) wcscpy(CompCode, TEXT("AN"));
   if (!wcscmp(CompCode, TEXT("AH E"))) wcscpy(CompCode, TEXT("AE"));
   if (!wcscmp(CompCode, TEXT("A  Z"))) wcscpy(CompCode, TEXT("AZ"));
   if (!wcscmp(CompCode, TEXT("A  N"))) wcscpy(CompCode, TEXT("AN"));
   if (!wcscmp(CompCode, TEXT("A  E"))) wcscpy(CompCode, TEXT("AE"));
   if (!wcscmp(CompCode, TEXT("BV Z"))) wcscpy(CompCode, TEXT("BZ"));
   if (!wcscmp(CompCode, TEXT("BV N"))) wcscpy(CompCode, TEXT("BN"));
   if (!wcscmp(CompCode, TEXT("BV E"))) wcscpy(CompCode, TEXT("BE"));
   if (!wcscmp(CompCode, TEXT("BH Z"))) wcscpy(CompCode, TEXT("BZ"));
   if (!wcscmp(CompCode, TEXT("BH N"))) wcscpy(CompCode, TEXT("BN"));
   if (!wcscmp(CompCode, TEXT("BH E"))) wcscpy(CompCode, TEXT("BE"));
   if (!wcscmp(CompCode, TEXT("B  Z"))) wcscpy(CompCode, TEXT("BZ"));
   if (!wcscmp(CompCode, TEXT("B  N"))) wcscpy(CompCode, TEXT("BN"));
   if (!wcscmp(CompCode, TEXT("B  E"))) wcscpy(CompCode, TEXT("BE"));
   if (!wcscmp(CompCode, TEXT("SV Z"))) wcscpy(CompCode, TEXT("SZ"));
   if (!wcscmp(CompCode, TEXT("SV N"))) wcscpy(CompCode, TEXT("SN"));
   if (!wcscmp(CompCode, TEXT("SV E"))) wcscpy(CompCode, TEXT("SE"));
   if (!wcscmp(CompCode, TEXT("SH Z"))) wcscpy(CompCode, TEXT("SZ"));
   if (!wcscmp(CompCode, TEXT("SH N"))) wcscpy(CompCode, TEXT("SN"));
   if (!wcscmp(CompCode, TEXT("SH E"))) wcscpy(CompCode, TEXT("SE"));
   if (!wcscmp(CompCode, TEXT("LV Z"))) wcscpy(CompCode, TEXT("VZ"));
   if (!wcscmp(CompCode, TEXT("LV N"))) wcscpy(CompCode, TEXT("VN"));
   if (!wcscmp(CompCode, TEXT("LV E"))) wcscpy(CompCode, TEXT("VE"));

   if (!wcscmp(CompCode, TEXT("S  R"))) wcscpy(CompCode, TEXT("SR"));
   if (!wcscmp(CompCode, TEXT("S  T"))) wcscpy(CompCode, TEXT("ST"));
   if (!wcscmp(CompCode, TEXT("L  R"))) wcscpy(CompCode, TEXT("LR"));
   if (!wcscmp(CompCode, TEXT("L  T"))) wcscpy(CompCode, TEXT("LT"));
   if (!wcscmp(CompCode, TEXT("AL R"))) wcscpy(CompCode, TEXT("AR"));
   if (!wcscmp(CompCode, TEXT("AL T"))) wcscpy(CompCode, TEXT("AT"));
   if (!wcscmp(CompCode, TEXT("AH R"))) wcscpy(CompCode, TEXT("AR"));
   if (!wcscmp(CompCode, TEXT("AH T"))) wcscpy(CompCode, TEXT("AT"));
   if (!wcscmp(CompCode, TEXT("A  R"))) wcscpy(CompCode, TEXT("AR"));
   if (!wcscmp(CompCode, TEXT("A  T"))) wcscpy(CompCode, TEXT("AT"));
   if (!wcscmp(CompCode, TEXT("BV R"))) wcscpy(CompCode, TEXT("BR"));
   if (!wcscmp(CompCode, TEXT("BV T"))) wcscpy(CompCode, TEXT("BT"));
   if (!wcscmp(CompCode, TEXT("SV R"))) wcscpy(CompCode, TEXT("SR"));
   if (!wcscmp(CompCode, TEXT("SV T"))) wcscpy(CompCode, TEXT("ST"));

   if (wcslen(CompCode)>2) return FALSE;
   return TRUE;
}






